/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CAccountTemplatesFile.cpp

  Description :	

  Last Modified $Date: 2006/12/21 06:24:08 $

  $Revision: 1.4 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CAccountTemplatesFile.h"
#include "tinyxml.h"
#include "CFrameWork.h"
#include "CFileSystem.h"
#include <stdio.h>
#include "CAssert.h"

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************
static const CString	s_szConfigFile( "Data/ACCOUNT_TEMPLATES.XML" );

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************
CAccountTemplatesFile::CAccountTemplateList	*CAccountTemplatesFile::s_pAccountTemplates;
TiXmlDocument 			*CAccountTemplatesFile::m_pDocument;

//******************************************************************************
//   Class Definition
//******************************************************************************

//******************************************************************************
//	
//******************************************************************************
bool	CAccountTemplatesFile::Open()
{
	// Create the xml document
	m_pDocument = new TiXmlDocument();
	
	if (m_pDocument == NULL)
	{
		ASSERT(0, "Failed to create the XML document");
		return false;
	}
	
	// Load the xml file
	if (m_pDocument->LoadFile(s_szConfigFile) == false)
	{
		ASSERT(0, "Failed to load the account templates file");
		return false;
	}
	
	// Create the list to hold the parsed templates
	s_pAccountTemplates = new CAccountTemplateList();
	
	// Start parsing the templates
	TiXmlHandle doc_handle( m_pDocument );
	TiXmlElement *p_templates(doc_handle.FirstChild("account_templates").Element());
	
	if (p_templates != NULL)
	{
		TiXmlElement *p_template(p_templates->FirstChildElement("account_template"));
		
		while (p_template != NULL)
		{
			// Template name
			CAccountTemplate *p_accountTemplate = new CAccountTemplate();
			TiXmlElement *p_templateName(p_template->FirstChildElement("name"));
			if (p_templateName != NULL) {
				p_accountTemplate->template_name = p_templateName->GetText();
			} else {
				p_accountTemplate->template_name = "Unnamed template";
			}
			
			// Params
			TiXmlElement *p_params(p_template->FirstChildElement("params"));
			
			if (p_params != NULL)
			{
				TiXmlElement *p_param(p_params->FirstChildElement("param"));
				
				while (p_param != NULL)
				{
					CAccountTemplateParam *p_accountParam = new CAccountTemplateParam();
					TiXmlElement *p_paramName(p_param->FirstChildElement("name"));
					if (p_paramName != NULL) {
						p_accountParam->name = p_paramName->GetText();
					} else {
						p_accountParam->name = CString("");
					}
					
					TiXmlElement *p_paramPrompt(p_param->FirstChildElement("prompt"));
					if (p_paramPrompt != NULL) {
						p_accountParam->prompt = p_paramPrompt->GetText();
					} else {
						p_accountParam->prompt = CString("");
					}
					
					TiXmlElement *p_paramValue(p_param->FirstChildElement("value"));
					if (p_paramValue != NULL) {
						p_accountParam->value = p_paramValue->GetText();
					} else {
						p_accountParam->value = CString("");
					}
					
					p_accountTemplate->p_paramList->push_back(p_accountParam);
					p_param = p_param->NextSiblingElement();
				}
			}
			
			// Values
			TiXmlElement *p_values(p_template->FirstChildElement("values"));
			
			if (p_values != NULL)
			{
				// Display name
				TiXmlElement *p_displayName(p_values->FirstChildElement("display_name"));
				if (p_displayName != NULL) {
					p_accountTemplate->display_name = p_displayName->GetText();
				} else {
					p_accountTemplate->display_name = CString("");
				}
				
				// Email address
				TiXmlElement *p_emailAddress(p_values->FirstChildElement("email_address"));
				if (p_emailAddress != NULL) {
					p_accountTemplate->email_address = p_emailAddress->GetText();
				} else {
					p_accountTemplate->email_address = CString("");
				}
				
				// Account type
				TiXmlElement *p_accountType(p_values->FirstChildElement("account_type"));
				if (p_accountType != NULL) {
					p_accountTemplate->account_type = p_accountType->GetText();
				} else {
					p_accountTemplate->account_type = CString("");
				}
				
				// Incoming server
				TiXmlElement *p_incomingServer(p_values->FirstChildElement("incoming_server"));
				if (p_incomingServer != NULL) {
					p_accountTemplate->incoming_server = p_incomingServer->GetText();
				} else {
					p_accountTemplate->incoming_server = CString("");
				}
				
				// Incoming port
				TiXmlElement *p_incomingPort(p_values->FirstChildElement("incoming_server_port"));
				if (p_incomingPort != NULL) {
					p_accountTemplate->incoming_server_port = p_incomingPort->GetText();
				} else {
					p_accountTemplate->incoming_server_port = CString("110");
				}
				
				// Incoming username
				TiXmlElement *p_incomingUsername(p_values->FirstChildElement("incoming_server_username"));
				if (p_incomingUsername != NULL) {
					p_accountTemplate->incoming_server_username = p_incomingUsername->GetText();
				} else {
					p_accountTemplate->incoming_server_username = CString("");
				}
				
				// Incoming password
				TiXmlElement *p_incomingPassword(p_values->FirstChildElement("incoming_server_password"));
				if (p_incomingPassword != NULL) {
					p_accountTemplate->incoming_server_password = p_incomingPassword->GetText();
				} else {
					p_accountTemplate->incoming_server_password = CString("");
				}
				
				// Incoming tls
				TiXmlElement *p_incomingTls(p_values->FirstChildElement("incoming_server_tls"));
				if (p_incomingTls != NULL) {
					p_accountTemplate->incoming_server_tls = p_incomingTls->GetText();
				} else {
					p_accountTemplate->incoming_server_tls = CString("false");
				}
				
				// Outgoing server
				TiXmlElement *p_outgoingServer(p_values->FirstChildElement("outgoing_server"));
				if (p_outgoingServer != NULL) {
					p_accountTemplate->outgoing_server = p_outgoingServer->GetText();
				} else {
					p_accountTemplate->outgoing_server = CString("");
				}
				
				// Outgoing port
				TiXmlElement *p_outgoingPort(p_values->FirstChildElement("outgoing_server_port"));
				if (p_outgoingPort != NULL) {
					p_accountTemplate->outgoing_server_port = p_outgoingPort->GetText();
				} else {
					p_accountTemplate->outgoing_server_port = CString("25");
				}
				
				// Outgoing username
				TiXmlElement *p_outgoingUsername(p_values->FirstChildElement("outgoing_server_username"));
				if (p_outgoingUsername != NULL) {
					p_accountTemplate->outgoing_server_username = p_outgoingUsername->GetText();
				} else {
					p_accountTemplate->outgoing_server_username = CString("");
				}
				
				// Outgoing password
				TiXmlElement *p_outgoingPassword(p_values->FirstChildElement("outgoing_server_password"));
				if (p_outgoingPassword != NULL) {
					p_accountTemplate->outgoing_server_password = p_outgoingPassword->GetText();
				} else {
					p_accountTemplate->outgoing_server_password = CString("");
				}
				
				// Outgoing tls
				TiXmlElement *p_outgoingTls(p_values->FirstChildElement("outgoing_server_tls"));
				if (p_outgoingTls != NULL) {
					p_accountTemplate->outgoing_server_tls = p_outgoingTls->GetText();
				} else {
					p_accountTemplate->outgoing_server_tls = CString("false");
				}
				
				// Leave messages on server
				TiXmlElement *p_leaveMessages(p_values->FirstChildElement("leave_messages_on_server"));
				if (p_leaveMessages != NULL) {
					p_accountTemplate->leave_messages_on_server = p_leaveMessages->GetText();
				} else {
					p_accountTemplate->leave_messages_on_server = CString("true");
				}
				
				// Reply to
				TiXmlElement *p_replyTo(p_values->FirstChildElement("reply_to"));
				if (p_replyTo != NULL) {
					p_accountTemplate->reply_to = p_replyTo->GetText();
				} else {
					p_accountTemplate->reply_to = CString("");
				}
			}
			else
			{
				p_accountTemplate->display_name = CString("");
				p_accountTemplate->email_address = CString("");
				p_accountTemplate->account_type = CString("");
				p_accountTemplate->incoming_server = CString("");
				p_accountTemplate->incoming_server_port = CString("110");
				p_accountTemplate->incoming_server_username = CString("");
				p_accountTemplate->incoming_server_password = CString("");
				p_accountTemplate->incoming_server_tls = CString("false");
				p_accountTemplate->outgoing_server = CString("");
				p_accountTemplate->outgoing_server_port = CString("25");
				p_accountTemplate->outgoing_server_username = CString("");
				p_accountTemplate->outgoing_server_password = CString("");
				p_accountTemplate->outgoing_server_tls = CString("false");
				p_accountTemplate->leave_messages_on_server = CString("true");
				p_accountTemplate->reply_to = CString("");
			}
			
			s_pAccountTemplates->push_back(p_accountTemplate);
			p_template = p_template->NextSiblingElement();
		}
	}
	
	return true;
}

//******************************************************************************
//	
//******************************************************************************
void	CAccountTemplatesFile::Close()
{
	if (s_pAccountTemplates != NULL) {
		CAccountTemplateList::iterator it = s_pAccountTemplates->begin();
		while(it !=  s_pAccountTemplates->end()) {
			SAFE_DELETE((*it));
			++it;
		}
		SAFE_DELETE(s_pAccountTemplates);
	}
}

//******************************************************************************
//	
//******************************************************************************
CAccountTemplate::CAccountTemplate(CAccountTemplate* account)
{
	p_paramList = new CAccountTemplateParamList();
	
	display_name = account->display_name;
	email_address = account->email_address;
	account_type = account->account_type;
	incoming_server = account->incoming_server;
	incoming_server_port = account->incoming_server_port;
	incoming_server_username = account->incoming_server_username;
	incoming_server_password = account->incoming_server_password;
	incoming_server_tls = account->incoming_server_tls;
	outgoing_server = account->outgoing_server;
	outgoing_server_port = account->outgoing_server_port;
	outgoing_server_username = account->outgoing_server_username;
	outgoing_server_password = account->outgoing_server_password;
	outgoing_server_tls = account->outgoing_server_tls;
	leave_messages_on_server = account->leave_messages_on_server;
	reply_to = account->reply_to;
	CAccountTemplateParamList::iterator it = account->p_paramList->begin();
	while (it != account->p_paramList->end()) {
			p_paramList->push_back(new CAccountTemplateParam(*it));
			it++;
	}
}

CAccountTemplate::~CAccountTemplate()
{
	if (p_paramList != NULL) {
		CAccountTemplateParamList::iterator it = p_paramList->begin();
		while(it !=  p_paramList->end()) {
			SAFE_DELETE((*it));
			++it;
		}
		SAFE_DELETE(p_paramList);
	}
}

//******************************************************************************
//	
//******************************************************************************
CString CAccountTemplate::replaceWithParams(CString str) {
	CString returnString = CString(str.GetPtr());
	
	CAccountTemplateParamList::iterator it = p_paramList->begin();
	while (it != p_paramList->end()) 
	{
		CString replaceString = CString("");
		replaceString.Printf("${%s}", ((*it)->name).GetPtr());
		
		int paramPosition = returnString.Find(replaceString);
		if (paramPosition != -1)
		{
			CString firstChunk = CString("");
			if (paramPosition != 0) 
			{
				firstChunk = returnString.Substring(0, paramPosition);
			}
			CString secondChunk = CString("");
			if (paramPosition + replaceString.Length() != returnString.Length()) 
			{
				secondChunk = returnString.Substring(paramPosition + replaceString.Length(), returnString.Length() - paramPosition - replaceString.Length());	
			}
			returnString.Printf("%s%s%s", firstChunk.GetPtr(), ((*it)->value).GetPtr(), secondChunk.GetPtr());
		}
		++it;
	}
	
	return returnString;
}
//*******************************  END OF FILE  ********************************
