/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CBenTestWindow.cpp

  Description :	

  Last Modified $Date: 2007/01/01 21:55:46 $

  $Revision: 1.2 $

***********************************************************************************/

//**********************************************************************************
//   Include Files
//**********************************************************************************
#include "CBenTestWindow.h"
#include "CFrameWork.h"
#include "CTextureManager.h"
#include "CFont.h"
#include "CInput.h"
#include "CSkinManager.h"
#include "CGfx.h"
#include <psprtc.h>
#include "CWindow.h"
#include "CPSPoste.h"
#include "CConnectionManager.h"
#include "CHUHelp.h"
#include "CSmtp.h"
#include "CPop3.h"
#include "CConfigFile.h"

//**********************************************************************************
//   Local Macros
//**********************************************************************************

//**********************************************************************************
//   Local Constants
//**********************************************************************************
#define OUT1_X 5
#define OUT1_Y 5
#define OUT2_X 5
#define OUT2_Y 5 + 13
#define MONITOR1_X CGfx::s_ScreenWidth/2 + 100
#define MONITOR1_Y 5
#define MONITOR2_X CGfx::s_ScreenWidth/2 + 100
#define MONITOR2_Y 5 + 13

//**********************************************************************************
//   Static Prototypes
//**********************************************************************************

//**********************************************************************************
//   Global Variables
//**********************************************************************************

//**********************************************************************************
//   Static Variables
//**********************************************************************************

//**********************************************************************************
//   Class Definition
//**********************************************************************************


//**********************************************************************************
//**********************************************************************************
//
//	Main Window implementation
//
//**********************************************************************************
//**********************************************************************************

//**********************************************************************************
//	
//**********************************************************************************
CBenTestWindow::CBenTestWindow()
{
	resetTest();
	m_activeTest = EBenTest_ConnMan;
}

//**********************************************************************************
//	
//**********************************************************************************
CBenTestWindow::~CBenTestWindow()
{

}

//**********************************************************************************
//	
//**********************************************************************************
void	CBenTestWindow::Render()
{
	// Draw white background
	CGfx::DrawQuad( V2( 0, 0 ), V2( CGfx::s_ScreenWidth, CGfx::s_ScreenHeight), ARGB(255, 255, 255, 255));

	CFont::GetActiveUiFont()->Print(m_output1, V2(OUT1_X, OUT1_Y), ARGB(0, 0, 0));
	CFont::GetActiveUiFont()->Print(m_output2, V2(OUT2_X, OUT2_Y), ARGB(0, 0, 0));
	CFont::GetActiveUiFont()->Print(m_monitor1, V2(MONITOR1_X, MONITOR1_Y), ARGB(0, 0, 0));
	CFont::GetActiveUiFont()->Print(m_monitor2, V2(MONITOR2_X, MONITOR2_Y), ARGB(0, 0, 0));
}

//**********************************************************************************
//	
//**********************************************************************************
void	CBenTestWindow::Process()
{
	CHUHelp::Clear();
	CWindow::Process();
	
	switch (m_activeTest)
	{
		case EBenTest_ConnMan:
			connManTest();
			break;
		default:
			break;
	}
}

//**********************************************************************************
//	
//**********************************************************************************
bool	CBenTestWindow::ProcessInput()
{
	if ( CInput::IsButtonClicked( CInput::CROSS ) == true ) {
		CPSPoste::s_NewWindow = CWindow::WINDOW_MAIN;
	}

	return false;
}

//**********************************************************************************
//	
//**********************************************************************************
void	CBenTestWindow::resetTest()
{
	m_activeTest = EBenTest_None;
	m_output1 = "";
	m_output2 = "";
	m_monitor1 = "";
	m_monitor2 = "";
	m_waitForInput = false;
	m_testIdx = 0;
	m_subTestIdx = 0;
	m_delay = 0;
}

//**********************************************************************************
//	
//**********************************************************************************
void	CBenTestWindow::connManTest()
{
	// The "monitor" should always be updated because the connection manager is
	// asynchronous
	if (m_testIdx > 0)
	{
		m_monitor1.Printf("Error: 0x%08x", CConnectionManager::CheckError());
		
		switch (CConnectionManager::GetState())
		{
			case EConnManState_Connected:
				m_monitor2.Printf("State: Connected");
				break;
			case EConnManState_Connecting:
				m_monitor2.Printf("State: Connecting");
				break;
			case EConnManState_Disconnected:
				m_monitor2.Printf("State: Disconnected");
				break;
			default:
				m_monitor2.Printf("State: Invalid");
				break;
		}
	}
		
	// If we have a delay set, we just decrement it and return
	if (m_delay)
	{
		m_delay--;
		return;
	}
	
	// If we're waiting for input, we just return if cross hasn't been pressed
	if (m_waitForInput)
	{
		if (CInput::IsButtonReleased(CInput::CIRCLE))
			m_waitForInput = false;
		else
			return;
	}
	
	// Now for the test processing
	switch (m_testIdx)
	{
		case 0: // Just output info about the test
			m_output1 = "Press O to begin the test";
			m_waitForInput = true;
			m_testIdx++;
			break;
			
		case 1: // 1.1.2
			m_output1 = "Connecting with an invalid connection [20]";
			m_output2.Printf("Press O when the error is 0x%08x", CON_MAN_ERR_CONNECT);
			CConnectionManager::RequestConnect(20);
			m_waitForInput = true;
			m_testIdx++;
			break;
			
		case 2: // 1.1.3
			m_output1 = "Connecting with connection configured for another AP [3]";
			m_output2.Printf("Press O when the error is 0x%08x", CON_MAN_ERR_AP);
			CConnectionManager::RequestConnect(3);
			m_waitForInput = true;
			m_testIdx++;
			break;
			
		case 3: // 1.1.1
			m_output1 = "Connecting with a valid connection [1]";
			m_output2 = "Press O when the state is 'Conntected'";
			CConnectionManager::RequestConnect(1);
			m_waitForInput = true;
			m_testIdx++;
			break;
			
		case 4: // 1.1.4
			m_output1 = "Connecting when already connected";
			m_output2.Printf("Press O when the error is 0x%08x", CON_MAN_ERR_ALREADY_CONNECTED);
			CConnectionManager::RequestConnect(1);
			m_waitForInput = true;
			m_testIdx++;
			break;
			
		case 5: // 1.2.1
			m_output1 = "Disconnecting when connected";
			m_output2 = "Press O when the state is 'Disconnected'";
			CConnectionManager::RequestDisconnect();
			m_waitForInput = true;
			m_testIdx++;
			break;
			
		case 6: // 1.2.2
			m_output1 = "Disconnecting when disconnected";
			m_output2 = "Press O when the state is 'Disconnected'";
			CConnectionManager::RequestDisconnect();
			m_waitForInput = true;
			m_testIdx++;
			break;
			
		case 7: // 1.2.3
			if (m_subTestIdx == 0)
			{
				m_output1 = "Disconnecting while connecting";
				m_output2 = "Press O when state is 'Disconnected'";
				CConnectionManager::RequestConnect(1);
				m_delay = 90;
				m_subTestIdx++;
			}
			else
			{
				CConnectionManager::RequestDisconnect();
				m_waitForInput = true;
				m_testIdx++;
				m_subTestIdx = 0;
			}
			
			break;
			
		case 8:
			resetTest();
			break;
			
		default:
			break;
	}
}

//*******************************  END OF FILE  ************************************
