#include "CCertRequestHandler.h"
#include "CDialogCertRequest.h"
#include "CCertRequest.h"
#include "CCertificateManager.h"

CCertRequestHandler::CCertRequestHandler(vmime::ref<CCertificateManager> aCertManager) :
	m_certManager(aCertManager)
{
	
}

CCertRequestHandler::~CCertRequestHandler()
{
	while (m_requestList.empty() == false)
	{
		CDialogCertRequest* requestDlg(m_requestList.front());
		requestDlg->getCertRequest()->decline();
		delete requestDlg;
	}
}

void CCertRequestHandler::incomingCertRequest(CCertRequest* aCertRequest)
{
	m_requestList.push_back(new CDialogCertRequest(aCertRequest));
}

void CCertRequestHandler::processCertRequests()
{
	TCertDialogList clearList;

	// Go over all certificate request dialogs and see if any have responded
	for (TCertDialogList::iterator it = m_requestList.begin(); it != m_requestList.end(); ++it)
	{
		CDialogCertRequest* certDlg = (*it);
		switch (certDlg->getExitValue())
		{
			case -1:
				break;
			case DIALOG_BUTTON_ACCEPT:
				certDlg->getCertRequest()->accept(m_certManager);
				clearList.push_back(certDlg);
				break;
			case DIALOG_BUTTON_DECLINE:
				certDlg->getCertRequest()->decline();
				clearList.push_back(certDlg);
				break;
			case DIALOG_BUTTON_ACCEPT_ONCE:
				certDlg->getCertRequest()->acceptForSession(m_certManager);
				clearList.push_back(certDlg);
				break;
			default:
				break;
		}
	}
	
	// Clean up the dialogs that have responded
	for (TCertDialogList::iterator it = clearList.begin(); it != clearList.end(); ++it)
	{
		CDialogCertRequest* certDlg = (*it);
		m_requestList.remove(certDlg);
		SAFE_DELETE(certDlg);
	}
	
	clearList.clear();
}
