#include "CCertificateManager.h"

CCertificateException::~CCertificateException() throw ()
{
}

CCertificateManager::CCertificateManager(CString aPermPath, CString aFlashPath) :
	m_tempStore(),
	m_permStore(aPermPath),
	m_flashStore(aFlashPath)
{
	VX509CertificateRefList permList = m_permStore.getCertList();
	VX509CertificateRefList tempList = m_tempStore.getCertList();
	VX509CertificateRefList flashList = m_flashStore.getCertList();
	
	m_sema = sceKernelCreateSema("CCertificateManager_Sema", 0, 1, 1, NULL);
	setX509RootCAs(flashList);
	
	for (VX509CertificateRefList::iterator it = permList.begin(); it != permList.end(); ++it)
	{
		m_certSet.insert(*it);
	}
	
	for (VX509CertificateRefList::iterator it = tempList.begin(); it != tempList.end(); ++it)
	{
		m_certSet.insert(*it);
	}
}

CCertificateManager::~CCertificateManager()
{
	lock();
	m_certSet.clear();
	unlock();
	sceKernelDeleteSema(m_sema);
	m_sema = 0;
}

bool CCertificateManager::addTempCert(VX509CertificateRef aCert)
{
	lock();
	m_certSet.insert(aCert);
	unlock();
	return true;
}

bool CCertificateManager::addPermCert(VX509CertificateRef aCert)
{
	lock();
	if (m_permStore.storeCert(aCert))
	{
		m_certSet.insert(aCert);
		unlock();
		return true;
	}
	
	unlock();
	return false;
}

void CCertificateManager::verify(VCertificateChainRef aCertChain)
{
	VX509CertificateRef verifyCert = aCertChain->getAt(0).dynamicCast <VX509Certificate> ();

	lock();
	
	try
	{
		bool autoTrust;
		CCertHashSet::iterator it = m_certSet.find(verifyCert);

		/* If the cert exists in the hash set, we just need to validate the chain and dates */
		if (it != m_certSet.end())
			autoTrust = true;
		else
			autoTrust = false;
		
		vmime::security::cert::defaultCertificateVerifier::verify(aCertChain, autoTrust);
	}
	catch (vmime::exceptions::certificate_verification_exception& e)
	{
		/* We compare the string of the exception to ensure we don't request
		 * the user in cases where his acceptance won't affect the outcome.
		 * ex: if the certificate date is expired */
		if (CString(e.what()) == "Cannot verify certificate against trusted certificates.")
		{
			unlock();
			throw CCertificateException(verifyCert, e.what());
		}
		
		unlock();
		throw e;
	}
	catch (std::exception& e)
	{
		unlock();
		throw e;
	}
	
	unlock();
}

void CCertificateManager::lock()
{
	sceKernelWaitSema(m_sema, 1, NULL);
}

void CCertificateManager::unlock()
{
	sceKernelSignalSema(m_sema, 1);
}
