/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CComposeWindow.cpp

  Description :	

  Last Modified $Date: 2007/01/01 21:55:47 $

  $Revision: 1.6 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CComposeWindow.h"
#include "CFrameWork.h"
#include "CTextureManager.h"
#include "CFont.h"
#include "CGfx.h"
#include <psprtc.h>
#include "CWindow.h"
#include "CPSPoste.h"
#include "CControlTextBox.h"
#include "CControl.h"
#include "CDialogBoxInput.h"
#include "CControlButton.h"
#include "CHUHelp.h"
#include "CContextMenu.h"
#include "CControlAddress.h"
#include "CMailbox.h"
#include "CKeyboardNavWrapper.h"

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************

//******************************************************************************
//   Class Definition
//******************************************************************************


//******************************************************************************
//******************************************************************************
//
//	Main Window implementation
//
//******************************************************************************
//******************************************************************************

//******************************************************************************
//	
//******************************************************************************
CComposeWindow::CComposeWindow(CMailbox* mailbox) : m_pMailbox(mailbox)
{
	init();
    m_sendEmpty = true;
}

//******************************************************************************
//	
//******************************************************************************
CComposeWindow::CComposeWindow(CMailbox* mailbox, CContactEntry* contact) : m_pMailbox(mailbox)
{
	init();
	m_sendEmpty = false;

	m_pToField->AddAddress(contact);
	
}

//******************************************************************************
//	
//******************************************************************************
CComposeWindow::CComposeWindow(CMailbox* mailbox,
								CControlAddress::AddressEntryList* to,
								CControlAddress::AddressEntryList* cc,
								CControlAddress::AddressEntryList* bcc,
								CText* subject,
								CText* body) : m_pMailbox(mailbox)
{
	init();
	m_sendEmpty = false;

	CControlAddress::AddressEntryList::iterator it;

	// Add to
	if (to != NULL) {
		it = to->begin();
		while (it != to->end()) {
			m_pToField->AddAddress(*it);
			it++;
		}
	}

	SAFE_DELETE(to);

	// Add cc
	if (cc != NULL) {
		it = cc->begin();
		while (it != cc->end()) {
			m_pCcField->AddAddress(*it);
			it++;
		}
	}
	SAFE_DELETE(cc);

	// Add bcc
	if (bcc != NULL) {
		it = bcc->begin();
		while (it != bcc->end()) {
			m_pBccField->AddAddress(*it);
			it++;
		}
	}
	SAFE_DELETE(bcc);

	// Add subject
	if (subject != NULL) {
		m_pSubjectField->setText(subject);
	}

	// Add body
	if (body != NULL) {
		m_pMessageField->setText(body);
	}
}

//******************************************************************************
//	
//******************************************************************************
CComposeWindow::~CComposeWindow()
{
	m_pMailbox->unsubscribe(this);

	SAFE_DELETE(m_pControlManager); //will delete all of the controls
	SAFE_DELETE(m_pContextMenu);
	SAFE_DELETE(m_pMsgDialog);
}

//******************************************************************************
//	
//******************************************************************************
void CComposeWindow::init()
{
	//Initialize
	m_pMsgDialog = NULL;
	m_pContextMenu = NULL;
	m_exitValue = -1;
	m_sending = eSendingState_Inactive;

	m_pToField = new CControlAddress(V2(423, 20), V2(55, 5) );
	// These will need to be adjusted eventually for proper positioning
	m_pCcField = new CControlAddress(V2(423, 20), V2(55, 5) );
	m_pBccField = new CControlAddress(V2(423, 20), V2(55, 5) );
	m_pCcField->hide();
	m_pBccField->hide();

	m_pSubjectField = new CControlTextBox(V2(423, 20), V2(55, 30));
	m_pMessageField = new CControlMsgBox(V2(476, 216), V2(2, 55));
	m_pControlManager = new CControlManager();

	m_pToField->m_pDown = m_pSubjectField;
	m_pSubjectField->m_pUp = m_pToField;
	m_pSubjectField->m_pDown = m_pMessageField;
	m_pMessageField->m_pUp = m_pSubjectField;
   
	m_pControlManager->addControl(m_pToField);
	m_pControlManager->addControl(m_pCcField);
	m_pControlManager->addControl(m_pBccField);
	m_pControlManager->addControl(m_pSubjectField);
	m_pControlManager->addControl(m_pMessageField);

	m_pControlManager->addControl(new CControlLabel(V2(50, 20), V2(4, 8), 0xFF000000, CString("To:")));
	m_pControlManager->addControl(new CControlLabel(V2(50, 20), V2(4, 33), 0xFF000000, CString("Subject:")));

	m_pControlManager->selectControl(m_pToField);

	m_pMailbox->subscribe(this);

	SetFocus(true);
}

//******************************************************************************
//	
//******************************************************************************
void	CComposeWindow::Render()
{
	// Draw white background
	CGfx::DrawQuad( V2( 0, 0 ), V2( CGfx::s_ScreenWidth, CGfx::s_ScreenHeight), ARGB(255, 255, 255, 255));

	m_pControlManager->RenderControl();
}

//******************************************************************************
//	
//******************************************************************************
void	CComposeWindow::Process()
{
	handleHUH();
	handleSendingStateMachine();
	handleContextMenu();

	m_pControlManager->ProcessControl();

	CWindow::Process();
}

//******************************************************************************
//	
//******************************************************************************
bool	CComposeWindow::ProcessInput()
{
	if ( HasFocus() == true && m_pMsgDialog == NULL ) {	
		if (m_pControlManager->ProcessControlInput()) {
			if ( CKeyboardNavWrapper::CloseInputWindow() ) {
				CPSPoste::s_NewWindow = CWindow::WINDOW_MAIN;
			}
			if ( CKeyboardNavWrapper::OpenContextMenu() ) {
				// Buld the context menu
				CContextMenu::CContextMenuItemList * itemList = new CContextMenu::CContextMenuItemList();
				
				itemList->push_back( new CContextMenuItem("Hide Menu", HIDE, true));
				itemList->push_back( new CContextMenuItem("Send", SEND, true));
				itemList->push_back( new CContextMenuItem("Close", CLOSE, false));
				
				// Create the context menu
				m_pContextMenu = new CContextMenu(itemList, new V2(10, 10));
						
				SetFocus( false );
			}
		}
	}
	return false;
}

//******************************************************************************
//
//******************************************************************************
int	CComposeWindow::getExitValue()
{
	return m_exitValue;
}

//******************************************************************************
//
//******************************************************************************
void CComposeWindow::sendMessage()
{
	//Build the message
	CSendMsg* msg = new CSendMsg();
	msg->subject = m_pSubjectField->getText()->toString();
	msg->body = m_pMessageField->getText()->toString();

	// Add recipients
	CControlAddress::AddressEntryList* to = m_pToField->getAddressList();
	CControlAddress::AddressEntryList* cc = m_pCcField->getAddressList();
	CControlAddress::AddressEntryList* bcc = m_pBccField->getAddressList();

	// Add "to" recipients
	CControlAddress::AddressEntryList::iterator it = to->begin();
	while (it != to->end()) {
		msg->to.appendAddress(vmime::create<vmime::mailbox>((*it)->GetEmail()));
		it++;
	}
	
	// Add "cc" recipients
	it = cc->begin();
	while (it != cc->end()) {
		msg->cc.appendAddress(vmime::create<vmime::mailbox>((*it)->GetEmail()));
		it++;
	}
		
	// Add "bcc" recipients
	// TODO: No field
	it = bcc->begin();
	while (it != bcc->end()) {
		msg->bcc.appendAddress(vmime::create<vmime::mailbox>((*it)->GetEmail()));
		it++;
	}
	
	//Queue the message
	m_pMailbox->addSendMsg(msg);

	//Send the message
	if (int sendErr = m_pMailbox->sendAllMessages()) {
		if (sendErr == SEND_ERROR_NO_MESSAGES)
			m_sendError = CString("No messages to send.");
		else if (sendErr == SEND_ERROR_ALREADY_SENDING)
			m_sendError = CString("Already sending messages, please wait.");
		
		m_sending = eSendingState_Failed;
	}
}

void CComposeWindow::handleHUH()
{
	if (m_pMsgDialog == NULL && m_pContextMenu == NULL && !m_pControlManager->hasActive()) {
		CHUHelp::Clear();
		CHUHelp::SetButton(CHUHelp::BUTTON_UP, "Navigate");
		CHUHelp::SetButton(CHUHelp::BUTTON_DOWN, "Navigate");
		CHUHelp::SetButton(CHUHelp::BUTTON_RIGHT, "Navigate");
		CHUHelp::SetButton(CHUHelp::BUTTON_LEFT, "Navigate");
		CHUHelp::SetButton(CHUHelp::BUTTON_CROSS, "Activate Control");
		CHUHelp::SetButton(CHUHelp::BUTTON_TRIANGLE, "Context Menu");
	}
}

void CComposeWindow::handleContextMenu()
{
	if ( m_pContextMenu != NULL ) {
		if ( m_pContextMenu->getExitValue() != -1 ) {
			//Menu has return a value, process it see eMenuItems enum
			switch(m_pContextMenu->getExitValue()) {
				case HIDE:
					break;
				case SEND:
					m_sending = eSendingState_Start;		
					break;									   
				case CLOSE:
					if (m_sendEmpty)
					{ 
						CPSPoste::s_NewWindow = CWindow::WINDOW_MAIN;
					}
					else
					{
						m_exitValue = OK;
					}			 	
					break;
				default:
					break;
			}
			//Remove the context menu
			SAFE_DELETE(m_pContextMenu);

			//Return the focus to this window
			SetFocus( true );
		}
    }
}

void CComposeWindow::handleSendingStateMachine()
{
	switch(m_sending) {
		case eSendingState_Inactive:
			// Nothing going on
			break;
		case eSendingState_Start:
			m_sending = eSendingState_AddressApproval;
			break;
		case eSendingState_AddressApproval:
			if (m_pToField->getAddressList()->size() == 0) {
				if (m_pMsgDialog == NULL) {
					m_pMsgDialog = new CDialogBoxInput(250, V2(-1, -1), "You must include a recipient.", false, false);
					SetFocus(false);
				} else {
					if (m_pMsgDialog->getExitValue() != -1) {
						switch(m_pMsgDialog->getExitValue()) {
							case CDialogBoxInput::OK:
								m_sending = eSendingState_Inactive;
        						break;
							case CDialogBoxInput::CANCEL:
								break;
							default:
								break;
						}
						//Remove the dialog box
						SAFE_DELETE(m_pMsgDialog);
						SetFocus(true);
					}
				}
			} else {
				m_sending = eSendingState_SubjectApproval;
			}
			break;
		case eSendingState_SubjectApproval:
			if (m_pSubjectField->getText()->size() == 0) {
				if (m_pMsgDialog == NULL) {
					m_pMsgDialog = new CDialogBoxInput(250, V2(-1, -1), "Send message without a subject?", false, true);
					SetFocus(false);
				} else {
					if (m_pMsgDialog->getExitValue() != -1) {
						switch(m_pMsgDialog->getExitValue()) {
							case CDialogBoxInput::OK:
								m_sending = eSendingState_BodyApproval;
        						break;
							case CDialogBoxInput::CANCEL:
								m_sending = eSendingState_Inactive;
								break;
							default:
								break;
						}
						//Remove the dialog box
						SAFE_DELETE(m_pMsgDialog);
						SetFocus(true);
					}
				}
			} else {
				m_sending = eSendingState_BodyApproval;
			}
			break;
		case eSendingState_BodyApproval:
			if (m_pMessageField->getText()->size() == 0) {
				if (m_pMsgDialog == NULL) {
					m_pMsgDialog = new CDialogBoxInput(250, V2(-1, -1), "Send message without a body?", false, true);
					SetFocus(false);
				} else {
					if (m_pMsgDialog->getExitValue() != -1) {
						switch(m_pMsgDialog->getExitValue()) {
							case CDialogBoxInput::OK:
								m_sending = eSendingState_Approved;
        						break;
							case CDialogBoxInput::CANCEL:
								m_sending = eSendingState_Inactive;
								break;
							default:
								break;
						}
						//Remove the dialog box
						SAFE_DELETE(m_pMsgDialog);
						SetFocus(true);
					}
				}
			} else {
				m_sending = eSendingState_Approved;
			}
			break;
		case eSendingState_Approved:
			m_sending = eSendingState_Active;
			sendMessage();
			break;
		case eSendingState_Active:
			// Get out of this state from event from the mailbox
			if (m_pMsgDialog == NULL) {
				m_pMsgDialog = new CDialogBoxInput(250, V2(-1, -1), "Please wait while your message is being sent.", false, false);
				SetFocus(false);
			}
			break;
		case eSendingState_Failed:
			// Create the message for the user
			SAFE_DELETE(m_pMsgDialog);
			if (m_pMsgDialog == NULL) {
				CString dlgText;
				dlgText.Printf("Message send failed. %s", m_sendError.GetPtr());
				m_pMsgDialog = new CDialogBoxInput(250, V2(-1, -1), dlgText, false, false);
				SetFocus(false);
			}
			m_sending = eSendingState_FailedAwaitingConfirmation;
			break;
		case eSendingState_FailedAwaitingConfirmation:
			// Read input
			if (m_pMsgDialog->getExitValue() != -1) {
				switch(m_pMsgDialog->getExitValue()) {
					case CDialogBoxInput::OK:
						m_sending = eSendingState_Inactive;
						break;
					default:
						break;
				}
				//Remove the dialog box
				SAFE_DELETE(m_pMsgDialog);
				SetFocus(true);
			}
			break;
		case eSendingState_Complete:
			// Create the message for the user
			SAFE_DELETE(m_pMsgDialog)
			if (m_pMsgDialog == NULL) {
				m_pMsgDialog = new CDialogBoxInput(250, V2(-1, -1), "Message sent succesfully.", false, false);
				SetFocus(false);
			}
			m_sending = eSendingState_CompleteAwaitingConfirmation;
			break;
		case eSendingState_CompleteAwaitingConfirmation:
			if (m_pMsgDialog->getExitValue() != -1) {
				switch(m_pMsgDialog->getExitValue()) {
					case CDialogBoxInput::OK:
						m_sending = eSendingState_Inactive;
						break;
					default:
						break;
				}
				//Remove the dialog box
				SAFE_DELETE(m_pMsgDialog);
				SetFocus(true);
			}
			break;
		default:
			break;
	}
}

void CComposeWindow::sendDone(CMailbox* aMailbox, u32 aTotal)
{
	m_sending = eSendingState_Complete;
}

void CComposeWindow::sendFailed(CMailbox* aMailbox, CString aErrorText)
{
	m_sending = eSendingState_Failed;
	m_sendError = aErrorText;
}
//*******************************  END OF FILE  ********************************
