/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CConfigFile.h

  Description :	

  Last Modified $Date: 2007/02/05 01:28:40 $

  $Revision: 1.6 $

*******************************************************************************/

#ifndef CCONFIGFILE_H_
#define CCONFIGFILE_H_

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CString.h"
#include "CTypes.h"
#include "tinyxml.h"
#include <stdio.h>

//******************************************************************************
//   Macros
//******************************************************************************

//******************************************************************************
//   Types
//******************************************************************************
class CSettings;
class CAccount;

//******************************************************************************
//   Constants
//******************************************************************************
#define LATEST_SETTINGS_VERSION 2

//******************************************************************************
//   Class definitions
//******************************************************************************

//******************************************************************************
//******************************************************************************
//
//	Config file reader class
//
//******************************************************************************
//******************************************************************************
class CConfigFile
{
	public:

		static bool				Open();
		static void				Close();
		static void				Save();

		static CSettings*		s_pSettings;
	
	protected:

		CConfigFile();
		~CConfigFile();
		static void PopulateXmlDocument(TiXmlDocument* document);

	private:

		static TiXmlDocument	*m_pDocument;

};

class CSettings
{
	public:

		CSettings() :
			skin(""),
			keyboard(""),
			display_bcc_and_cc_in_view_message(false),
			message_sort_style("DATE_ASC"),
			version(0),
			connection_selected(1),
			connection_internet_timeout(0),
			connection_socket_timeout(0),
			selectedAccount(NULL),
			accountList(NULL),
			analogNav(false)
		{
			accountList = new CAccountList();
		}

		/* Copy constructor */
		CSettings(CSettings* settings);
		~CSettings();
		
		void PopulateWithDefault();

	public:
		typedef std::vector< CAccount * >	CAccountList;

	public: 
		CString skin;
		CString	keyboard;
		bool display_bcc_and_cc_in_view_message;
		CString	message_sort_style;
		u32 version;
		u32 connection_selected;
		u32 connection_internet_timeout;
		u32 connection_socket_timeout;
		CAccount *selectedAccount;
		CAccountList *accountList; 
		bool analogNav;
};

class CAccount
{
	public:
		CAccount() 
		{
			display_name = CString("");
			email_address = CString("");
			account_type = CString("POP3");
			incoming_server = CString("");
			incoming_server_port = 0;
			incoming_server_username = CString("");
			incoming_server_password = CString("");
			incoming_server_tls = false;
			outgoing_server = CString("");
			outgoing_server_port = 0;
			outgoing_server_username = CString("");
			outgoing_server_password = CString("");
			outgoing_server_tls = false;
			leave_messages_on_server = true;
			reply_to = CString("");
		}

		/* Copy Constructor */
		CAccount(CAccount* account)
		{
			display_name = account->display_name;
			email_address = account->email_address;
			account_type = account->account_type;
			incoming_server = account->incoming_server;
			incoming_server_port = account->incoming_server_port;
			incoming_server_username = account->incoming_server_username;
			incoming_server_password = account->incoming_server_password;
			incoming_server_tls = account->incoming_server_tls;
			outgoing_server = account->outgoing_server;
			outgoing_server_port = account->outgoing_server_port;
			outgoing_server_username = account->outgoing_server_username;
			outgoing_server_password = account->outgoing_server_password;
			outgoing_server_tls = account->outgoing_server_tls;
			leave_messages_on_server = account->leave_messages_on_server;
			reply_to = account->reply_to;
		}

		~CAccount()
		{

		}

	public: 
		CString		display_name;
		CString		email_address;
		CString		account_type;
		CString		incoming_server;
		u32			incoming_server_port;
		CString		incoming_server_username;
		CString		incoming_server_password;
		bool		incoming_server_tls;
		CString		outgoing_server;
		u32			outgoing_server_port;
		CString		outgoing_server_username;
		CString		outgoing_server_password;
		bool		outgoing_server_tls;
		bool		leave_messages_on_server;
		CString		reply_to;
};

//******************************************************************************
//   Externs
//******************************************************************************

//******************************************************************************
//   Prototypes
//******************************************************************************

#endif /* CCONFIGFILE_H_ */
