/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CContacts.h

  Description :	

  Last Modified $Date: 2007/01/27 19:45:12 $

  $Revision: 1.5 $

*******************************************************************************/

#ifndef CCONTACTS_H_
#define CCONTACTS_H_

//******************************************************************************
//   Include Files
//******************************************************************************
#include "tinyxml.h"
#include "CString.h"
#include "CTypes.h"

//******************************************************************************
//   Macros
//******************************************************************************

//******************************************************************************
//   Types
//******************************************************************************
class CContactEntry;
class CContactEmailEntry;
class CContactPhoneEntry;

//******************************************************************************
//   Constants
//******************************************************************************

//******************************************************************************
//   Class definitions
//******************************************************************************

//******************************************************************************
//******************************************************************************
//
//	Config file reader class
//
//******************************************************************************
//******************************************************************************
class CContacts
{
	public:

		typedef std::vector< CContactEntry * >	CContactList;
		typedef std::vector< CContactEmailEntry * > CContactEmailList;
		typedef std::vector< CContactPhoneEntry * > CContactPhoneList;

	public:

		static bool				Open();
		static void				Close();
		static void				Save();
		
		static void				AddEntry( CContactEntry * const p_var);

	protected:

		CContacts();
		~CContacts();
		static void CreateDefaultContacts();
		static void PopulateXmlDocument(TiXmlDocument* document);

	private:

		static TiXmlDocument *	m_pDocument;
		static bool m_createdDefault;
       
	public:

		static CContactList *	s_pContacts;

};

//******************************************************************************
//******************************************************************************
//
//	Contact entry class
//
//******************************************************************************
//******************************************************************************
class CContactEntry
{
	public:

		CContactEntry( const CString & name, const CString & address, CContacts::CContactEmailList * p_emailList, CContacts::CContactPhoneList *p_phoneList)
			:	m_szName(name),
				m_szAddress(address),
				m_pContactEmailList(p_emailList),
				m_pContactPhoneList(p_phoneList)
		{
			if (m_pContactEmailList == NULL) {
				m_pContactEmailList = new CContacts::CContactEmailList();
			}
			if (m_pContactPhoneList == NULL) {
				m_pContactPhoneList = new CContacts::CContactPhoneList();
			}
        }

		~CContactEntry() {
		/*
		if (m_pContactEmailList != NULL) {
				CContacts::CContactEmailList::iterator it = m_pContactEmailList->begin();
				while (it != m_pContactEmailList->end()) {
					SAFE_DELETE(*it);
					it++;
				}
				SAFE_DELETE(m_pContactEmailList);
			}
		*/
		};
		
		const CString & GetName() const
        {
			return m_szName;
        }

		const CString & GetAddress() const
        {
            return m_szAddress;
        }
        
        void SetName(CString & name)
        {
			m_szName = name;
        }

		void SetAddress(CString & address)
        {
            m_szAddress = address;
        }

		CContacts::CContactEmailList * GetEmailList()
		{
			return m_pContactEmailList;
		}

		CContacts::CContactPhoneList * GetPhoneList()
		{
			return m_pContactPhoneList;
		}

	public:
		CString	m_szName;
		CString m_szAddress;
		CContacts::CContactEmailList *m_pContactEmailList;
		CContacts::CContactPhoneList *m_pContactPhoneList;
};

//******************************************************************************
//
//	Contact Email entry class
//
//******************************************************************************
class CContactEmailEntry
{
	public:

		CContactEmailEntry( const CString & name, const CString & email )
			:	m_szName( name ),
				m_szEmail( email ),
                m_bRemoveOnCancel(false)
		{

        }

		virtual ~CContactEmailEntry() {};
		
		const CString & GetName() const
        {
			return m_szName;
        }

        const CString & GetEmail() const
        {
            return m_szEmail;
        }
        
        void SetName(CString & newName)
        {
        	m_szName = newName;
        }
        
        void SetEmail(CString & newEmail)
        {
        	m_szEmail = newEmail;
        }

	public:

		CString	m_szName;
		CString	m_szEmail;
		bool m_bRemoveOnCancel;

};

//******************************************************************************
//
//  Contact Phone entry class
//
//******************************************************************************
class CContactPhoneEntry
{
    public:

        CContactPhoneEntry( const CString & name, const CString & phone )
            :   m_szName( name ),
                m_szPhone( phone ),
                m_bRemoveOnCancel(false)
        {

        }

        virtual ~CContactPhoneEntry() {};

        const CString & GetName() const
        {
            return m_szName;
        }

        const CString & GetPhone() const
        {
            return m_szPhone;
        }
        
        void SetName(CString & newName)
        {
        	m_szName = newName;
        }
        
        void SetPhone(CString & newPhone)
        {
        	m_szPhone = newPhone;
        }

    public:

        CString m_szName;
        CString m_szPhone;
        bool m_bRemoveOnCancel;

};


//******************************************************************************
//   Externs
//******************************************************************************

//******************************************************************************
//   Prototypes
//******************************************************************************

#endif /* CCONTACTS_H_ */
