/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CContactsEditWindow.cpp

  Description :	

  Last Modified $Date: 2007/01/27 18:42:53 $

  $Revision: 1.7 $

  Copyright (C) David Cummings, Sajeeth Cherian and Ben Biron

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CContactsEditWindow.h"
#include "CFrameWork.h"
#include "CTextureManager.h"
#include "CFont.h"
#include "CInput.h"
#include "CGfx.h"
#include <psprtc.h>
#include "CWindow.h"
#include "CPSPoste.h"
#include "CControlTextBox.h"
#include "CControl.h"
#include "CDialogBoxInput.h"
#include "CControlButton.h"
#include "CHUHelp.h"
#include "CContextMenu.h"
#include "CControlAddress.h"
#include "CContacts.h"
#include "CKeyboardNavWrapper.h"

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************
#define LEFT_MOST_X 4
#define RIGHT_MOST_X 474
#define TOP_MOST_Y 4
#define BOTTOM_MOST_Y 268
#define Y_SPACING 4
#define MIN_X_SPACING 2
#define PREFERRED_X_PADDING 4
#define BUTTON_TEXT_MIN_PADDING 10

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************

//******************************************************************************
//   Class Definition
//******************************************************************************


//******************************************************************************
//******************************************************************************
//
//	Main Window implementation
//
//******************************************************************************
//******************************************************************************

//******************************************************************************
//	
//******************************************************************************
CContactsEditWindow::CContactsEditWindow() : 
	m_pDialogBox(NULL),
	m_pNewEmailDialog(NULL),
	m_pNewPhoneDialog(NULL),
	m_pContextMenu(NULL),
	m_pMsgDialog(NULL),
	m_newContact(true),
	m_exitValue(-1),
	m_newEmailButtonIsCancel(false),
	m_newPhoneButtonIsCancel(false)
{
	m_pContact = new CContactEntry("new", "new", NULL, NULL);
	CreateWindowControls();
	m_pControlManager->selectControl(m_pNameField);
	SetEmailUiStateEmpty();
	SetPhoneUiStateEmpty();
	SetFocus(true);
}

//******************************************************************************
//	
//******************************************************************************
CContactsEditWindow::CContactsEditWindow(CContactEntry* contact) : 
	m_pDialogBox(NULL),
	m_pNewEmailDialog(NULL),
	m_pNewPhoneDialog(NULL),
	m_pContextMenu(NULL),
	m_pContact(contact),
	m_pMsgDialog(NULL),
	m_newContact(false),
	m_exitValue(-1),
	m_newEmailButtonIsCancel(false),
	m_newPhoneButtonIsCancel(false)
{
	CreateWindowControls();
	
	/* Get the contact emails */
	RebuildEmailSelect();
	
	/* Get the contact phone numbers */
	RebuildPhoneSelect();
	
	/* Set the appropriate UI state */
	SetEmailUiStateNormalOrEmpty();
	SetPhoneUiStateNormalOrEmpty();
	
	/* Populate the name and address fields */
	m_pNameField->setText(new CText(contact->GetName().GetStr()));
	m_pAddressField->setText(new CText(contact->GetAddress().GetStr()));
	m_pControlManager->selectControl(m_pNameField);
	SetFocus(true);
}

void CContactsEditWindow::CreateWindowControls()
{
	/* Label and button text */
	const char nameLabelText[] = "Name:";
	const char addressLabelText[] = "Address:";
	
	const char groupBoxText[] = "Email & Phone Info";
	
	const char emailsLabelText[] = "Emails:";
	const char emailNameLabelText[] = "Email Name:";
	const char emailAddressLabelText[] = "Email Address:";
	const char newEmailButtonText[] = "New Email";
	const char saveEmailButtonText[] = "Save Email";
	const char removeEmailButtonText[] = "Remove Email";
	
	const char phoneNumbersLabelText[] = "Phone Numbers:";
	const char phoneNameLabelText[] = "Phone Name:";
	const char phoneNumberLabelText[] = "Phone Number:";
	const char newPhoneButtonText[] = "New Phone";
	const char savePhoneButtonText[] = "Save Phone";
	const char removePhoneButtonText[] = "Remove Phone";
	
	/* Dimensions and positions */
	const int labelHeight = 20;
	const int textAndButtonHeight = 20;
	const int nameAndAddressLabelWidth = (int)CFont::GetActiveUiFont()->GetStringSize(addressLabelText).x;
	const int nameAndAddressControlX = LEFT_MOST_X + nameAndAddressLabelWidth + MIN_X_SPACING;
	const int nameAndAddressControlWidth = RIGHT_MOST_X - nameAndAddressControlX;
	const int groupBoxWidth = RIGHT_MOST_X - LEFT_MOST_X;
	const int groupBoxHeight = (9 * textAndButtonHeight) + (9 * Y_SPACING);
	const int groupBoxSeperator = (5 * textAndButtonHeight) + (4 * Y_SPACING);
	const int emailLabelsX = LEFT_MOST_X + PREFERRED_X_PADDING;
	const int emailLabelsWidth = (int)CFont::GetActiveUiFont()->GetStringSize(phoneNumbersLabelText).x;
	const int emailControlsX = emailLabelsX + emailLabelsWidth + MIN_X_SPACING;
	const int emailControlsWidth = RIGHT_MOST_X - PREFERRED_X_PADDING - emailControlsX;
	const int emailButtonsWidth = (int)CFont::GetActiveUiFont()->GetStringSize(removeEmailButtonText).x + (2*BUTTON_TEXT_MIN_PADDING);
	const int emailButtonsX = emailControlsX;
	const int phoneLabelsX = LEFT_MOST_X + PREFERRED_X_PADDING;
	const int phoneLabelsWidth = (int)CFont::GetActiveUiFont()->GetStringSize(phoneNumbersLabelText).x;
	const int phoneControlsX = phoneLabelsX + phoneLabelsWidth + MIN_X_SPACING;
	const int phoneControlsWidth = RIGHT_MOST_X - PREFERRED_X_PADDING - phoneControlsX;
	const int phoneButtonsWidth = emailButtonsWidth;
	const int phoneButtonsX = emailButtonsX;
	
	m_selectedEmail = -1;
	m_selectedPhone = -1;
	
	CControlSelect::CControlSelectItemList * emailItemList = new CControlSelect::CControlSelectItemList();	
	CControlSelectItem *currentSelectedEmail;
	CString comboText("(empty)");
	currentSelectedEmail = new CControlSelectItem(comboText, 0); 
	emailItemList->push_back(currentSelectedEmail);
	
	CControlSelect::CControlSelectItemList * phoneItemList = new CControlSelect::CControlSelectItemList();	
	CControlSelectItem *currentSelectedPhone;
	currentSelectedPhone = new CControlSelectItem(comboText, 0); 
	phoneItemList->push_back(currentSelectedPhone);
	
	/* Name & Address fields */
	int curY = TOP_MOST_Y;
	m_pNameField = new CControlTextBox(V2(nameAndAddressControlWidth, textAndButtonHeight), V2(nameAndAddressControlX, curY));
	curY += (textAndButtonHeight + Y_SPACING);
	m_pAddressField = new CControlTextBox (V2(nameAndAddressControlWidth, textAndButtonHeight), V2(nameAndAddressControlX, curY));
	
	curY += (textAndButtonHeight + Y_SPACING);
	m_pGroupBox = new CControlGroupBox(V2(groupBoxWidth, groupBoxHeight), V2(LEFT_MOST_X, curY), groupBoxText, groupBoxSeperator);
	
	/* Email fields */
	curY += (textAndButtonHeight);
	m_pEmailSelected = new CControlSelect(V2(emailControlsWidth, textAndButtonHeight), V2(emailControlsX, curY), 4, emailItemList, currentSelectedEmail);
	curY += (textAndButtonHeight + Y_SPACING);
	m_pEmailNameField = new CControlTextBox (V2(emailControlsWidth, textAndButtonHeight), V2(emailControlsX, curY));
	curY += (textAndButtonHeight + Y_SPACING);
	m_pEmailField = new CControlTextBox (V2(emailControlsWidth, textAndButtonHeight), V2(emailControlsX, curY));
	curY += (textAndButtonHeight + Y_SPACING);
	m_pEmailNewCancelButton = new CControlButton(V2(emailButtonsWidth, textAndButtonHeight), V2(emailButtonsX + 0*(emailButtonsWidth + PREFERRED_X_PADDING), curY), CString(newEmailButtonText));
	m_pEmailSaveButton = new CControlButton(V2(emailButtonsWidth, textAndButtonHeight), V2 (emailButtonsX + 1*(emailButtonsWidth + PREFERRED_X_PADDING), curY), CString(saveEmailButtonText));
	m_pEmailDeleteButton = new CControlButton(V2(emailButtonsWidth, textAndButtonHeight), V2 (emailButtonsX + 2*(emailButtonsWidth + PREFERRED_X_PADDING), curY), CString(removeEmailButtonText));
	
	/* "Seperator" line */
	curY += (textAndButtonHeight + (2*Y_SPACING));
	
	/* Phone fields */
	m_pPhoneSelected = new CControlSelect(V2(phoneControlsWidth, textAndButtonHeight), V2(phoneControlsX, curY), 4, phoneItemList, currentSelectedPhone);
	curY += (textAndButtonHeight + Y_SPACING);
	m_pPhoneNameField = new CControlTextBox (V2(phoneControlsWidth, textAndButtonHeight), V2(phoneControlsX, curY));
	curY += (textAndButtonHeight + Y_SPACING);
	m_pPhoneField = new CControlTextBox (V2(phoneControlsWidth, textAndButtonHeight), V2(phoneControlsX, curY));
	curY += (textAndButtonHeight + Y_SPACING);
	m_pPhoneNewCancelButton = new CControlButton(V2(phoneButtonsWidth, textAndButtonHeight), V2(phoneButtonsX + 0*(phoneButtonsWidth + PREFERRED_X_PADDING), curY), CString(newPhoneButtonText));
	m_pPhoneSaveButton = new CControlButton(V2(phoneButtonsWidth, textAndButtonHeight), V2 (phoneButtonsX + 1*(phoneButtonsWidth + PREFERRED_X_PADDING), curY), CString(savePhoneButtonText));
	m_pPhoneDeleteButton = new CControlButton(V2(phoneButtonsWidth, textAndButtonHeight), V2 (phoneButtonsX + 2*(phoneButtonsWidth + PREFERRED_X_PADDING), curY), CString(removePhoneButtonText));
	
	m_pEmailSelected->addSelectionListener(m_pGroupBox);
	m_pEmailNameField->addSelectionListener(m_pGroupBox);
	m_pEmailField->addSelectionListener(m_pGroupBox);
	m_pEmailNewCancelButton->addSelectionListener(m_pGroupBox);
	m_pEmailSaveButton->addSelectionListener(m_pGroupBox);
	m_pEmailDeleteButton->addSelectionListener(m_pGroupBox);
	m_pPhoneSelected->addSelectionListener(m_pGroupBox);
	m_pPhoneNameField->addSelectionListener(m_pGroupBox);
	m_pPhoneField->addSelectionListener(m_pGroupBox);
	m_pPhoneNewCancelButton->addSelectionListener(m_pGroupBox);
	m_pPhoneSaveButton->addSelectionListener(m_pGroupBox);
	m_pPhoneDeleteButton->addSelectionListener(m_pGroupBox);
	
	m_pEmailSelected->subscribe(this);
	m_pPhoneSelected->subscribe(this);
		
	m_pControlManager = new CControlManager();

	m_pNameField->m_pDown = m_pAddressField;
	m_pAddressField->m_pDown = m_pEmailSelected;
	m_pEmailSelected->m_pUp = m_pAddressField;
	m_pAddressField->m_pUp = m_pNameField;
	m_pEmailSelected->m_pDown = m_pEmailNameField;
	m_pEmailNameField->m_pUp = m_pEmailSelected;
	m_pEmailNameField->m_pDown = m_pEmailField;
	m_pEmailField->m_pUp = m_pEmailNameField;
	m_pEmailField->m_pDown = m_pEmailNewCancelButton;
	m_pEmailNewCancelButton->m_pUp = m_pEmailField;
	m_pEmailNewCancelButton->m_pDown = m_pPhoneSelected;
	m_pPhoneSelected->m_pUp = m_pEmailNewCancelButton;
	m_pPhoneSelected->m_pDown = m_pPhoneNameField;
	m_pPhoneNameField->m_pUp = m_pPhoneSelected;
	m_pPhoneNameField->m_pDown = m_pPhoneField;
	m_pPhoneField->m_pUp = m_pPhoneNameField;
	m_pPhoneField->m_pDown = m_pPhoneNewCancelButton;
	m_pPhoneNewCancelButton->m_pUp = m_pPhoneField;
	
	m_pEmailNewCancelButton->m_pRight = m_pEmailSaveButton;
	m_pEmailSaveButton->m_pLeft = m_pEmailNewCancelButton;
	m_pEmailSaveButton->m_pRight = m_pEmailDeleteButton;
	m_pEmailSaveButton->m_pDown = m_pPhoneSelected;
	m_pEmailSaveButton->m_pUp = m_pEmailField;
	m_pEmailDeleteButton->m_pLeft = m_pEmailSaveButton;
	m_pEmailDeleteButton->m_pDown = m_pPhoneSelected;
	m_pEmailDeleteButton->m_pUp = m_pEmailField;
	
	m_pPhoneNewCancelButton->m_pRight = m_pPhoneSaveButton;
	m_pPhoneSaveButton->m_pLeft = m_pPhoneNewCancelButton;
	m_pPhoneSaveButton->m_pRight = m_pPhoneDeleteButton;
	m_pPhoneSaveButton->m_pUp = m_pPhoneField;
	m_pPhoneDeleteButton->m_pLeft = m_pPhoneSaveButton;
	m_pPhoneDeleteButton->m_pUp = m_pPhoneField;
   
	m_pControlManager->addControl(m_pNameField);
	m_pControlManager->addControl(m_pEmailSelected);
	m_pControlManager->addControl(m_pEmailNameField);
	m_pControlManager->addControl(m_pEmailField);
	m_pControlManager->addControl(m_pEmailNewCancelButton);
	m_pControlManager->addControl(m_pEmailSaveButton);
	m_pControlManager->addControl(m_pEmailDeleteButton);
	m_pControlManager->addControl(m_pPhoneSelected);
	m_pControlManager->addControl(m_pPhoneNameField);
	m_pControlManager->addControl(m_pPhoneField);
	m_pControlManager->addControl(m_pPhoneNewCancelButton);
	m_pControlManager->addControl(m_pPhoneSaveButton);
	m_pControlManager->addControl(m_pPhoneDeleteButton);
	m_pControlManager->addControl(m_pAddressField);
	m_pControlManager->addControl(m_pGroupBox);

	curY = TOP_MOST_Y;
	m_pControlManager->addControl(new CControlLabel(V2(nameAndAddressLabelWidth, labelHeight), V2(LEFT_MOST_X, curY), 0xFF000000, CString(nameLabelText)));
	curY += (textAndButtonHeight + Y_SPACING);
	m_pControlManager->addControl(new CControlLabel(V2(nameAndAddressLabelWidth, labelHeight), V2(LEFT_MOST_X, curY), 0xFF000000, CString(addressLabelText)));
	curY += 2*(textAndButtonHeight + Y_SPACING); /* x2 for the top GroupBox line */
	m_pControlManager->addControl(new CControlLabel(V2(emailLabelsWidth, labelHeight), V2(emailLabelsX, curY), 0xFF000000, CString(emailsLabelText)));
	curY += (textAndButtonHeight + Y_SPACING);
	m_pControlManager->addControl(new CControlLabel(V2(emailLabelsWidth, labelHeight), V2(emailLabelsX, curY), 0xFF000000, CString(emailNameLabelText)));
	curY += (textAndButtonHeight + Y_SPACING);
	m_pControlManager->addControl(new CControlLabel(V2(emailLabelsWidth, labelHeight), V2(emailLabelsX, curY), 0xFF000000, CString(emailAddressLabelText)));
	curY += 2*(textAndButtonHeight + Y_SPACING); /* x2 because of the row of buttons between emails and phone */
	m_pControlManager->addControl(new CControlLabel(V2(phoneLabelsWidth, labelHeight), V2(phoneLabelsX, curY), 0xFF000000, CString(phoneNumbersLabelText)));
	curY += (textAndButtonHeight + Y_SPACING);
	m_pControlManager->addControl(new CControlLabel(V2(phoneLabelsWidth, labelHeight), V2(phoneLabelsX, curY), 0xFF000000, CString(phoneNameLabelText)));
	curY += (textAndButtonHeight + Y_SPACING);
	m_pControlManager->addControl(new CControlLabel(V2(phoneLabelsWidth, labelHeight), V2(phoneLabelsX, curY), 0xFF000000, CString(phoneNumberLabelText)));
}

//******************************************************************************
//	
//******************************************************************************
CContactsEditWindow::~CContactsEditWindow()
{
	SAFE_DELETE(m_pControlManager); //will delete all of the controls
	SAFE_DELETE(m_pContextMenu);
}

//******************************************************************************
//	
//******************************************************************************
void	CContactsEditWindow::Render()
{
	// Draw white background
	CGfx::DrawQuad( V2( 0, 0 ), V2( CGfx::s_ScreenWidth, CGfx::s_ScreenHeight), ARGB(255, 255, 255, 255));

	m_pControlManager->RenderControl();
}

//******************************************************************************
//	
//******************************************************************************
void	CContactsEditWindow::Process()
{
	if ( m_pDialogBox == NULL && m_pContextMenu == NULL && !m_pControlManager->hasActive()) {
		CHUHelp::Clear();
		CHUHelp::SetButton(CHUHelp::BUTTON_UP, "Navigate Controls");
		CHUHelp::SetButton(CHUHelp::BUTTON_DOWN, "Navigate Controls");
		CHUHelp::SetButton(CHUHelp::BUTTON_RIGHT, "Navigate Controls");
		CHUHelp::SetButton(CHUHelp::BUTTON_LEFT, "Navigate Controls");
		CHUHelp::SetButton(CHUHelp::BUTTON_CROSS, "Activate control");
		CHUHelp::SetButton(CHUHelp::BUTTON_TRIANGLE, "Open context menu");
	}

	if ( m_pContextMenu != NULL ) {
		if ( m_pContextMenu->getExitValue() != -1 ) {
			//Menu has return a value, process it see eMenuItems enum
			switch(m_pContextMenu->getExitValue()) {
				case HIDE:
					break;									   
				case CANCEL:
					if (m_newContact)
					{
					   	SAFE_DELETE(m_pContact);
					}
					m_exitValue = OK;		 	
					break;
				case SAVE:
					if (m_newContact)
					{
					   	SAFE_DELETE(m_pContextMenu);
						SetFocus(true);   			 
						if (SaveNewContact())
						{
						   	m_exitValue = OK;
						}
						return; 
					}
					else
					{
					 	SAFE_DELETE(m_pContextMenu);
						SetFocus(true);  
						SaveExistingContact();
						m_exitValue = OK;
						return;
					}
					break;
				default:
					break;
			}
			//Remove the context menu
			SAFE_DELETE(m_pContextMenu);

			//Return the focus to this window
			SetFocus( true );
		}
    }

    if (m_pEmailNewCancelButton->getClicked()) {
    	m_selectedEmail = -1;
		
		if (m_newEmailButtonIsCancel)
			SetEmailUiStateNormalOrEmpty();
		else
			SetEmailUiStateNew();
	}

    if (m_pEmailSaveButton->getClicked()) {
		if (SaveEmailToContact())
			SetEmailUiStateNormalOrEmpty();
	}
	
	if (m_pEmailDeleteButton->getClicked()) {
		RemoveEmailFromContact();
		SetEmailUiStateNormalOrEmpty();
	}
	
	if (m_pPhoneNewCancelButton->getClicked()) {
		m_selectedPhone = -1;
		m_pPhoneNameField->setText(new CText(""));
		m_pPhoneField->setText(new CText(""));
		
		if (m_newPhoneButtonIsCancel)
			SetPhoneUiStateNormalOrEmpty();
		else
			SetPhoneUiStateNew();
	}
	
	if (m_pPhoneSaveButton->getClicked()) {
		if (SavePhoneToContact())
			SetPhoneUiStateNormalOrEmpty();
	}
	
	if (m_pPhoneDeleteButton->getClicked()) {
		RemovePhoneFromContact();
		SetPhoneUiStateNormalOrEmpty();
	}
	
	if (m_pMsgDialog != NULL) {
		if (m_pMsgDialog->getExitValue() != -1) {
			//Remove the dialog box
			SAFE_DELETE(m_pMsgDialog);
			SetFocus(true);
		}
	}

	m_pControlManager->ProcessControl();

	CWindow::Process();
}

//******************************************************************************
//	
//******************************************************************************
bool	CContactsEditWindow::ProcessInput()
{
	if ( HasFocus() == true && m_pMsgDialog == NULL ) {	
		if (m_pControlManager->ProcessControlInput()) {
			if ( CKeyboardNavWrapper::CloseInputWindow() ) {
				if (m_newContact)
				{
				   	SAFE_DELETE(m_pContact);
				}
				m_exitValue = OK;
			}
			if ( CKeyboardNavWrapper::OpenContextMenu() ) {
				// Buld the context menu
				CContextMenu::CContextMenuItemList * itemList = new CContextMenu::CContextMenuItemList();
				
				itemList->push_back(new CContextMenuItem("Hide Menu", HIDE, true));
				if (m_newContact)
				{
					itemList->push_back(new CContextMenuItem("Cancel", CANCEL, false));
					itemList->push_back(new CContextMenuItem("Add", SAVE, false));
				}
				else
				{
					itemList->push_back(new CContextMenuItem("Close", SAVE, false));	
				}
				
				// Create the context menu
				m_pContextMenu = new CContextMenu(itemList, new V2(10, 10));
						
				SetFocus( false );
			}
		}
	}
	return false;
}

int CContactsEditWindow::getExitValue()
{
	return m_exitValue;
}

bool CContactsEditWindow::SaveEmailToContact()
{
	if (m_pEmailNameField->getText()->size() == 0)
	{
   		m_pMsgDialog = new CDialogBoxInput(250, V2(-1, -1), "You must enter a name for this e-mail address.", false, false);
   		SetFocus(false);
   		return false;
	}
	
	if (m_pEmailField->getText()->size() == 0)
	{
   		m_pMsgDialog = new CDialogBoxInput(250, V2(-1, -1), "You must enter an e-mail address.", false, false);
   		SetFocus(false);
   		return false;
	}
	
	if (m_selectedEmail == -1)
	{
		/* This is a new email entry */
		CString name = m_pEmailNameField->getText()->toString();
		CString email = m_pEmailField->getText()->toString();
		if (IsValidEmailName(name))
		{
		 	CContactEmailEntry* p_newEmailEntry = new CContactEmailEntry(name, email);
		 	p_newEmailEntry->m_bRemoveOnCancel = true;
			m_pContact->GetEmailList()->push_back(p_newEmailEntry);   
			RebuildEmailSelect(m_pContact->GetEmailList()->size() - 1);
		}
		else
		{
	   		m_pMsgDialog = new CDialogBoxInput(250, V2(-1, -1), "Email name already in use.", false, false);
	   		SetFocus(false);
			return false;
		}
	}
	else
	{
		/* This is a modified email entry */
		CString name = m_pEmailNameField->getText()->toString();
		CString email = m_pEmailField->getText()->toString();
		if (IsValidEmailName(name, m_selectedEmail))
		{
			CContactEmailEntry* entry = m_pContact->GetEmailList()->at(m_selectedEmail);
			entry->SetName(name);
			entry->SetEmail(email);
			RebuildEmailSelect(m_selectedEmail);
		}
		else
		{
	   		m_pMsgDialog = new CDialogBoxInput(250, V2(-1, -1), "Email name already in use.", false, false);
	   		SetFocus(false);
			return false;
		}
	}
	
	return true;
}

void CContactsEditWindow::RemoveEmailFromContact()
{
	if (m_pContact->GetEmailList()->size() == 1)
	{
   		m_pMsgDialog = new CDialogBoxInput(250, V2(-1, -1), "You must have at least 1 e-mail address per contact.", false, false);
   		SetFocus(false);
   		return;
	}
	else
	{ 	  
		CString text("");
		CContacts::CContactEmailList::iterator itEmail = m_pContact->GetEmailList()->begin();
		while (itEmail != m_pContact->GetEmailList()->end())
		{
			text.Printf("%s (%s)", (const char *)(*itEmail)->GetName(), (const char *)(*itEmail)->GetEmail());
			if (m_pEmailSelected->getSelectedText().Equals(text))
			{
				m_pContact->GetEmailList()->erase(itEmail);
				break;
			}
			itEmail++;
		}
		RebuildEmailSelect();
	}	
}

void CContactsEditWindow::RebuildEmailSelect(u32 selectedIndex)
{
	CString text;
	CControlSelect::CControlSelectItemList * emailItemList = new CControlSelect::CControlSelectItemList();
	
	if (m_pContact->GetEmailList()->size() == 0)
	{
		text = "(empty)";
		emailItemList->push_back(new CControlSelectItem(text, 0));
		m_pEmailSelected->rebuildList(emailItemList, emailItemList->at(0));
		m_pEmailNameField->setText(new CText(""));
		m_pEmailField->setText(new CText(""));
		m_selectedEmail = -1;
	}
	else
	{
		int i = 0;
		CContacts::CContactEmailList::iterator itEmail = m_pContact->GetEmailList()->begin();
		while (itEmail != m_pContact->GetEmailList()->end())
		{
			text.Printf("%s (%s)", (const char *)(*itEmail)->GetName(), (const char *)(*itEmail)->GetEmail());
			emailItemList->push_back(new CControlSelectItem(text, i));
			i += 1;
			itEmail++;
		}
		
		/* If the selectedIndex is invalid */
		if ((selectedIndex > m_pContact->GetEmailList()->size()) || (selectedIndex < 0))
			selectedIndex = 0;
		
		CContactEmailEntry* selectedEmail = m_pContact->GetEmailList()->at(selectedIndex);
		m_pEmailSelected->rebuildList(emailItemList, emailItemList->at(selectedIndex));
		m_pEmailNameField->setText(new CText(selectedEmail->GetName().GetStr()));
		m_pEmailField->setText(new CText(selectedEmail->GetEmail().GetStr()));
		m_selectedEmail = selectedIndex;
	}
}

bool CContactsEditWindow::SavePhoneToContact()
{
	if (m_pPhoneNameField->getText()->size() == 0)
	{
   		m_pMsgDialog = new CDialogBoxInput(250, V2(-1, -1), "You must enter a name for this phone number.", false, false);
   		SetFocus(false);
   		return false;
	}
	
	if (m_pPhoneField->getText()->size() == 0)
	{
   		m_pMsgDialog = new CDialogBoxInput(250, V2(-1, -1), "You must enter a phone number.", false, false);
   		SetFocus(false);
   		return false;
	}
	
	if (m_selectedPhone == -1)
	{
		CString name = m_pPhoneNameField->getText()->toString();
		CString phone = m_pPhoneField->getText()->toString();
		if (IsValidPhoneName(name))
		{
			/* This is a new phone entry */
		 	CContactPhoneEntry* p_newPhoneEntry = new CContactPhoneEntry(name, phone);
		 	p_newPhoneEntry->m_bRemoveOnCancel = true;
			m_pContact->GetPhoneList()->push_back(p_newPhoneEntry);   
			RebuildPhoneSelect(m_pContact->GetPhoneList()->size() - 1);
		}
		else
		{
	   		m_pMsgDialog = new CDialogBoxInput(250, V2(-1, -1), "Phone name already in use.", false, false);
	   		SetFocus(false);
	   		return false;
		}
	}
	else
	{
		/* This is a modified phone entry */
		CString name = m_pPhoneNameField->getText()->toString();
		CString phone = m_pPhoneField->getText()->toString();
		if (IsValidPhoneName(name, m_selectedPhone))
		{
			CContactPhoneEntry* entry = m_pContact->GetPhoneList()->at(m_selectedPhone);
			entry->SetName(name);
			entry->SetPhone(phone);
			RebuildPhoneSelect(m_selectedPhone);
		}
		else
		{
	   		m_pMsgDialog = new CDialogBoxInput(250, V2(-1, -1), "Phone name already in use.", false, false);
	   		SetFocus(false);
	   		return false;
		}
	}
	
	return true;
}

void CContactsEditWindow::RemovePhoneFromContact()
{
 	// Delete Phone Number from Phone Number List 
	CString text("");
	CContacts::CContactPhoneList::iterator itPhone = m_pContact->GetPhoneList()->begin();
	while (itPhone != m_pContact->GetPhoneList()->end())
	{
		text.Printf("%s (%s)", (const char *)(*itPhone)->GetName(), (const char *)(*itPhone)->GetPhone());
		if (m_pPhoneSelected->getSelectedText().Equals(text))
		{
			m_pContact->GetPhoneList()->erase(itPhone);
			break;
		}
		itPhone++;
	}	
	// Redraw the Phone Number List
	RebuildPhoneSelect();		
}

void CContactsEditWindow::RebuildPhoneSelect(u32 selectedIndex)
{
	CString text;
	CControlSelect::CControlSelectItemList * phoneItemList = new CControlSelect::CControlSelectItemList();
	
	if (m_pContact->GetPhoneList()->size() == 0)
	{
		text = "(empty)";
		phoneItemList->push_back(new CControlSelectItem(text, 0));	   									
		m_pPhoneSelected->rebuildList(phoneItemList, phoneItemList->at(0));
		m_pPhoneNameField->setText(new CText(""));
		m_pPhoneField->setText(new CText(""));
		m_selectedPhone = -1;
	}
	else
	{
		int i = 0;
		CContacts::CContactPhoneList::iterator itPhone = m_pContact->GetPhoneList()->begin();
		while (itPhone != m_pContact->GetPhoneList()->end()) {
			text.Printf("%s (%s)", (const char *)(*itPhone)->GetName(), (const char *)(*itPhone)->GetPhone());
			phoneItemList->push_back(new CControlSelectItem(text, i));		 	 
			i += 1;
			itPhone++;
		}
		
		/* If the selectedIndex is invalid */
		if ((selectedIndex > m_pContact->GetPhoneList()->size()) || (selectedIndex < 0))
			selectedIndex = 0;
		
		CContactPhoneEntry* selectedPhone = m_pContact->GetPhoneList()->at(selectedIndex);
		m_pPhoneSelected->rebuildList(phoneItemList, phoneItemList->at(selectedIndex));
		m_pPhoneNameField->setText(new CText(selectedPhone->GetName().GetStr()));
		m_pPhoneField->setText(new CText(selectedPhone->GetPhone().GetStr()));
		m_selectedPhone = selectedIndex;
	}
}

bool CContactsEditWindow::SaveNewContact()
{
	if (m_pNameField->getText()->size() == 0)
	{
   		m_pMsgDialog = new CDialogBoxInput(250, V2(-1, -1), "You must have a name for this contact.", false, false);
   		SetFocus(false);
   		return false;
	} 
 	CString text("(empty)"); 
	if (m_pEmailSelected->getSelectedText().Equals(text))
	{
   		m_pMsgDialog = new CDialogBoxInput(250, V2(-1, -1), "You must add at least 1 e-mail address.", false, false);
   		SetFocus(false);
   		return false;
	}
 	text = m_pNameField->getText()->toString();
	m_pContact->SetName(text);
	text = m_pAddressField->getText()->toString();
	m_pContact->SetAddress(text);
	CContacts::AddEntry(m_pContact);
	return true;
}

void CContactsEditWindow::SaveExistingContact()
{
 	CString text(""); 
 	text = m_pNameField->getText()->toString();
	m_pContact->SetName(text);
	text = m_pAddressField->getText()->toString();
	m_pContact->SetAddress(text);
}

void CContactsEditWindow::onChange(CControlSelect* control)
{
	if (control == m_pEmailSelected)
	{
		u32 selectedEmail = m_pEmailSelected->getSelectedValue();
		m_pEmailNameField->setText(new CText(m_pContact->GetEmailList()->at(selectedEmail)->GetName().GetPtr()));
		m_pEmailField->setText(new CText(m_pContact->GetEmailList()->at(selectedEmail)->GetEmail().GetPtr()));
		m_selectedEmail = (int)selectedEmail;
	}
	else if (control == m_pPhoneSelected)
	{
		u32 selectedPhone = m_pPhoneSelected->getSelectedValue();
		m_pPhoneNameField->setText(new CText(m_pContact->GetPhoneList()->at(selectedPhone)->GetName().GetPtr()));
		m_pPhoneField->setText(new CText(m_pContact->GetPhoneList()->at(selectedPhone)->GetPhone().GetPtr()));
		m_selectedPhone = selectedPhone;
	}
}

void CContactsEditWindow::SetEmailUiStateEmpty()
{
	SetNewEmailButtonIsCancel(false);
	m_pControlManager->disableControl(m_pEmailSelected);
	m_pControlManager->disableControl(m_pEmailNameField);
	m_pControlManager->disableControl(m_pEmailField);
	m_pControlManager->enableControl(m_pEmailNewCancelButton);
	m_pControlManager->disableControl(m_pEmailSaveButton);
	m_pControlManager->disableControl(m_pEmailDeleteButton);
	
	m_pEmailNameField->setText(new CText(""));
	m_pEmailField->setText(new CText(""));
}

void CContactsEditWindow::SetEmailUiStateNew()
{
	SetNewEmailButtonIsCancel(true);
	m_pControlManager->disableControl(m_pEmailSelected);
	m_pControlManager->enableControl(m_pEmailNameField);
	m_pControlManager->enableControl(m_pEmailField);
	m_pControlManager->enableControl(m_pEmailNewCancelButton);
	m_pControlManager->enableControl(m_pEmailSaveButton);
	m_pControlManager->disableControl(m_pEmailDeleteButton);
	
	m_pEmailNameField->setText(new CText(""));
	m_pEmailField->setText(new CText(""));
	
	m_pControlManager->selectControl(m_pEmailNameField);
}

void CContactsEditWindow::SetEmailUiStateNormal()
{
	SetNewEmailButtonIsCancel(false);
	m_pControlManager->enableControl(m_pEmailSelected);
	m_pControlManager->enableControl(m_pEmailNameField);
	m_pControlManager->enableControl(m_pEmailField);
	m_pControlManager->enableControl(m_pEmailNewCancelButton);
	m_pControlManager->enableControl(m_pEmailSaveButton);
	m_pControlManager->enableControl(m_pEmailDeleteButton);
	
	CContactEmailEntry* selectedEmail = m_pContact->GetEmailList()->at(m_pEmailSelected->getSelectedValue());
	m_pEmailNameField->setText(new CText(selectedEmail->GetName().GetStr()));
	m_pEmailField->setText(new CText(selectedEmail->GetEmail().GetStr()));
}

void CContactsEditWindow::SetEmailUiStateNormalOrEmpty()
{
	if (m_pContact->GetEmailList()->size() == 0)
		SetEmailUiStateEmpty();
	else
		SetEmailUiStateNormal();
}

void CContactsEditWindow::SetNewEmailButtonIsCancel(bool isCancel)
{
	if (m_newEmailButtonIsCancel != isCancel)
	{
		m_newEmailButtonIsCancel = isCancel;
		if (isCancel)
			m_pEmailNewCancelButton->setText("Cancel");
		else
			m_pEmailNewCancelButton->setText("New Email");
	}
}

void CContactsEditWindow::SetPhoneUiStateEmpty()
{
	SetNewPhoneButtonIsCancel(false);
	m_pControlManager->disableControl(m_pPhoneSelected);
	m_pControlManager->disableControl(m_pPhoneNameField);
	m_pControlManager->disableControl(m_pPhoneField);
	m_pControlManager->enableControl(m_pPhoneNewCancelButton);
	m_pControlManager->disableControl(m_pPhoneSaveButton);
	m_pControlManager->disableControl(m_pPhoneDeleteButton);
	
	m_pPhoneNameField->setText(new CText(""));
	m_pPhoneField->setText(new CText(""));
}

void CContactsEditWindow::SetPhoneUiStateNew()
{
	SetNewPhoneButtonIsCancel(true);
	m_pControlManager->disableControl(m_pPhoneSelected);
	m_pControlManager->enableControl(m_pPhoneNameField);
	m_pControlManager->enableControl(m_pPhoneField);
	m_pControlManager->enableControl(m_pPhoneNewCancelButton);
	m_pControlManager->enableControl(m_pPhoneSaveButton);
	m_pControlManager->disableControl(m_pPhoneDeleteButton);
	
	m_pPhoneNameField->setText(new CText(""));
	m_pPhoneField->setText(new CText(""));
	
	m_pControlManager->selectControl(m_pPhoneNameField);
}

void CContactsEditWindow::SetPhoneUiStateNormal()
{
	SetNewPhoneButtonIsCancel(false);
	m_pControlManager->enableControl(m_pPhoneSelected);
	m_pControlManager->enableControl(m_pPhoneNameField);
	m_pControlManager->enableControl(m_pPhoneField);
	m_pControlManager->enableControl(m_pPhoneNewCancelButton);
	m_pControlManager->enableControl(m_pPhoneSaveButton);
	m_pControlManager->enableControl(m_pPhoneDeleteButton);
	
	CContactPhoneEntry* selectedPhone = m_pContact->GetPhoneList()->at(m_pPhoneSelected->getSelectedValue());
	m_pPhoneNameField->setText(new CText(selectedPhone->GetName().GetStr()));
	m_pPhoneField->setText(new CText(selectedPhone->GetPhone().GetStr()));
}

void CContactsEditWindow::SetPhoneUiStateNormalOrEmpty()
{
	if (m_pContact->GetPhoneList()->size() == 0)
		SetPhoneUiStateEmpty();
	else
		SetPhoneUiStateNormal();
}

void CContactsEditWindow::SetNewPhoneButtonIsCancel(bool isCancel)
{
	if (m_newPhoneButtonIsCancel != isCancel)
	{
		m_newPhoneButtonIsCancel = isCancel;
		if (isCancel)
			m_pPhoneNewCancelButton->setText("Cancel");
		else
			m_pPhoneNewCancelButton->setText("New Phone");
	}
}

bool CContactsEditWindow::IsValidEmailName(CString name, int skip)
{
	int i = 0;
	CContacts::CContactEmailList::iterator itEmail = m_pContact->GetEmailList()->begin();
	while (itEmail != m_pContact->GetEmailList()->end())
	{
		if ((i++ != skip) && (*itEmail)->GetName().IEquals(name))
			return false;
		
		itEmail++;
	}
	
	return true;
}

bool CContactsEditWindow::IsValidPhoneName(CString name, int skip)
{
	int i = 0;
	CContacts::CContactPhoneList::iterator itPhone = m_pContact->GetPhoneList()->begin();
	while (itPhone != m_pContact->GetPhoneList()->end())
	{
		if ((i++ != skip) && (*itPhone)->GetName().IEquals(name))
			return false;
		
		itPhone++;
	}
	
	return true;
}

//*******************************  END OF FILE  ********************************
