/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CContactsEditWindow.h

  Description :	

  Last Modified $Date: 2007/01/27 18:06:08 $

  $Revision: 1.3 $

*******************************************************************************/

#ifndef CCONTACTSEDITWINDOW_H_
#define CCONTACTSEDITWINDOW_H_

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CTypes.h"
#include "CWindow.h"
#include "CFileSystem.h"
#include "CString.h"
#include "CGfx.h"
#include "CControlTextBox.h"
#include "CControlSelect.h"
#include "CControlButton.h"
#include "CControlAddress.h"
#include "CControlMsgBox.h"
#include "CDialogBoxInput.h"
#include "CContextMenu.h"
#include "CControlManager.h"
#include "CContacts.h"
#include "CControlGroupBox.h"

//******************************************************************************
//   Macros
//******************************************************************************

//******************************************************************************
//   Types
//******************************************************************************

//******************************************************************************
//   Constants
//******************************************************************************


//******************************************************************************
//   Class definitions
//******************************************************************************
class CContactsEditWindow : public CWindow, public MControlSelectEventListener
{
	public:
		   
		enum eExitCode {
			OK
		};
		
		CContactsEditWindow();
		CContactsEditWindow(CContactEntry* contact);
		~CContactsEditWindow();

		virtual void			Render();
		virtual void			Process();
		virtual bool			ProcessInput();
		int						getExitValue();
		
	public: // Inherited from MControlSelectEventListener
		void onChange(CControlSelect* control);

	private:

		CDialogBoxInput* m_pDialogBox;
		CDialogBoxInput* m_pNewEmailDialog;
		CDialogBoxInput* m_pNewPhoneDialog;
		
		void CreateWindowControls();
		bool SaveEmailToContact();
		void RemoveEmailFromContact();
		void RebuildEmailSelect(u32 selectedIndex = 0);
		bool SavePhoneToContact();
		void RemovePhoneFromContact();
		void RebuildPhoneSelect(u32 selectedIndex = 0);
		bool SaveNewContact();
		void SaveExistingContact();
		
		void SetEmailUiStateEmpty();
		void SetEmailUiStateNew();
		void SetEmailUiStateNormal();
		void SetEmailUiStateNormalOrEmpty();
		void SetNewEmailButtonIsCancel(bool isCancel);
		bool IsValidEmailName(CString name, int skip = -1);
		
		void SetPhoneUiStateEmpty();
		void SetPhoneUiStateNew();
		void SetPhoneUiStateNormal();
		void SetPhoneUiStateNormalOrEmpty();
		void SetNewPhoneButtonIsCancel(bool isCancel);
		bool IsValidPhoneName(CString name, int skip = -1);

	private:

		CControlTextBox* m_pNameField;
		CControlSelect* m_pEmailSelected;
		CControlTextBox* m_pEmailNameField;
		CControlTextBox* m_pEmailField;
		CControlButton* m_pEmailNewCancelButton;
		CControlButton* m_pEmailSaveButton;
		CControlButton* m_pEmailDeleteButton;
		CControlSelect* m_pPhoneSelected;
		CControlTextBox* m_pPhoneNameField;
		CControlTextBox* m_pPhoneField;
		CControlButton* m_pPhoneNewCancelButton;
		CControlButton* m_pPhoneSaveButton;
		CControlButton* m_pPhoneDeleteButton;
		CControlTextBox* m_pAddressField;
		CControlGroupBox* m_pGroupBox;
		CControlManager* m_pControlManager;

		CContextMenu* m_pContextMenu;
		CContactEntry* m_pContact;
		CDialogBoxInput* m_pMsgDialog;
		
		bool m_newContact;
		int m_exitValue;
		int m_selectedEmail;
		int m_selectedPhone;
		bool m_newEmailButtonIsCancel;
		bool m_newPhoneButtonIsCancel;

		enum eMenuItems {
			HIDE,
			CANCEL,
			SAVE
		};
};

//******************************************************************************
//   Externs
//******************************************************************************

//******************************************************************************
//   Prototypes
//******************************************************************************

#endif /* CCONTACTSEDITWINDOW_H_ */
