/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CContactsWindow.cpp

  Description :	

  Last Modified $Date: 2007/01/27 18:03:06 $

  $Revision: 1.6 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CContactsWindow.h"
#include "CFrameWork.h"
#include "CTextureManager.h"
#include "CFont.h"
#include "CInput.h"
#include "CGfx.h"
#include <psprtc.h>
#include "CWindow.h"
#include "CPSPoste.h"
#include "CContacts.h"
#include "CHUHelp.h"
#include "CKeyboardNavWrapper.h"

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************
/* General message list consts */
#define FIRST_CONTACT_Y_POSITION 2
#define CONTACT_Y_SPACING 18
#define ROW_HEIGHT 16

/* Column label sizes and positions */
#define CONTACT_COL_WIDTH 460
#define CONTACT_COL_X 4

/* Highlight bar of selected messages */
#define SELECTED_BORDER_WIDTH 2
#define SELECTED_BAR_WIDTH 468
#define SELECTED_BAR_X 0

/* The scroll bar */
#define SCROLL_BAR_WIDTH 12
#define SCROLL_BAR_HEIGHT 272
#define SCROLL_BAR_X 468
#define SCROLL_BAR_Y 1

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************

//******************************************************************************
//   Class Definition
//******************************************************************************

//******************************************************************************
//******************************************************************************
//
//	Main Window implementation
//
//******************************************************************************
//******************************************************************************

//******************************************************************************
//	
//******************************************************************************
CContactsWindow::CContactsWindow(CMailbox* mailbox) :
	m_pComposeWindow(NULL),
	m_pContactsEditWindow(NULL),							   
	m_pContextMenu(NULL),
	m_pControlManager(NULL),
	m_pScrollBar(NULL),
	m_pSelectedContact(NULL),
	m_selectedContactNum(0),
	m_pFirstContact(NULL),
	m_firstContactNum(0),
	m_saveChanges(false),
	m_pMailbox(mailbox)
{
 	m_pControlManager = new CControlManager();
	m_pScrollBar = new CControlScrollBar(V2(SCROLL_BAR_WIDTH, SCROLL_BAR_HEIGHT),
										 V2(SCROLL_BAR_X, SCROLL_BAR_Y),
										 DISPLAYED_CONTACT_COUNT);
	m_pControlManager->addControl(m_pScrollBar);
	
 	m_pDisplayedContacts = new CContacts::CContactList();
 	
 	// Lets instantiate the message labels
	int yPosition = FIRST_CONTACT_Y_POSITION;
	for (int i = 0; i < DISPLAYED_CONTACT_COUNT; i++)
	{
		m_pContactsLabels[i] = new CControlLabel(
		               V2(CONTACT_COL_WIDTH, ROW_HEIGHT),
		               V2(CONTACT_COL_X, yPosition),
		               CSkinManager::GetColor("message_list",
		                                      "inactive_text_color",
		                                      0xFF96CBFE),
		               CString("Test1"),
		               CControlLabel::MIDDLE_ALIGN
		           );
		yPosition += CONTACT_Y_SPACING;
	}
 	SetFocus(true);
 	BuildDisplayListFromScratch();
 	UpdateDisplayListText();
}

//******************************************************************************
//	
//******************************************************************************
CContactsWindow::~CContactsWindow()
{
	SAFE_DELETE(m_pDisplayedContacts); 
	SAFE_DELETE(m_pComposeWindow);
	SAFE_DELETE(m_pContextMenu); 
	
	for (int i = 0; i < DISPLAYED_CONTACT_COUNT; i++)
	{
		SAFE_DELETE(m_pContactsLabels[i]);
	}
	SAFE_DELETE(m_pControlManager); 
}

//******************************************************************************
//	
//******************************************************************************
void	CContactsWindow::Render()
{
	// Draw white background
	CGfx::DrawQuad( V2( 0, 0 ), V2( CGfx::s_ScreenWidth, CGfx::s_ScreenHeight), ARGB(255, 255, 255, 255));
	
	m_pControlManager->RenderControl();
	
	int yPosition = FIRST_CONTACT_Y_POSITION;
	int selectedYPosition = yPosition + m_selectedContactNum * CONTACT_Y_SPACING;

	// Draw Selected Message Box under the currently selected item
	CGfx::DrawQuad(V2(0, selectedYPosition - SELECTED_BORDER_WIDTH),
	      V2(SELECTED_BAR_X + SELECTED_BAR_WIDTH, ROW_HEIGHT + 2*SELECTED_BORDER_WIDTH),
	      CSkinManager::GetColor("message_list",
	                             "selected_msg_hilight_border_color",
	                             0xFF96CBFE));

	CGfx::DrawQuad(V2(SELECTED_BAR_X + SELECTED_BORDER_WIDTH, selectedYPosition),
	      V2(SELECTED_BAR_WIDTH - 2*SELECTED_BORDER_WIDTH, ROW_HEIGHT),
	      CSkinManager::GetColor("message_list",
	                             "selected_msg_hilight_color",
	                             0xFF96CBFE));
		                             
	int displayedSize = m_pDisplayedContacts->size();
    for (int i = 0; i < displayedSize; i++)
	{
		m_pContactsLabels[i]->RenderControl();
	}
    /*
	while (it != CContacts::s_pContacts->end()) {
		text.Printf("Name: %s", (const char *)(*it)->GetName());
		CFont::GetDefaultFont()->Print(text, V2(7, start), 0xff000000);
		start += increment;
		text.Printf("Address: %s", (const char *)(*it)->GetAddress());
		CFont::GetDefaultFont()->Print(text, V2(7, start), 0xff000000);
		start += increment;

        CContacts::CContactEmailList::iterator itEmail = (*it)->GetEmailList()->begin();
       	while (itEmail != (*it)->GetEmailList()->end()) {
           	text.Printf("Email (%s): %s\n ", (const char *) (*itEmail)->GetName(), (const char *) (*itEmail)->GetEmail());
			CFont::GetDefaultFont()->Print(text, V2(7, start), 0xff000000);
			start += increment;
            itEmail++;
        }

		CContacts::CContactPhoneList::iterator itPhone = (*it)->GetPhoneList()->begin();
        while (itPhone != (*it)->GetPhoneList()->end()) {
            text.Printf("Phone (%s): %s\n ", (const char *) (*itPhone)->GetName(), (const char *) (*itPhone)->GetPhone());
            CFont::GetDefaultFont()->Print(text, V2(7, start), 0xff000000);
            start += increment;
            itPhone++;
        }

        it++;

		text.Printf("\n");
		CFont::GetDefaultFont()->Print(text, V2(7, start), 0xff000000);
		start += increment;
    } */
}

//******************************************************************************
//	
//******************************************************************************
void	CContactsWindow::Process()
{

	if ( m_pComposeWindow != NULL ) {
		if ( m_pComposeWindow->getExitValue() != -1 ) {
			//Menu has return a value, process it see eMenuItems enum
			switch(m_pComposeWindow->getExitValue()) {
				case CComposeWindow::OK:
					break;
				default:
					break;
			}
			//Remove the compose window
			SAFE_DELETE(m_pComposeWindow);

			//Return the focus to this window
			SetFocus(true);
			SetVisible(true);
		}
	}
	
	if ( m_pContactsEditWindow != NULL ) {
		if ( m_pContactsEditWindow->getExitValue() != -1 ) {
			//Menu has return a value, process it see eMenuItems enum
			switch(m_pContactsEditWindow->getExitValue()) {
				case CContactsEditWindow::OK:
					break;
				default:
					break;
			}
			//Remove the contacts edit window
			SAFE_DELETE(m_pContactsEditWindow);
			
			BuildDisplayListFromScratch();
			UpdateDisplayListText();
			//Return the focus to this window
			SetFocus(true);
			SetVisible(true);
		}
	}  		
 		

	if (m_pContextMenu == NULL && m_pContactsEditWindow == NULL) {
		CHUHelp::Clear();
		CHUHelp::SetButton(CHUHelp::BUTTON_CROSS, "Edit contact");
		CHUHelp::SetButton(CHUHelp::BUTTON_UP, "Navigate");
		CHUHelp::SetButton(CHUHelp::BUTTON_DOWN, "Navigate");
		CHUHelp::SetButton(CHUHelp::BUTTON_TRIANGLE, "Context Menu");
	}

	if ( m_pContextMenu != NULL ) {
		if ( m_pContextMenu->getExitValue() != -1 ) {
			//Menu has return a value, process it see eMenuItems enum
			switch(m_pContextMenu->getExitValue()) {
				case HIDE:
					break;
				case ADD:
					m_pContactsEditWindow = new CContactsEditWindow();
					m_saveChanges = true;
					SetFocus(false);
					SetVisible(false);
					SAFE_DELETE(m_pContextMenu);
					CWindow::Process();
					return;
					break;
				case DELETE:
					DeleteContact(m_pSelectedContact);
					break;
				case EDIT:
					m_pContactsEditWindow = new CContactsEditWindow(m_pSelectedContact);
					m_saveChanges = true;
					SetFocus(false);
					SetVisible(false);
					SAFE_DELETE(m_pContextMenu);
					CWindow::Process();
					return;
					break;
				case SEND:
					if (CContacts::s_pContacts->size() > 0)
					{
						m_pComposeWindow = new CComposeWindow(m_pMailbox, m_pSelectedContact);
						SetFocus(false);
						SetVisible(false);
						SAFE_DELETE(m_pContextMenu);
						CWindow::Process();
						return;
					}
					break;
				case CANCEL:
					CContacts::Open();		   				  
					CPSPoste::s_NewWindow = CWindow::WINDOW_MAIN;
					break;									   
				case CLOSE:
					if (m_saveChanges)
					{
						CContacts::Save(); 					   				  
					} 
					CPSPoste::s_NewWindow = CWindow::WINDOW_MAIN;
					break;
				default:
					break;
			}
			//Remove the context menu
			SAFE_DELETE(m_pContextMenu);

			//Return the focus to this window
			SetFocus(true);
		}
    }
	
	CWindow::Process();
}

//******************************************************************************
//	
//******************************************************************************
bool	CContactsWindow::ProcessInput()
{
	if (HasFocus() == true) {
		if ( CKeyboardNavWrapper::CloseInputWindow() ) {
			CPSPoste::s_NewWindow = CWindow::WINDOW_MAIN;
		}
		if ( CKeyboardNavWrapper::OpenContextMenu() ) {
			// Buld the context menu
			CContextMenu::CContextMenuItemList * itemList = new CContextMenu::CContextMenuItemList();
			
			itemList->push_back( new CContextMenuItem("Hide Menu", HIDE, true));
			itemList->push_back( new CContextMenuItem("Add New Contact", ADD, true));
			
			if (CContacts::s_pContacts->size() > 0)
			{
				itemList->push_back( new CContextMenuItem("Delete Contact", DELETE, false));
				itemList->push_back( new CContextMenuItem("Edit Contact", EDIT, false));
				itemList->push_back( new CContextMenuItem("E-mail Contact", SEND, true));
			}
				
			itemList->push_back( new CContextMenuItem("Cancel", CANCEL, true));
			itemList->push_back( new CContextMenuItem("Save & Close", CLOSE, false));
			
			// Create the context menu
			m_pContextMenu = new CContextMenu(itemList, new V2(10, 10));
			
			SetFocus( false );
		}
		
		if (CContacts::s_pContacts->size() == 0)
		{
		   	return false;
		}
		
		if ( CKeyboardNavWrapper::Up() ) {
			MoveUp();
		}
		
		if ( CKeyboardNavWrapper::Down() ) {
			MoveDown();
		}
		
		if ( CKeyboardNavWrapper::PageUp() ) {
			MovePageUp();
		}
		
		if ( CKeyboardNavWrapper::PageDown() ) {
			MovePageDown();
		}
		
		if ( CKeyboardNavWrapper::ActivateControl() ) {
			m_pContactsEditWindow = new CContactsEditWindow(m_pSelectedContact);
			m_saveChanges = true;
			SetFocus(false);
			SetVisible(false);
		}
	}
	return false;
}

//******************************************************************************
//	
//******************************************************************************
void CContactsWindow::DeleteContact(CContactEntry* contact)
{
	CContacts::CContactList::iterator it = CContacts::s_pContacts->begin();
	while(it !=  CContacts::s_pContacts->end()) {
		if (contact == (*it))
		{
			CContacts::s_pContacts->erase(it);
			break;  
		}		  
		it++;
	}
	m_saveChanges = true;
	BuildDisplayListFromScratch();
	UpdateDisplayListText();
}

//******************************************************************************
//	
//******************************************************************************
void CContactsWindow::BuildDisplayListFromScratch()
{
	int i = 0;
	CContacts::CContactList::iterator it = CContacts::s_pContacts->begin();
	
	m_pDisplayedContacts->clear();
	
	while(it !=  CContacts::s_pContacts->end() && i < DISPLAYED_CONTACT_COUNT) {
		if (i == 0)
		{
			m_pFirstContact = (*it);
			m_pSelectedContact = (*it);
			m_selectedContactNum = 0;
			m_firstContactNum = 0;  
		}
		m_pDisplayedContacts->push_back((*it));   
		i++;
		it++;
	}

	UpdateScrollBar();
}

//******************************************************************************
//	
//******************************************************************************
void CContactsWindow::UpdateDisplayListText()
{

	CContacts::CContactList::iterator it = m_pDisplayedContacts->begin();	
	CString text("");
	int msgNum = 0;
	while (it != m_pDisplayedContacts->end()) {
        CContacts::CContactEmailList::iterator itEmail = (*it)->GetEmailList()->begin();
       	while (itEmail != (*it)->GetEmailList()->end()) {
           	text.Printf("%s (%s)", (const char *)(*it)->GetName(), (const char *) (*itEmail)->GetEmail());
           	m_pContactsLabels[msgNum]->setText(text);
			break;
        }
        it++;
		msgNum++;
    }

	UpdateScrollBar();
}

//******************************************************************************
//	
//******************************************************************************
void CContactsWindow::MoveUp()
{
 	// Check if we are at the top of the displayed contact list 
	if (m_selectedContactNum != 0) {
	   	// We are not at the top of the displayed contact list, we can still move up					 
		m_selectedContactNum--;
		CContacts::CContactList::iterator it = m_pDisplayedContacts->begin();
		while( it !=  m_pDisplayedContacts->end() )
		{
			if ((*it) == m_pSelectedContact)
			{
				it--;
				m_pSelectedContact = (*it);
				break;	
			}				   	
			it++;
		}
	}
	else
	{
	 	// We are at the top of the displayed contact list
	 	// Check if we are at the top of the contact list 
		if (m_firstContactNum != 0)
		{
		   	// We are not at the top of the contact list, we can still move up			  
		   	m_firstContactNum--;
		   	int i = 0;				
		   	CContacts::CContactList::iterator it = CContacts::s_pContacts->begin();
		   	while(it !=  CContacts::s_pContacts->end())
			{
				if ((*it) == m_pFirstContact)
				{
					it--;						   		  
					m_pFirstContact = (*it);
					m_pSelectedContact = (*it);
					m_pDisplayedContacts->clear();
					i = 1;
				}
				if (i > 0)
				{
					m_pDisplayedContacts->push_back((*it));
					i++;
				}
				if (i == (DISPLAYED_CONTACT_COUNT+1))
				{
					break;
				}
				it++; 						
			}								   									   
		}	   
	}
	UpdateDisplayListText();
}

//******************************************************************************
//	
//******************************************************************************
void CContactsWindow::MoveDown()
{
 	// Check if we are at the end of the displayed contact list 
	if (m_selectedContactNum != (DISPLAYED_CONTACT_COUNT-1)) {
	   	// We are not at the end of the displayed contact list
	   	// Check if we are at the end of the contact list
		if (m_selectedContactNum != ((signed)CContacts::s_pContacts->size() - 1)) {
		 	// We are not at the end, we can still move down					     				  
		 	m_selectedContactNum++;
		 	CContacts::CContactList::iterator it = m_pDisplayedContacts->begin();	
		 	while(it !=  m_pDisplayedContacts->end())
		 	{
			 	if ((*it) == m_pSelectedContact)
			 	{
				 	it++;
				 	m_pSelectedContact = (*it);
				 	break;	
			 	}				   	
			 	it++;
		 	}
		}
	}
	else
	{
	 	// We are at the end of the displayed contact list
		if ((m_firstContactNum + (DISPLAYED_CONTACT_COUNT+1)) <= (signed)CContacts::s_pContacts->size())
		{
		   	m_firstContactNum++;
		   	int i = 0;
		   	CContacts::CContactList::iterator it = CContacts::s_pContacts->begin();
		   	while(it !=  CContacts::s_pContacts->end())
			{
				if ((*it) == m_pFirstContact)
				{
					it++;
					m_pFirstContact = (*it);
					m_pDisplayedContacts->clear();
					i = 1;
				}
				if (i > 0)
				{
					m_pDisplayedContacts->push_back((*it));
					i++;
				}
				if (i == (DISPLAYED_CONTACT_COUNT+1))
				{
					m_pSelectedContact = (*it);
					break;
				}
				it++;
			}
		}
	}
	UpdateDisplayListText();
}

//******************************************************************************
//	
//******************************************************************************
void CContactsWindow::MovePageUp()
{
 	// Check if we will hit the top of the contact list 
	if (m_firstContactNum >= DISPLAYED_CONTACT_COUNT)
	{
	   	// We will not hit the top of the contact list, we are fine				  
		m_firstContactNum -= DISPLAYED_CONTACT_COUNT;
		CContacts::CContactList::iterator it = CContacts::s_pContacts->begin();
		int i = 0;
		while(it !=  CContacts::s_pContacts->end())
		{
			if ((*it) == m_pFirstContact)
			{
				// For some reason the code below didn't work, for loop added as fix				   	
				//it -= DISPLAYED_CONTACT_COUNT;	  
				for (int j = 0; j < DISPLAYED_CONTACT_COUNT; j++)
				{
					it--;	 
				}
				m_pFirstContact = (*it);
				m_pSelectedContact = (*it);
				m_pDisplayedContacts->clear();
				i = 1;
			}
			if (i > 0)
			{
				m_pDisplayedContacts->push_back((*it));
				i++;
			}
			if (i == (DISPLAYED_CONTACT_COUNT+1))
			{
				break;
			}
			it++;
		}
		m_selectedContactNum = 0;
	}
	else if (m_firstContactNum > 0 && m_firstContactNum < DISPLAYED_CONTACT_COUNT)
	{
	 	// We can't display beyond the top of the displayed contact list
		// We must readjust the displayed contact list  
	 	m_firstContactNum = 0;
		CContacts::CContactList::iterator it = CContacts::s_pContacts->begin();
		int i = 0;
		while(it !=  CContacts::s_pContacts->end())
		{
			if (i == 0)
			{
				m_pFirstContact = (*it);
				m_pSelectedContact = (*it);
				m_pDisplayedContacts->clear();
			}
			m_pDisplayedContacts->push_back((*it));
			i++;
			if (i == DISPLAYED_CONTACT_COUNT)
			{
				break;
			}
			it++;
		}
		m_selectedContactNum = 0;
	}
	UpdateDisplayListText();
}

//******************************************************************************
//	
//******************************************************************************
void CContactsWindow::MovePageDown()
{
 	// Check if we are displaying the bottom of the contact list 
	if ((m_firstContactNum + DISPLAYED_CONTACT_COUNT) < (signed)CContacts::s_pContacts->size())
	{
	   	// We are not displaying the bottom of the contact list				   
		m_firstContactNum += DISPLAYED_CONTACT_COUNT;
		CContacts::CContactList::iterator it = CContacts::s_pContacts->begin();
		int i = 0;
		// Check if we will display beyond the bottom of the list
		if ((m_firstContactNum + DISPLAYED_CONTACT_COUNT) > (signed)CContacts::s_pContacts->size())
		{
		   	// We will display beyond the bottom of the list, we must readjust the list
			// so this does not happen   				   
			m_firstContactNum = (signed)CContacts::s_pContacts->size() - DISPLAYED_CONTACT_COUNT;
			while(it !=  CContacts::s_pContacts->end())
			{
			 	if (i == m_firstContactNum)
				{
					m_pFirstContact = (*it);
					m_pSelectedContact = (*it);
					m_pDisplayedContacts->clear();
				}
				if (i >= m_firstContactNum)
				{
					m_pDisplayedContacts->push_back((*it));
				}
				i++;
				it++;
			}
		}
		else
		{
		 	// We will not display beyond the bottom of the list, we are fine   							   									   
			it = CContacts::s_pContacts->begin();
			while(it !=  CContacts::s_pContacts->end())
			{
				if ((*it) == m_pFirstContact)
				{
				   	// For some reason the code below didn't work, for loop added as fix				   	
				   	//it += DISPLAYED_CONTACT_COUNT;	  
				   	for (int j = 0; j < DISPLAYED_CONTACT_COUNT; j++)
				   	{
					   	it++;	 
				   	}
					m_pFirstContact = (*it);
					m_pSelectedContact = (*it);
					m_pDisplayedContacts->clear();
					i = 1;
				}
				if (i > 0)
				{
					m_pDisplayedContacts->push_back((*it));
					i++;
				}
				if (i == (DISPLAYED_CONTACT_COUNT+1))
				{
					break;
				}
				it++;
			}
		}
		m_selectedContactNum = 0;
	}
	UpdateDisplayListText();
}

//******************************************************************************
//	
//******************************************************************************
void CContactsWindow::UpdateScrollBar() 
{
	m_pScrollBar->update(m_firstContactNum, (signed)CContacts::s_pContacts->size());
}
//*******************************  END OF FILE  ********************************
