/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CControlAddress.h

  Description :	

*******************************************************************************/

#ifndef CControlAddress_H_
#define CControlAddress_H_

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CTypes.h"
#include "CWindow.h"
#include "CControl.h"
#include "CGfx.h"
#include "CString.h"
#include "CDialogBoxInput.h"
#include "CContextMenu.h"
#include "CScrollingText.h"
#include "CContacts.h"

//******************************************************************************
//   Macros
//******************************************************************************

//******************************************************************************
//   Types
//******************************************************************************
class InputBoxAddressEntry;

//******************************************************************************
//   Constants
//******************************************************************************

//******************************************************************************
//   Class definitions
//******************************************************************************
class CControlAddress : public CControl
{
	public:
		typedef std::vector< InputBoxAddressEntry * >	AddressEntryList;

		enum eList
        {
			LIST_AVAILABLE,
            LIST_SELECTED
        };

		enum eMenuItems {
            CONTEXT_HIDE,
			CONTEXT_SINGLE_USE_EMAIL,
            CONTEXT_CLOSE
        };



	public:
		CControlAddress(V2 const size, V2 const position);

		virtual ~CControlAddress();
		
		virtual void RenderControl();
		virtual void ProcessControlInput();
		virtual void ProcessControl();
		void AddAddress(CContactEntry* contact);
		void AddAddress(InputBoxAddressEntry* entry);

	public:
		AddressEntryList* const getAddressList();

	private:
		void buildDisplayList(AddressEntryList *list, AddressEntryList *displayList, InputBoxAddressEntry **firstDisplayed);
		void updateFirstDisplayed(AddressEntryList *list, AddressEntryList *displayList, InputBoxAddressEntry **selected, InputBoxAddressEntry **firstDisplayed);
		void AddToSelectedList();
		void DeleteFromSelectedList();
 
	private:
		V2 m_size;
		V2 m_position; // relative to top left corner

		AddressEntryList		*m_pAvailAddressList;
		AddressEntryList		*m_pAvailDisplayAddressList;
		InputBoxAddressEntry	*m_pAvailSelectedItem;
		InputBoxAddressEntry	*m_pAvailDisplayedItem;
		CScrollingText			*m_pAvailScrolling;

		AddressEntryList		*m_pSelectAddressList;
		AddressEntryList		*m_pSelectDisplayAddressList;
		InputBoxAddressEntry	*m_pSelectSelectedItem;
		InputBoxAddressEntry	*m_pSelectDisplayedItem;
		CScrollingText			*m_pSelectScrolling;

		AddressEntryList		*m_pSingleEntryEmailList; //keep track of entries for deletion		
		CDialogBoxInput			*m_pDialogBox;
		CContextMenu			*m_pContextMenu;

		CString					m_szAddresses;
		CScrollingText			*m_pAddressScrolling;

		eList m_selectedList;			
};

//******************************************************************************
//
//  InputBoxAddressEntry class
//
//******************************************************************************
class InputBoxAddressEntry
{
    public:

        InputBoxAddressEntry( const CString & name, const CString & email )
            :   m_szName( name ),
                m_szEmail( email )
        {

        }

        virtual ~InputBoxAddressEntry() {};

        virtual const CString & GetName() const
        {
            return m_szName;
        }

        virtual const CString & GetEmail() const
        {
            return m_szEmail;
        }

    public:

        CString m_szName;
        CString m_szEmail;

};


//******************************************************************************
//   Externs
//******************************************************************************

//******************************************************************************
//   Prototypes
//******************************************************************************

#endif /* CControlAddress_H_ */
