/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CControlButton.cpp

  Description :	

  Last Modified $Date: 2007/01/27 18:05:50 $

  $Revision: 1.5 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CControlButton.h"
#include "CTextureManager.h"
#include "CFont.h"
#include "CGfx.h"
#include "CSkinManager.h"
#include "CFrameWork.h"
#include "CControl.h"

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************
static const float	WINDOW_BORDER( 1.f ); // Border around context menu (all sides)
static const float  WINDOW_PADDING( 1.f ); // Padding between border and menu item (all sides)

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************

//******************************************************************************
//   Class Definition
//******************************************************************************

//******************************************************************************
//	
//******************************************************************************
CControlButton::CControlButton(V2 const size, V2 const position, CString text) :
	m_size(size),
	m_position(position),
	m_text(text),
	m_clicked(false)
{

}

//******************************************************************************
//	
//******************************************************************************
CControlButton::~CControlButton()
{
	
}	

//******************************************************************************
//	
//******************************************************************************
void	CControlButton::RenderControl()
{
	if(isHidden()) {
		return;
	}

	// Draw the borders
    // Sort of cheating here, couldn't send bgcolor to none opaque, if we want that functionality
	// need to draw 4 seperate quads
	ARGB borderColor;
	if (isSelected())
		borderColor = CSkinManager::GetColor( "button", "selected_border_color", 0xFFAAFFAA);
	else
		borderColor = CSkinManager::GetColor( "button", "border_color", 0xFFAAFFAA);
	
	CGfx::DrawQuad( V2( m_position.x, m_position.y ),
					V2( m_size.x, m_size.y),
					borderColor);

	// Draw menu background (overlay over the menu border quad)
	ARGB backgroundColor;
	if (isEnabled())
		backgroundColor = CSkinManager::GetColor( "button", "background_color", 0xFF00FF00);
	else
		backgroundColor = CSkinManager::GetColor( "button", "disabled_background_color", 0xFF00FF00);
	
	CGfx::DrawQuad( V2( m_position.x + WINDOW_BORDER, m_position.y + WINDOW_BORDER),
					V2( m_size.x - WINDOW_BORDER * 2, m_size.y - WINDOW_BORDER * 2),
					backgroundColor);

	// Draw text
	V2 text_size( CFont::GetActiveUiFont()->GetStringSize( m_text ));
    V2 text_position( m_position.x + m_size.x / 2 - text_size.x /2, m_position.y + m_size.y /2 - text_size.y /2 );
	CFont::GetActiveUiFont()->Print( m_text, text_position, CSkinManager::GetColor( "button", "text_color", 0xFFFFFFFF));
}

//******************************************************************************
//
//******************************************************************************
void	CControlButton::ProcessControlInput()
{

}

//******************************************************************************
// Override the activate (so the focus doesn't stay on the button!)
//******************************************************************************
void    CControlButton::activate()
{
    //CControl::activate();
    m_clicked = true;
}

//******************************************************************************
//
//******************************************************************************
bool    CControlButton::isClicked()
{
    return m_clicked;
}

//******************************************************************************
//
//******************************************************************************
bool    CControlButton::getClicked()
{
	if (m_clicked) {
		m_clicked = false;
		return true;
	} 
	return false;
}

//******************************************************************************
// 
//******************************************************************************
void	CControlButton::ProcessControl()
{

}

//******************************************************************************
// 
//******************************************************************************
void	CControlButton::setText(CString text)
{
	m_text = text;
}

//*******************************  END OF FILE  ********************************
