/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CControlCheckBox.cpp

  Description :	

  Last Modified $Date: 2007/02/05 01:28:40 $

  $Revision: 1.8 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CControlCheckBox.h"
#include "CTextureManager.h"
#include "CFont.h"
#include "CInput.h"
#include "CGfx.h"
#include "CSkinManager.h"
#include "CFrameWork.h"
#include "CControl.h"
#include "CHUHelp.h"
#include "CConfigFile.h"

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************
static const float	WINDOW_BORDER( 1.f ); // Border around context menu (all sides)
static const float  WINDOW_PADDING( 1.f ); // Padding between border and menu item (all sides)

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************

//******************************************************************************
//   Class Definition
//******************************************************************************

//******************************************************************************
//	
//******************************************************************************
CControlCheckBox::CControlCheckBox(V2 const size, V2 const position, bool checked) :
	m_size(size),
	m_position(position),
	m_checked(checked)
{
	m_checked = checked;
}

//******************************************************************************
//	
//******************************************************************************
CControlCheckBox::~CControlCheckBox()
{

}	

//******************************************************************************
//	
//******************************************************************************
void	CControlCheckBox::RenderControl()
{
	if(isHidden()) {
		return;
	}

	// Draw the borders
    // Sort of cheating here, couldn't send bgcolor to none opaque, if we want that functionality
	// need to draw 4 seperate quads
	ARGB borderColor;
	if (isSelected())
		borderColor = CSkinManager::GetColor( "checkbox", "selected_border_color", 0xFFAAFFAA);
	else
		borderColor = CSkinManager::GetColor( "checkbox", "border_color", 0xFFAAFFAA);
	
	CGfx::DrawQuad( V2( m_position.x, m_position.y ),
	                V2( m_size.x, m_size.y),
	                borderColor);

	// Draw menu background (overlay over the menu border quad)
	ARGB backgroundColor;
	if (isEnabled())
		backgroundColor = CSkinManager::GetColor( "checkbox", "background_color", 0xFF00FF00);
	else
		backgroundColor = CSkinManager::GetColor( "checkbox", "disabled_background_color", 0xFF00FF00);
		
	CGfx::DrawQuad( V2( m_position.x + WINDOW_BORDER, m_position.y + WINDOW_BORDER),
					V2( m_size.x - WINDOW_BORDER * 2, m_size.y - WINDOW_BORDER * 2),
					backgroundColor);

	if (m_checked) 
	{		
		ARGB fullColor = CSkinManager::GetColor( "checkbox", "check_color", 0xFF00FF00);
		ARGB aaColor = (fullColor.color & 0xffffff) | ((int)((fullColor.color >> 24) * 0.50) << 24); // 50% Alpha
		
		// X vertices
		V2 xTopLeft, xTopRight, xBottomLeft, xBottomRight;
		xTopLeft.x = m_position.x + WINDOW_BORDER + WINDOW_PADDING + 2; // +2 for AA lines
		xTopLeft.y = m_position.y + WINDOW_BORDER + WINDOW_PADDING + 2; // +2 for AA lines
		xTopRight.x = m_position.x + m_size.x - WINDOW_BORDER - WINDOW_PADDING - 2; // -2 for AA lines
		xTopRight.y = xTopLeft.y;
		xBottomLeft.x = xTopLeft.x;
		xBottomLeft.y = m_position.y + m_size.y - WINDOW_BORDER - WINDOW_PADDING - 2; // -2 for AA lines
		xBottomRight.x = xTopRight.x;
		xBottomRight.y = xBottomLeft.y;
		
		// Full color centre of x
		DrawX(xTopLeft, xTopRight, xBottomLeft, xBottomRight, fullColor);
		
		// AA Surrounding line #1
		DrawXAA(xTopLeft, xTopRight, xBottomLeft, xBottomRight, 1, aaColor);
		
/*
		ARGB fullColor = CSkinManager::GetColor( "checkbox", "check_color", 0xFF00FF00);
		ARGB aaColor1 = (fullColor.color & 0xffffff) | ((int)((fullColor.color >> 24) * 0.70) << 24); // 70% Alpha
		ARGB aaColor2 = (fullColor.color & 0xffffff) | ((int)((fullColor.color >> 24) * 0.30) << 24); // 30% Alpha

		// CheckMark vertices
		V2 checkLeft, checkBottom, checkRight;
		checkLeft.x = m_position.x + WINDOW_BORDER + WINDOW_PADDING + 2; // +2 for AA lines
		checkLeft.y = m_position.y + WINDOW_BORDER + 2.0/3.0 * (m_size.y - 2 * WINDOW_BORDER) - 2; // -2 for AA lines
		checkBottom.x = m_position.x + WINDOW_BORDER + 1.0/3.0 * (m_size.x - 2 * WINDOW_BORDER);
		checkBottom.y = m_position.y + m_size.y - WINDOW_BORDER - WINDOW_PADDING - 2; // -2 for AA lines
		checkRight.x = m_position.x + m_size.x - WINDOW_BORDER - WINDOW_PADDING - 2; // -2 for AA lines
		checkRight.y = m_position.y + WINDOW_BORDER + WINDOW_PADDING + 2; // + 2 for AA lines
		
		// Full color centre of the checkpoint
		DrawCheckMark(checkLeft, checkBottom, checkRight, fullColor);
		
		// AA Surrounding line #1
		DrawCheckMarkAA(checkLeft, checkBottom, checkRight, 1, aaColor1);
		
		// AA Surrounding line #2
		DrawCheckMarkAA(checkLeft, checkBottom, checkRight, 2, aaColor2);
*/
	}
}

//******************************************************************************
//
//******************************************************************************
void CControlCheckBox::DrawCheckMark(V2 left, V2 bottom, V2 right, ARGB color)
{
	CGfx::DrawLine(left, bottom, color);
	CGfx::DrawLine(bottom, right, color);
}

//******************************************************************************
//
//******************************************************************************
void CControlCheckBox::DrawCheckMarkAA(V2 left, V2 bottom, V2 right, float offset, ARGB color)
{
	V2 startPosAA, endPosAA;
	
	startPosAA.x = left.x;
	startPosAA.y = left.y - offset;
	endPosAA.x = startPosAA.x - offset;
	endPosAA.y = left.y;
	CGfx::DrawLine(startPosAA, endPosAA, color);
	
	startPosAA.x = endPosAA.x;
	startPosAA.y = endPosAA.y;
	endPosAA.x = bottom.x;
	endPosAA.y = bottom.y + offset;
	CGfx::DrawLine(startPosAA, endPosAA, color);
	
	startPosAA.x = endPosAA.x;
	startPosAA.y = endPosAA.y;
	endPosAA.x = right.x + offset;
	endPosAA.y = right.y;
	CGfx::DrawLine(startPosAA, endPosAA, color);
	
	startPosAA.x = endPosAA.x;
	startPosAA.y = endPosAA.y;
	endPosAA.x = right.x;
	endPosAA.y = right.y - offset;
	CGfx::DrawLine(startPosAA, endPosAA, color);
	
	startPosAA.x = endPosAA.x;
	startPosAA.y = endPosAA.y;
	endPosAA.x = bottom.x;
	endPosAA.y = bottom.y - offset;
	CGfx::DrawLine(startPosAA, endPosAA, color);
	
	startPosAA.x = endPosAA.x;
	startPosAA.y = endPosAA.y;
	endPosAA.x = left.x;
	endPosAA.y = left.y - offset;
	CGfx::DrawLine(startPosAA, endPosAA, color);
}

//******************************************************************************
//
//******************************************************************************
void CControlCheckBox::DrawX(V2 topLeft, V2 topRight, V2 bottomLeft, V2 bottomRight, ARGB color)
{
	CGfx::DrawLine(topLeft, bottomRight, color);
	CGfx::DrawLine(bottomLeft, topRight, color);
}

//******************************************************************************
//
//******************************************************************************
void CControlCheckBox::DrawXAA(V2 topLeft, V2 topRight, V2 bottomLeft, V2 bottomRight, float offset, ARGB color)
{
	V2 startPosAA, endPosAA, middle;
	middle.x = topLeft.x + ((topRight.x - topLeft.x) / 2);
	middle.y = topLeft.y + ((bottomLeft.y - topLeft.y) / 2);
	
	startPosAA.x = topLeft.x;
	startPosAA.y = topLeft.y - offset;
	endPosAA.x = topLeft.x - offset;
	endPosAA.y = topLeft.y;
	CGfx::DrawLine(startPosAA, endPosAA, color);
	
	startPosAA.x = endPosAA.x;
	startPosAA.y = endPosAA.y;
	endPosAA.x = middle.x - offset;
	endPosAA.y = middle.y;
	CGfx::DrawLine(startPosAA, endPosAA, color);
	
	startPosAA.x = endPosAA.x;
	startPosAA.y = endPosAA.y;
	endPosAA.x = bottomLeft.x - offset;
	endPosAA.y = bottomLeft.y;
	CGfx::DrawLine(startPosAA, endPosAA, color);
	
	startPosAA.x = endPosAA.x;
	startPosAA.y = endPosAA.y;
	endPosAA.x = bottomLeft.x;
	endPosAA.y = bottomLeft.y + offset;
	CGfx::DrawLine(startPosAA, endPosAA, color);
	
	startPosAA.x = endPosAA.x;
	startPosAA.y = endPosAA.y;
	endPosAA.x = middle.x;
	endPosAA.y = middle.y + offset;
	CGfx::DrawLine(startPosAA, endPosAA, color);
	
	startPosAA.x = endPosAA.x;
	startPosAA.y = endPosAA.y;
	endPosAA.x = bottomRight.x;
	endPosAA.y = bottomRight.y + offset;
	CGfx::DrawLine(startPosAA, endPosAA, color);
	
	startPosAA.x = endPosAA.x;
	startPosAA.y = endPosAA.y;
	endPosAA.x = bottomRight.x + offset;
	endPosAA.y = bottomRight.y;
	CGfx::DrawLine(startPosAA, endPosAA, color);
	
	startPosAA.x = endPosAA.x;
	startPosAA.y = endPosAA.y;
	endPosAA.x = middle.x + offset;
	endPosAA.y = middle.y;
	CGfx::DrawLine(startPosAA, endPosAA, color);
	
	startPosAA.x = endPosAA.x;
	startPosAA.y = endPosAA.y;
	endPosAA.x = topRight.x + offset;
	endPosAA.y = topRight.y;
	CGfx::DrawLine(startPosAA, endPosAA, color);
	
	startPosAA.x = endPosAA.x;
	startPosAA.y = endPosAA.y;
	endPosAA.x = topRight.x;
	endPosAA.y = topRight.y - offset;
	CGfx::DrawLine(startPosAA, endPosAA, color);
	
	startPosAA.x = endPosAA.x;
	startPosAA.y = endPosAA.y;
	endPosAA.x = middle.x;
	endPosAA.y = middle.y - offset;
	CGfx::DrawLine(startPosAA, endPosAA, color);
	
	startPosAA.x = endPosAA.x;
	startPosAA.y = endPosAA.y;
	endPosAA.x = topLeft.x;
	endPosAA.y = topLeft.y - offset;
	CGfx::DrawLine(startPosAA, endPosAA, color);
}

//******************************************************************************
//
//******************************************************************************
void CControlCheckBox::ProcessControlInput()
{

}

//******************************************************************************
//	
//******************************************************************************
bool CControlCheckBox::isChecked()
{
	return m_checked;
}

//******************************************************************************
//	
//******************************************************************************
void CControlCheckBox::check()
{
	m_checked = true;
}

//******************************************************************************
//	
//******************************************************************************
void CControlCheckBox::unCheck()
{
	m_checked = false;
}

//******************************************************************************
//	
//******************************************************************************
void CControlCheckBox::setCheck(bool check)
{
	m_checked = check;	
}

//******************************************************************************
// 
//******************************************************************************
void	CControlCheckBox::ProcessControl() {

}

//******************************************************************************
//	
//******************************************************************************
void CControlCheckBox::activate()
{
	m_checked = (m_checked) ? false : true;
}

//*******************************  END OF FILE  ********************************
