/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CControlHeader.cpp

  Description :	

  Last Modified $Date: 2007/02/10 02:24:12 $

  $Revision: 1.7 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CControlHeader.h"
#include "CTextureManager.h"
#include "CFont.h"
#include "CGfx.h"
#include "CSkinManager.h"
#include "CFrameWork.h"
#include "CControl.h"
#include "CHUHelp.h"
#include "CConnectionManager.h"

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************
static s32	s_BatteryFlash( 0 );

//******************************************************************************
//   Class Definition
//******************************************************************************

//******************************************************************************
//	
//******************************************************************************
CControlHeader::CControlHeader():
	m_pTimeLabel(NULL),
	m_pDateLabel(NULL),
	m_pNewMsgLabel(NULL),
	m_pTitleLabel(NULL),
	m_pConnectionLabel(NULL),
	m_pAccountLabel(NULL)
{
	m_pTimeLabel = new CControlLabel(V2(60, 16), V2(5, 5), CSkinManager::GetColor("main_window", "time_text_color", 0xFF0000), CString("12:00 AM"));
	m_pDateLabel = new CControlLabel(V2(160, 16), V2(5, 25), CSkinManager::GetColor("main_window", "date_text_color", 0xFF0000), CString("Wednesday November 30"));
	m_pNewMsgLabel = new CControlLabel(V2(40, 16), V2(197, 24), CSkinManager::GetColor("main_window", "unread_msgs_text_color", 0xFF0000), CString("9999"), CControlLabel::RIGHT_JUSTIFY);
	m_pTitleLabel = new CControlLabel(V2(480, 18), V2(0, 42), CSkinManager::GetColor("main_window", "title_bar_text_color", 0xFF0000), CString("Psprint Game"), CControlLabel::MIDDLE_JUSTIFY|CControlLabel::MIDDLE_ALIGN);
	m_pConnectionLabel = new CControlLabel(V2(160, 20), V2(315, 25), CSkinManager::GetColor( "main_window", "time_text_color", 0xFF0000), CString(" "), CControlLabel::RIGHT_JUSTIFY);
	m_pAccountLabel = new CControlLabel(V2(480, 18), V2(0, 5), CSkinManager::GetColor( "main_window", "account_text_color", 0xFF0000), CString(" "), CControlLabel::MIDDLE_JUSTIFY);
}

//******************************************************************************
//	
//******************************************************************************
CControlHeader::~CControlHeader()
{
	SAFE_DELETE(m_pTimeLabel);
	SAFE_DELETE(m_pDateLabel);
	SAFE_DELETE(m_pNewMsgLabel);
	SAFE_DELETE(m_pTitleLabel);
	SAFE_DELETE(m_pConnectionLabel);
	SAFE_DELETE(m_pAccountLabel);
}	

//******************************************************************************
//	
//******************************************************************************
void	CControlHeader::RenderControl()
{
	// Draw white background
	CGfx::DrawQuad( V2( 0, 0 ), V2( CGfx::s_ScreenWidth, 59), CSkinManager::GetColor( "main_window", "background_color", 0x0FFFFFFF));

	// Draw black line
	CGfx::DrawQuad( V2( 0, 42 ), V2( CGfx::s_ScreenWidth, 1),  CSkinManager::GetColor( "main_window", "title_bar_outline_color", 0xFF000000));
	
	// Draw a wack tacky light blue bar
	CGfx::DrawQuad( V2( 0, 43 ), V2( CGfx::s_ScreenWidth, 16), CSkinManager::GetColor( "main_window", "title_bar_color", 0xFF96CBFE));

	// Draw black line
	CGfx::DrawQuad( V2( 0, 59 ), V2( CGfx::s_ScreenWidth, 1),  CSkinManager::GetColor( "main_window", "title_bar_outline_color", 0xFF000000));

    // Draw the battery meter
    CTexture *      p_battery;
    const s32       battery_power( scePowerGetBatteryLifePercent() );

    if ( battery_power <= 10 )              p_battery = CSkinManager::GetComponent( CSkinManager::SC_BATTERY_00 )->GetTexture();
    else if ( battery_power <= 35 )         p_battery = CSkinManager::GetComponent( CSkinManager::SC_BATTERY_01 )->GetTexture();
	else if ( battery_power <= 75 )         p_battery = CSkinManager::GetComponent( CSkinManager::SC_BATTERY_02 )->GetTexture();
	else                                    p_battery = CSkinManager::GetComponent( CSkinManager::SC_BATTERY_03 )->GetTexture();

	if ( scePowerIsBatteryExist() == 1 ) {

		if ( scePowerIsBatteryCharging() == 0 )
		{
			s_BatteryFlash = 0;
		}
		else
		{
			s_BatteryFlash = ( s_BatteryFlash + 1 ) % 64;
		}
	
		if ( ( s_BatteryFlash >> 5 ) == 0  ) {
			CGfx::DrawQuad( p_battery, V2( 451, 5), V2( 24, 12 ), 0xffffffff );
		}
	}

	// Draw time
	char timeStr[32];
	time_t tm = time(NULL);
	struct tm *ltm = localtime(&tm);
	strftime(timeStr, 32, "%I:%M %p", ltm);
	m_pTimeLabel->setText(timeStr);
	m_pTimeLabel->RenderControl();

	// Draw date
	char dateStr[64];
	strftime(dateStr, 64, "%A %B %d", ltm);
	m_pDateLabel->setText(dateStr);
	m_pDateLabel->RenderControl();
	
	// Draw name
	m_pTitleLabel->RenderControl();

	// Draw new messages icon and # of unread messages
	if (m_pNewMsgLabel->getText() != "0")
	{
		m_pNewMsgLabel->RenderControl();   							  
		CGfx::DrawQuad( CSkinManager::GetComponent( CSkinManager::SC_ICON_NEWMAIL )->GetTexture(), V2( 240, 23), V2( 16, 16 ), 0xffffffff );   							  
	}
	
	// Draw connection
	m_pConnectionLabel->RenderControl();
	
	// Draw account
	m_pAccountLabel->RenderControl();
}

//******************************************************************************
//
//******************************************************************************
void	CControlHeader::ProcessControlInput()
{

}

//******************************************************************************
// 
//******************************************************************************
void	CControlHeader::ProcessControl() {

}

//******************************************************************************
//
//******************************************************************************
void CControlHeader::setNewMessages(int num) {
	CString numberOfMessages; 
	numberOfMessages.Printf("%d", num);
	m_pNewMsgLabel->setText(numberOfMessages);
}

//******************************************************************************
//
//******************************************************************************
void CControlHeader::setTitle(CString text) {
	m_pTitleLabel->setText(text);
}

//******************************************************************************
//
//******************************************************************************
void CControlHeader::setConnection(CString text) {
	m_pConnectionLabel->setText(text);
}

//******************************************************************************
//
//******************************************************************************
void CControlHeader::setAccount(CString text) {
	m_pAccountLabel->setText(text);
}

//*******************************  END OF FILE  ********************************
