/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CControlLabel.cpp

  Description :	

  Last Modified $Date: 2007/01/01 21:55:47 $

  $Revision: 1.4 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CControlLabel.h"
#include "CTextureManager.h"
#include "CFont.h"
#include "CGfx.h"
#include "CSkinManager.h"
#include "CFrameWork.h"
#include "CControl.h"
#include "CHUHelp.h"

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************
static const u64 SCROLL_SPD = 150; //scroll every SCROLL_SPEED cpu ticks
static const CString SPACE("   "); // between the items

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************

//******************************************************************************
//   Class Definition
//******************************************************************************

//******************************************************************************
//	
//******************************************************************************
CControlLabel::CControlLabel(V2 const size, V2 const position, ARGB const color, CString text, u32 settings) :
	m_size(size),
	m_position(position),
	m_color(color),
	m_text(text),
	m_settings(settings),
	m_scroll(false)
{

}

//******************************************************************************
//	
//******************************************************************************
CControlLabel::~CControlLabel()
{
	
}	

//******************************************************************************
//	
//******************************************************************************
void	CControlLabel::RenderControl()
{
	if(isHidden()) {
		return;
	}

	// Actual position
	V2 actualPosition = V2(0, 0);
	V2 textSize = CFont::GetActiveUiFont()->GetStringSize(m_text);

	switch (m_settings & MASK_JUSTIFY) {
		case LEFT_JUSTIFY:
			actualPosition.x = m_position.x;
			break;
		case MIDDLE_JUSTIFY:
			actualPosition.x = m_position.x + m_size.x/2 - textSize.x/2;
			break;
		case RIGHT_JUSTIFY:
			actualPosition.x = m_position.x + m_size.x - textSize.x;
			break;	
	}

	switch (m_settings & MASK_ALIGN) {
		case TOP_ALIGN:
			actualPosition.y = m_position.y;
			break;
		case MIDDLE_ALIGN:
			actualPosition.y = m_position.y + m_size.y/2 - textSize.y/2;
			break;
		case BOTTOM_ALIGN:
			actualPosition.y = m_position.y + m_size.y - textSize.y;
			break;
	}
 
	V2 text_size(CFont::GetActiveUiFont()->GetStringSize(m_text));
	V2 spacer_size(CFont::GetActiveUiFont()->GetStringSize(SPACE));

	if (m_scroll && text_size.x > m_size.x) {

		u64 thisTicks = CFrameWork::GetTicks();
		u64 l_cur = thisTicks - m_lastTime;

		if (l_cur > SCROLL_SPD) {
			m_currentOffset += l_cur / SCROLL_SPD;
			m_currentOffset %= (int)(text_size.x + spacer_size.x)*SCROLL_SPD;
			m_lastTime = thisTicks;
		}

		CGfx::ClearClipRegions();
		CGfx::SetClipRegion(actualPosition, m_size);
		CFont::GetActiveUiFont()->Print( m_text, actualPosition - V2(m_currentOffset/SCROLL_SPD, 0), m_color);
		CFont::GetActiveUiFont()->Print( m_text + SPACE, actualPosition - V2(m_currentOffset/SCROLL_SPD - text_size.x - spacer_size.x, 0), m_color);
		CGfx::DisableClipRegions();
	} else {
		// Draw text
		CGfx::ClearClipRegions();
		CGfx::SetClipRegion(actualPosition, m_size);
		if (m_settings & MASK_WRAP == WRAP) {
			CFont::GetActiveUiFont()->Print(CFont::GetActiveUiFont()->GetWrappedString(m_text, m_size.x), actualPosition, m_color);
		} else {
			CFont::GetActiveUiFont()->Print(m_text, actualPosition, m_color);
		}
		CGfx::DisableClipRegions();
	}
}

//******************************************************************************
//
//******************************************************************************
void	CControlLabel::ProcessControlInput()
{

}

//******************************************************************************
//
//******************************************************************************
CString CControlLabel::getText() {
	return m_text;
}

//******************************************************************************
//
//******************************************************************************
void CControlLabel::setText(CString text) {
	m_text = text;
	initScrollParameters();
}

//******************************************************************************
//
//******************************************************************************
void CControlLabel::setColor(ARGB color) {
	m_color = color;
}

//******************************************************************************
// 
//******************************************************************************
void CControlLabel::ProcessControl() {

}

//******************************************************************************
// 
//******************************************************************************
void CControlLabel::scroll(bool shouldScroll) {
	if (shouldScroll) {
		if (!m_scroll) {
			m_scroll = true;
			initScrollParameters();
		}
	} else {
		m_scroll = false;
	}
}

//******************************************************************************
// 
//******************************************************************************
void CControlLabel::initScrollParameters() {
	m_lastTime = CFrameWork::GetTicks();
	m_currentOffset = 0;
}
//*******************************  END OF FILE  ********************************
