/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CControlLabel.h

  Description :	

  Last Modified $Date: 2006/11/25 16:41:26 $

  $Revision: 1.2 $

*******************************************************************************/

#ifndef CControlLabel_H_
#define CControlLabel_H_

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CTypes.h"
#include "CWindow.h"
#include "CControl.h"
#include "CGfx.h"
#include "CString.h"

//******************************************************************************
//   Macros
//******************************************************************************

//******************************************************************************
//   Types
//******************************************************************************

//******************************************************************************
//   Constants
//******************************************************************************

//******************************************************************************
//   Class definitions
//******************************************************************************
class CControlLabel : public CControl
{

	public:

		CControlLabel(V2 const size, V2 const position, ARGB const color,  CString text, u32 settings = 0);

		virtual ~CControlLabel();
		
		virtual void RenderControl();
		virtual void ProcessControlInput();
		virtual void ProcessControl();
		CString getText();
		void setText(CString text);
		void setColor(ARGB color);			
		void scroll(bool shouldScroll);

	private:
		void initScrollParameters();

	private:

		V2 m_size;
		V2 m_position; // relative to top left corner
		ARGB m_color;
		CString m_text;
		u32 m_settings; // holds settings

		// scrolling vars
		bool m_scroll;
		unsigned int m_currentOffset;
		u32	m_lastTime;
		u64	m_tickResolution;

	public:

		static const u32 WRAP = 1;
		static const u32 LEFT_JUSTIFY = 0;
		static const u32 RIGHT_JUSTIFY = 2;
		static const u32 MIDDLE_JUSTIFY = 4;
		static const u32 TOP_ALIGN = 0; // 00 0000
		static const u32 MIDDLE_ALIGN = 16; // 01 0000
		static const u32 BOTTOM_ALIGN = 32; // 10 0000

		static const u32 MASK_WRAP = 1; // 00 0001
		static const u32 MASK_JUSTIFY = 6; // 00 0110
		static const u32 MASK_ALIGN = 48; // 11 0000

};


//******************************************************************************
//   Externs
//******************************************************************************

//******************************************************************************
//   Prototypes
//******************************************************************************

#endif /* CControlLabel_H_ */

