/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CControlProgressBar.cpp

  Description :	

  Last Modified $Date: 2007/01/07 17:08:28 $

  $Revision: 1.4 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CControlProgressBar.h"
#include "CTextureManager.h"
#include "CFont.h"
#include "CGfx.h"
#include "CSkinManager.h"
#include "CFrameWork.h"
#include "CControl.h"
#include "CControlLabel.h"

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************
static const float	WINDOW_BORDER( 1.f ); // Border around progress bar

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************

//******************************************************************************
//   Class Definition
//******************************************************************************

//******************************************************************************
//	
//******************************************************************************
CControlProgressBar::CControlProgressBar(V2 const size, V2 const position, u32 total, ARGB color, CString initText):
	m_size(size),
	m_position(position),
	m_currentPhase(0),
	m_totalPhases(total),
	m_color(color),
	m_initText(initText),
	m_pSelectedPhase(NULL),
	m_pPreviousPhase(NULL),
	m_pLabel(NULL)
{
	m_alpha = 255 / total;

	m_pLabel = new CControlLabel(	V2(m_size.x - WINDOW_BORDER * 2, m_size.y/* - WINDOW_BORDER * 2*/),
									V2(m_position.x + WINDOW_BORDER, m_position.y + WINDOW_BORDER),
									CSkinManager::GetColor( "progress_bar", "text_color", 0xFF000000),
									m_initText,
									CControlLabel::MIDDLE_JUSTIFY|CControlLabel::MIDDLE_ALIGN);
									
}

//******************************************************************************
//	
//******************************************************************************
CControlProgressBar::~CControlProgressBar()
{
	SAFE_DELETE(m_pPreviousPhase);
	SAFE_DELETE(m_pSelectedPhase);
}	

//******************************************************************************
//	
//******************************************************************************
void	CControlProgressBar::RenderControl()
{
	if(isHidden()) {
		return;
	}

	// Draw the border
	// Sort of cheating here, couldn't send bgcolor to none opaque, if we want that functionality
	// need to draw 4 seperate quads
	CGfx::DrawQuad( V2( m_position.x, m_position.y ),
					V2( m_size.x, m_size.y),
					CSkinManager::GetColor( "progress_bar", "border_color", 0xFFD4D4D4));
	
	// Draw menu background (overlay over the menu border quad)
	CGfx::DrawQuad( V2(m_position.x + WINDOW_BORDER, m_position.y + WINDOW_BORDER),
				    V2(m_size.x - WINDOW_BORDER * 2, m_size.y - WINDOW_BORDER * 2),
				    CSkinManager::GetColor( "progress_bar", "background_color", 0xFFFFFFFF));

	if (m_pSelectedPhase != NULL) {
		float division = float(m_pSelectedPhase->m_current)/m_pSelectedPhase->m_total * (m_size.x - WINDOW_BORDER * 2);

		// Draw current item
		CGfx::DrawQuad( V2(m_position.x + WINDOW_BORDER, m_position.y + WINDOW_BORDER),
						V2(division, m_size.y - WINDOW_BORDER * 2),
						ARGB(m_alpha*m_pSelectedPhase->m_phaseNumber, m_color.r, m_color.g, m_color.b));

		// Draw previous item colour in remaining
		if (m_pSelectedPhase->m_current != m_pSelectedPhase->m_total && m_pPreviousPhase != NULL) {
			CGfx::DrawQuad( V2(m_position.x + WINDOW_BORDER + division, m_position.y + WINDOW_BORDER),
							V2(m_size.x - WINDOW_BORDER*2 - division, m_size.y - WINDOW_BORDER * 2),
							ARGB(m_alpha*m_pPreviousPhase->m_phaseNumber, m_color.r, m_color.g, m_color.b));
		}
	}

	m_pLabel->RenderControl();
}

//******************************************************************************
//
//******************************************************************************
void CControlProgressBar::ProcessControlInput()
{

}

//******************************************************************************
// 
//******************************************************************************
void CControlProgressBar::ProcessControl() {

}

//******************************************************************************
// 
//******************************************************************************
void CControlProgressBar::startNextPhase(const CString & name, u32 count, ePhaseFormat format) {
	//Don't go past the total number of allowed phases
	if (m_currentPhase != m_totalPhases && count > 0) {

		u32 phaseNumber;

		// Update the current phase if there is one, and remember it in previous
		if (m_pSelectedPhase != NULL) {
			//Finish the current phase
			m_pSelectedPhase->m_current = m_pSelectedPhase->m_total;
			SAFE_DELETE(m_pPreviousPhase); // no longer need it
			m_pPreviousPhase = m_pSelectedPhase;
			phaseNumber = m_pPreviousPhase->m_phaseNumber + 1;
		} else {
			phaseNumber = 1;
		}

		//Create the new phase
		CControlProgressBarPhase* p_phase = new CControlProgressBarPhase(name, count, phaseNumber, format);
		m_currentPhase++;
		m_pSelectedPhase = p_phase;
	}
	updateLabel();
}

//******************************************************************************
// 
//******************************************************************************
void CControlProgressBar::update(u32 increment) {
	// increment but not past the total
	if (m_pSelectedPhase != NULL)
	{
		if (m_currentPhase > 0 && m_pSelectedPhase->m_current != m_pSelectedPhase->m_total) {
			if ((m_pSelectedPhase->m_current + increment) > m_pSelectedPhase->m_total) {
				m_pSelectedPhase->m_current = m_pSelectedPhase->m_total;
			} else {
				m_pSelectedPhase->m_current += increment;
			}
		}
		updateLabel();
	}
	
}

//******************************************************************************
// 
//******************************************************************************
void CControlProgressBar::updateLabel() {
	if (m_pSelectedPhase != NULL)
	{
		CString text("");

		switch (m_pSelectedPhase->m_format) {
		case FORMAT_NUMBER:
		{
			text.Printf("%s (%d of %d)", 
						(const char *)m_pSelectedPhase->m_message,
						m_pSelectedPhase->m_current,
						m_pSelectedPhase->m_total);
			break;
		}
		case FORMAT_PERCENT:
		default:
		{
			text.Printf("%s (%1.0f%%)", 
						(const char *)m_pSelectedPhase->m_message,
						float(m_pSelectedPhase->m_current)/m_pSelectedPhase->m_total*100);
			break;
		}
	}

	m_pLabel->setText(text);
	}
}

//******************************************************************************
// 
//******************************************************************************
void CControlProgressBar::reset() {
	SAFE_DELETE(m_pPreviousPhase);
	SAFE_DELETE(m_pSelectedPhase);
	m_currentPhase = 0;
	m_pLabel->setText(m_initText);
}

//*******************************  END OF FILE  ********************************
