/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CControlProgressBar.h

  Description :	

  Last Modified $Date: 2006/11/25 16:41:27 $

  $Revision: 1.2 $

*******************************************************************************/

#ifndef CControlProgressBar_H_
#define CControlProgressBar_H_

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CTypes.h"
#include "CWindow.h"
#include "CControl.h"
#include "CGfx.h"
#include "CString.h"
#include "CControlLabel.h"

//******************************************************************************
//   Macros
//******************************************************************************

//******************************************************************************
//   Types
//******************************************************************************
class CControlProgressBarPhase;

//******************************************************************************
//   Constants
//******************************************************************************

//******************************************************************************
//   Class definitions
//******************************************************************************
class CControlProgressBar : public CControl
{
	public:
		enum ePhaseFormat
		{
			FORMAT_PERCENT,
			FORMAT_NUMBER
		};

	public:
		typedef std::vector< CControlProgressBarPhase * >	ProgressBarPhaseList;

	public:

		CControlProgressBar(V2 const size, V2 const position, u32 total, ARGB color, CString initText = CString("Initializing"));

		virtual ~CControlProgressBar();

		virtual void RenderControl();
		virtual void ProcessControlInput();
		virtual void ProcessControl();

		// Create a new phase in the list and move to it
		void startNextPhase(const CString & name, u32 count, ePhaseFormat format);

		// Increment the current phase by the current amount (do not go past!)
		void update(u32 increment); 
		
		// Clears the control back to it's original constructed state
		void reset();

	private:
		// Updates the text of label to the current position in the phase
		void updateLabel();

	private:

		V2 m_size;
		V2 m_position;
		u32 m_currentPhase;
		u32 m_totalPhases;
		ARGB m_color;
		u8 m_alpha;
		CString m_initText;
		CControlProgressBarPhase* m_pSelectedPhase;
		CControlProgressBarPhase* m_pPreviousPhase;
		CControlLabel* m_pLabel;
};


class CControlProgressBarPhase
{
	public:
		CControlProgressBarPhase(const CString & message, u32 total, u32 phaseNum, CControlProgressBar::ePhaseFormat format) :
			m_message(message),
			m_current(0),
			m_total(total),
			m_phaseNumber(phaseNum),
			m_format(format)
		{
		
		}

	public:
		CString	m_message;
		u32	m_current;
		u32 m_total;
		u32 m_phaseNumber;
		CControlProgressBar::ePhaseFormat m_format;
};

//******************************************************************************
//   Externs
//******************************************************************************

//******************************************************************************
//   Prototypes
//******************************************************************************

#endif /* CControlTextBox_H_ */
