/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CDialogModal.h

  Description :	

*******************************************************************************/
#ifndef CDIALOGMODAL_H_
#define CDIALOGMODAL_H_

#include "CTypes.h"
#include "CString.h"
#include <vector>

class CControlButton;
class CControlManager;

#define MAX_MODAL_DLG_BUTTONS 3

enum EDialogButtonConfig
{
	DIALOG_CONFIG_OK_ONLY,
	DIALOG_CONFIG_OK_CANCEL,
	DIALOG_CONFIG_YES_NO,
	DIALOG_CONFIG_ACCEPT_DECLINE, // Includes a third "Accept Once"
	DIALOG_CONFIG_MAX
};

enum EDialogButtonValue
{
	DIALOG_BUTTON_OK,
	DIALOG_BUTTON_CANCEL,
	DIALOG_BUTTON_YES,
	DIALOG_BUTTON_NO,
	DIALOG_BUTTON_ACCEPT,
	DIALOG_BUTTON_DECLINE,
	DIALOG_BUTTON_ACCEPT_ONCE,
	DIALOG_BUTTON_MAX,
	DIALOG_BUTTON_INVALID = -1
};

class CDialogModal;
typedef std::vector< CDialogModal* > TModalList;

class CDialogModal
{
	public: /* Constructors & destructors */
	
		CDialogModal(const int aWidth, const V2 aDialogPosition, const CString aText, EDialogButtonConfig aButtonConfig);
		virtual ~CDialogModal();
		
	public: /* Static methods */
	
		static void Open();
		static void Close();
		static void Process();
		static void Render();
		static bool HasDialog();
		
	public: /* New methods */
	
		virtual void RenderDialog();
		virtual int getExitValue();
	
	private: /* These methods are only to be called by the dialog handling code */
	
		virtual void ProcessDialog();
		// Note that modal dialogs don't need to return a bool since only the
		// top most dialog will get input.
		virtual void ProcessDialogInput();
	
	private: /* Helper methods */
	
		float getMinWidth(EDialogButtonConfig aButtonConfig);
		int getButtonCount(EDialogButtonConfig aButtonConfig);
		void linkButtons();
		
	private: /* Static data */
	
		static TModalList s_modalList;
		
	private: /* Data */
	
		V2 m_size;
		V2 m_position;
		CString m_text;
		int m_exitValue;
		EDialogButtonConfig m_buttonConfig;
		CControlButton* m_pButtons[MAX_MODAL_DLG_BUTTONS];
		CControlManager* m_pControlManager;
		
};

#endif /*CDIALOGMODAL_H_*/
