/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) T Swann, Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CInput.h

  Description :	

  Last Modified $Date: 2007/02/05 01:28:40 $

  $Revision: 1.3 $

*******************************************************************************/

#ifndef CINPUT_H_
#define CINPUT_H_

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CTypes.h"
#include "CVector.h"
#include "CString.h"

//******************************************************************************
//   Macros
//******************************************************************************

//******************************************************************************
//   Types
//******************************************************************************
class CKeyboard;
class CKeyboardNav;

//******************************************************************************
//   Constants
//******************************************************************************

//******************************************************************************
//   Class definitions
//******************************************************************************
class CInput
{
	public:

		enum eButton
		{
			SELECT,
			START,
			UP,
			RIGHT,
			DOWN,
			LEFT,
			LTRIGGER,
			RTRIGGER,
			TRIANGLE,
			CIRCLE,
			CROSS,
			SQUARE,
			HOME,
			HOLD,
			NOTE,

			MAX_BUTTONS
		};

		static bool	Open();
		static void	Close();
		static void	Process();

	public: /* PSP button stuff */

		static bool	IsButtonDown( eButton button );
		static bool	IsButtonClicked( eButton button );
		static bool	IsButtonRepeat( eButton button, s32 repeat );
		static bool	IsButtonReleased( eButton button );
		static V2	GetAnalogStick();
		
	public: /* Keyboard stuff */
	
		static void SetKeyboardFromName(CString aKeyboardName, bool analogNav = false);
		static void RenderKeyboard();
		static void EnableKeyboard();
		static void DisableKeyboard();
		static void ResetKeyboard();
		static bool	IsKeyboardPressed();
		static bool IsKeyboardEnabled();
		static bool ShouldDisableKeyboard();
		static u16 GetKeyboardControlKey();
		static char GetKeyboardCharKey();

	public:
	
		static const s32 PAD_SLOW_REPEAT;
		static const s32 PAD_FAST_REPEAT;
		static const s32 MAX_REPEAT_COUNT;
		
	private:
		static CKeyboard* s_keyboard;
		static CKeyboardNav* s_keyboardNav;
};

//******************************************************************************
//   Externs
//******************************************************************************

//******************************************************************************
//   Prototypes
//******************************************************************************

#endif /* CINPUT_H_ */
