/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CMailRetriever.cpp

  Description :	

*******************************************************************************/

#include "CMailRetriever.h"
#include "CLog.h"

int retrieverThreadCB(SceSize args, void* argp)
{
	int err;
	
	if (args == 0)
		return -1;
		
	CMailRetriever* retriever(reinterpret_cast<CMailRetriever*>(argp));
	void* data = NULL;
	struct SRetrieverMsg* msg = NULL;
	
	while (1)
	{
		if ((err = sceKernelReceiveMbx(retriever->m_retriever_mbx, &data, NULL)))
		{
			CLog::Error("[Retriever thread]: sceKernelReceiveMbx returned %x\n", err);
		}
		else
		{
			msg = reinterpret_cast<SRetrieverMsg*>(data);
			if (msg->m_type == ERetrieverMsgType_Retrieve)
			{
				retriever->retrieve();
			}
			else if (msg->m_type == ERetrieverMsgType_CertReply)
			{
				if (retriever->m_certComm.processCertReply(msg))
				{
					// The certificate was accepted, let's restart the retrieve
					retriever->certAccepted();
				}
				else
				{
					// The certificate was declined, let's tell the retriever
					retriever->certDeclined();
				}
				
				SAFE_DELETE(msg);
			}
		}
	}
}

CMailRetriever::CMailRetriever(SceUID aMailboxMbx, SceUID aRetrieverMbx, vmime::ref<CCertificateManager> aCertManager) :
	m_mailbox_mbx(aMailboxMbx),
	m_retriever_mbx(aRetrieverMbx),
	m_certComm(aMailboxMbx, aRetrieverMbx),
	m_certManager(aCertManager)
{
}
