/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CMailSender.cpp

  Description :	

*******************************************************************************/

#include "CMailSender.h"
#include "CLog.h"

int senderThreadCB(SceSize args, void* argp)
{
	int err;
	
	if (args == 0)
		return -1;
		
	CMailSender* sender(reinterpret_cast<CMailSender*>(argp));
	void* data = NULL;
	struct SSenderMsg* msg = NULL;
	
	while (1)
	{
		if ((err = sceKernelReceiveMbx(sender->m_sender_mbx, &data, NULL)))
		{
			CLog::Error("[Sender thread]: sceKernelReceiveMbx returned %x\n", err);
		}
		else
		{
			msg = reinterpret_cast<SSenderMsg*>(data);
			if (msg->m_type == ESenderMsgType_Send)
			{
				sender->send(msg->m_msgList);
			}
			else if (msg->m_type == ESenderMsgType_CertReply)
			{
				if (sender->m_certComm.processCertReply(msg))
				{
					// The certificate was accepted, let's restart the send
					sender->certAccepted();
				}
				else
				{
					// The certificate was declined, let's tell the send
					sender->certDeclined();
				}
			}
			
			// Don't delete the m_msgList within! It will get deleted on the
			// return message (success or fail)
			SAFE_DELETE(msg);
		}
	}
}

CMailSender::CMailSender(SceUID aMailboxMbx, SceUID aSenderMbx, vmime::ref<CCertificateManager> aCertManager) :
	m_mailbox_mbx(aMailboxMbx),
	m_sender_mbx(aSenderMbx),
	m_certComm(aMailboxMbx, aSenderMbx),
	m_certManager(aCertManager)
{
}
