/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CMailSender.h

  Description :	

*******************************************************************************/

#ifndef CMAILSENDER_H_
#define CMAILSENDER_H_

#include "CMailbox.h"
#include "CCertComm.h"
#include "CCertificateManager.h"
#include <pspthreadman.h>

int senderThreadCB(SceSize args, void* argp);

enum ESenderMsgType
{
	ESenderMsgType_Send,
	ESenderMsgType_CertReply = CERT_MSG_TYPE
};

class SSenderMsg : public SCertMsg
{
	public:
		SSenderMsg(u32 aType) : SCertMsg(aType) {};
	
	public:
		CMailbox::TSendMsgList* m_msgList; // Used by Send
};

class CMailSender
{
	public: // Constructors & Destructors
	
		CMailSender(SceUID aMailboxMbx, SceUID aRetrieverMbx, vmime::ref<CCertificateManager> aCertManager);
		virtual ~CMailSender() {};
	
	public: // New methods
	
		virtual void send(CMailbox::TSendMsgList* msgList) = 0;
		virtual void certDeclined() = 0;
		virtual void certAccepted() = 0;
		
	public: // Data members
	
		// Mailboxes used to communicate with the CMailbox thread. These need to
		// be public so the thread sender the messages can listen on the
		// correct Mbx
		SceUID m_mailbox_mbx;
		SceUID m_sender_mbx;
		
		CCertComm m_certComm;
		vmime::ref<CCertificateManager> m_certManager;
};

#endif /*CMAILSENDER_H_*/
