/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CMainWindow.cpp

  Description :	

  Last Modified $Date: 2006/12/21 20:35:09 $

  $Revision: 1.6 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CMainWindow.h"
#include "CFrameWork.h"
#include "CTextureManager.h"
#include "CFont.h"
#include "CInput.h"
#include "CGfx.h"
#include "CWindow.h"
#include "CPSPoste.h"
#include "CSkinManager.h"
#include "CMailbox.h"
#include "CHUHelp.h"
#include "CWindow.h"
#include "CConnectionManager.h"
#include "CContextMenu.h"
#include "CKeyboardNavWrapper.h"
#include <psppower.h>
#include <time.h>
//#include <danzeff.h>

//******************************************************************************
//   Local Macros
//******************************************************************************
CMainWindow::CMainWindowItemList *   m_pMainWindowItemList;
CMainWindowItem *       m_pSelectedItem;

//******************************************************************************
//   Local Constants
//******************************************************************************

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************

//******************************************************************************
//   Class Definition
//******************************************************************************


//******************************************************************************
//******************************************************************************
//
//	Main Window implementation
//
//******************************************************************************
//******************************************************************************

//******************************************************************************
//	
//******************************************************************************
CMainWindow::CMainWindow(CMailbox * mailbox, CWindow::eWindow selectedWindow) :
	m_pControlManager(NULL),
	m_pControlHeader(NULL),
	m_pCContextMenu(NULL)
{
	SetFocus(true);
	CMainWindowItem *messages = new CMainWindowItem(V2(18,77), "Messages", CSkinManager::GetComponent( CSkinManager::SC_ICON_MESSAGES )->GetTexture(), CWindow::WINDOW_MESSAGES);
	CMainWindowItem *compose = new CMainWindowItem(V2(84,77), "Compose", CSkinManager::GetComponent( CSkinManager::SC_ICON_COMPOSE )->GetTexture(), CWindow::WINDOW_COMPOSE);
	CMainWindowItem *contacts = new CMainWindowItem(V2(150,77), "Contacts", CSkinManager::GetComponent( CSkinManager::SC_ICON_CONTACTS )->GetTexture(), CWindow::WINDOW_CONTACTS);
	CMainWindowItem *settings = new CMainWindowItem(V2(216,77), "Settings", CSkinManager::GetComponent( CSkinManager::SC_ICON_SETTINGS )->GetTexture(), CWindow::WINDOW_SETTINGS);
#ifdef PSPRINT_GAME
	CMainWindowItem *game = new CMainWindowItem(V2(282,77), "PSprint Game", CSkinManager::GetComponent( CSkinManager::SC_ICON_GAME )->GetTexture(), CWindow::WINDOW_GAME);
	CMainWindowItem *exit = new CMainWindowItem(V2(348,77), "Quit", CSkinManager::GetComponent( CSkinManager::SC_ICON_EXIT )->GetTexture(), CWindow::WINDOW_EXIT);
#else
	CMainWindowItem *exit = new CMainWindowItem(V2(282,77), "Quit", CSkinManager::GetComponent( CSkinManager::SC_ICON_EXIT )->GetTexture(), CWindow::WINDOW_EXIT);
#endif
	
#ifndef RELEASE
	CMainWindowItem *ben = new CMainWindowItem(V2(18,150), "Ben Test", CSkinManager::GetComponent( CSkinManager::SC_ICON_EXIT )->GetTexture(), CWindow::WINDOW_BEN_TEST);
	messages->p_Down = ben;
	ben->p_Up = messages;
#endif
	
	messages->p_Left = exit;
	messages->p_Right = compose;
	compose->p_Left = messages;
	compose->p_Right = contacts;
	contacts->p_Left = compose;
	contacts->p_Right = settings;
	settings->p_Left = contacts;
#ifdef PSPRINT_GAME
	settings->p_Right = game;
	game->p_Left = settings;
	game->p_Right = exit;
	exit->p_Left = game;
#else
	settings->p_Right = exit;
	exit->p_Left = settings;
#endif
	exit->p_Right = messages;

	CMainWindowItem stackWindowItem = *messages;

	switch(selectedWindow) {
		case CWindow::WINDOW_MESSAGES:
			m_pSelectedItem = messages;
			break;
		case CWindow::WINDOW_COMPOSE:
			m_pSelectedItem = compose;
			break;
		case CWindow::WINDOW_CONTACTS:
			m_pSelectedItem = contacts;
			break;
		case CWindow::WINDOW_SETTINGS:
			m_pSelectedItem = settings;
			break;
#ifdef PSPRINT_GAME
		case CWindow::WINDOW_GAME:
			m_pSelectedItem = game;
			break;
#endif
		case CWindow::WINDOW_EXIT:
			m_pSelectedItem = exit;
			break;
#ifndef RELEASE
		case CWindow::WINDOW_BEN_TEST:
			m_pSelectedItem = ben;
			break;
#endif
		default:
			m_pSelectedItem = messages;
			break;
	}

	m_pMainWindowItemList = new CMainWindowItemList();
	m_pMainWindowItemList->push_back(messages);
	m_pMainWindowItemList->push_back(compose);
	m_pMainWindowItemList->push_back(contacts);
	m_pMainWindowItemList->push_back(settings);
#ifdef PSPRINT_GAME
	m_pMainWindowItemList->push_back(game);
#endif
	m_pMainWindowItemList->push_back(exit);
#ifndef RELEASE
	m_pMainWindowItemList->push_back(ben);
#endif

	m_pMailbox = mailbox;

	m_pControlManager = new CControlManager();
	m_pControlHeader = new CControlHeader();
	m_pControlHeader->setConnection(CConnectionManager::GetPSPConnectionName(CConfigFile::s_pSettings->connection_selected));
	m_pControlHeader->setAccount(CConfigFile::s_pSettings->selectedAccount->display_name);
	m_pControlManager->addControl(m_pControlHeader);
}

//******************************************************************************
//	
//******************************************************************************
CMainWindow::~CMainWindow()
{
	// Iterate through and delete all items in the list
	CMainWindowItemList::iterator it = m_pMainWindowItemList->begin();
	while( it !=  m_pMainWindowItemList->end() ) {
		SAFE_DELETE((*it));
		it++;
	}

	// Delete the list
	SAFE_DELETE(m_pMainWindowItemList);
	
	SAFE_DELETE(m_pCContextMenu);
}

//******************************************************************************
//	
//******************************************************************************
void	CMainWindow::Render()
{
	// Draw white background
	CGfx::DrawQuad( V2( 0, 0 ), V2( CGfx::s_ScreenWidth, CGfx::s_ScreenHeight), CSkinManager::GetColor( "main_window", "background_color", 0x0FFFFFFF));

	// Draw Selected Icon Outline Box under the currently selected item
	const V2 iconBorder(2,2);
	CGfx::DrawQuad( m_pSelectedItem->m_Position - iconBorder, V2( 48, 48) + iconBorder*2, CSkinManager::GetColor( "main_window", "selected_icon_outline_color", 0xFF96CBFE));

	// Draw Selected Icon Box under the currently selected item
	CGfx::DrawQuad( m_pSelectedItem->m_Position, V2( 48, 48), CSkinManager::GetColor( "main_window", "selected_icon_color", 0xFF96CBFE));

	// Iterate through all of the items and print them
	CMainWindowItemList::iterator it = m_pMainWindowItemList->begin();

	while( it !=  m_pMainWindowItemList->end() ) {
		CGfx::DrawQuad( (*it)->p_Texture, (*it)->m_Position, V2(48,48),  0xffffffff );
		it++;
	}
	// Update Title and New Messages
	m_pControlHeader->setTitle(m_pSelectedItem->m_szName);
	m_pControlHeader->setNewMessages(m_pMailbox->getUnreadMessageCount());
	
	//Draw Header
	m_pControlManager->RenderControl();
	
	//static SceCtrlData pspctrl;
	//sceCtrlReadBufferPositive( &pspctrl, 1 );
	//sceGuStart( GU_DIRECT, CGfx::GetDrawList() );
	//danzeff_readInput(pspctrl);
	//sceGuFinish();
	//sceGuSync( 0, 0 );
	////danzeff_render();
}

//******************************************************************************
//	
//******************************************************************************
void	CMainWindow::Process()
{
	if (m_pCContextMenu == NULL)
	{
		CHUHelp::Clear();
		CHUHelp::SetButton(CHUHelp::BUTTON_UP, "Navigate");
		CHUHelp::SetButton(CHUHelp::BUTTON_DOWN, "Navigate");
		CHUHelp::SetButton(CHUHelp::BUTTON_LEFT, "Navigate");
		CHUHelp::SetButton(CHUHelp::BUTTON_RIGHT, "Navigate");
		CHUHelp::SetButton(CHUHelp::BUTTON_CROSS, "Launch application");
		CHUHelp::SetButton(CHUHelp::BUTTON_TRIANGLE, "Switch active account");
	}

	if (m_pCContextMenu != NULL) {
		if (m_pCContextMenu->getExitValue() != -1) {
			//Menu has return a value, process it see eMenuItems enum
			switch(m_pCContextMenu->getExitValue()) {
				case 0: /* "Hide Menu" was selected */
					break;
				default:
					CPSPoste::Get()->ActivateAccount(m_pCContextMenu->getExitValue()-1);
					m_pControlHeader->setAccount(CConfigFile::s_pSettings->selectedAccount->display_name);
					break;
			}

			//Remove the context menu
			SAFE_DELETE(m_pCContextMenu);
			//Return the focus to this window
			SetFocus(true);
		}
    }

	CWindow::Process();
	m_pControlManager->ProcessControl();
}

//******************************************************************************
//	
//******************************************************************************
bool	CMainWindow::ProcessInput()
{
	if (HasFocus())
	{
		if (CKeyboardNavWrapper::Up()) {
			if (m_pSelectedItem->p_Up != NULL) {
				m_pSelectedItem = m_pSelectedItem->p_Up;
			}
		}
	
		if (CKeyboardNavWrapper::Down()) {
	        if (m_pSelectedItem->p_Down != NULL) {
	                m_pSelectedItem = m_pSelectedItem->p_Down;
	        }
	    }
	
		if (CKeyboardNavWrapper::Left()) {
	        if (m_pSelectedItem->p_Left != NULL) {
	                m_pSelectedItem = m_pSelectedItem->p_Left;
	        }
	    }
	
		if (CKeyboardNavWrapper::Right()) {
	        if (m_pSelectedItem->p_Right != NULL) {
	                m_pSelectedItem = m_pSelectedItem->p_Right;
	        }
		}
	
		if (CKeyboardNavWrapper::ActivateControl()) {
			CPSPoste::s_NewWindow = m_pSelectedItem->m_Window;
		}
		
		if (CKeyboardNavWrapper::OpenContextMenu() && (CConfigFile::s_pSettings->accountList->size() > 1)) {
			if (CPSPoste::Get()->IsAccountSwitchSafe())
				createAccountContextMenu();
			else
				createBogusContextMenu();

			SetFocus(false);
		}
	}

	return false;
}

//******************************************************************************
//	
//******************************************************************************
void CMainWindow::createAccountContextMenu()
{
	u32 accountIdx = 1;
	CSettings::CAccountList::iterator it = CConfigFile::s_pSettings->accountList->begin();
	CContextMenu::CContextMenuItemList* itemList = new CContextMenu::CContextMenuItemList();
	itemList->push_back(new CContextMenuItem("Hide Menu", 0, true));
	
	while (it != CConfigFile::s_pSettings->accountList->end())
	{
		if (*it != CConfigFile::s_pSettings->selectedAccount)
		{
			itemList->push_back(new CContextMenuItem((*it)->display_name, accountIdx, false));
		}
		++it;
		++accountIdx;
	}

	/* accountIdx starts at 1, and then there's the active account */
	if (accountIdx > 2)
	{
		m_pCContextMenu = new CContextMenu(itemList, new V2(10, 10));
	}
}

//******************************************************************************
//	
//******************************************************************************
void CMainWindow::createBogusContextMenu()
{
	CContextMenu::CContextMenuItemList* itemList = new CContextMenu::CContextMenuItemList();
	itemList->push_back(new CContextMenuItem("Switching disabled during send/retrieve", 0, false));
	m_pCContextMenu = new CContextMenu(itemList, new V2(10, 10));
}

//*******************************  END OF FILE  ********************************
