/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CMainWindow.h

  Description :	

  Last Modified $Date: 2006/12/13 02:52:45 $

  $Revision: 1.3 $

*******************************************************************************/

#ifndef CMAINWINDOW_H_
#define CMAINWINDOW_H_

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CTypes.h"
#include "CWindow.h"
#include "CFileSystem.h"
#include "CString.h"
#include "CGfx.h"
#include "CMailbox.h"
#include "CControlManager.h"
#include "CConfigFile.h"
#include "CControlHeader.h"

//******************************************************************************
//   Macros
//******************************************************************************

//******************************************************************************
//   Types
//******************************************************************************
class CMainWindowItem;
class CContextMenu;

//******************************************************************************
//   Constants
//******************************************************************************


//******************************************************************************
//   Class definitions
//******************************************************************************
class CMainWindow : public CWindow
{
	public:

		typedef std::vector<CMainWindowItem*> CMainWindowItemList;
		
	public:

		CMainWindow(CMailbox * mailbox, CWindow::eWindow selectedWindow = CWindow::WINDOW_MESSAGES);
		~CMainWindow();

		virtual void Render();
		virtual void Process();
		virtual bool ProcessInput();
		void setMailbox(CMailbox* aMailbox) { m_pMailbox = aMailbox; };

	private:
	
		void createAccountContextMenu();
		void createBogusContextMenu();

	private:
	
		CMailbox* m_pMailbox;
		CControlManager* m_pControlManager;
		CControlHeader* m_pControlHeader;
		CContextMenu* m_pCContextMenu;

};

//******************************************************************************
// Main Window Item definition
//******************************************************************************
class CMainWindowItem
{
	public:

		CMainWindowItem() {
		
		}

		CMainWindowItem( const V2 & position, const CString & name, const CTexture * texture, const CWindow::eWindow window )
			:	m_Position( position ),
				m_szName( name ),
				p_Texture( texture ),
				m_Window( window ),
				p_Up( NULL ),
				p_Down( NULL ),
				p_Left( NULL ),
				p_Right( NULL )
		{

		}

	public:
		V2					m_Position;
		CString				m_szName;
		const CTexture * 	p_Texture;
		CWindow::eWindow	m_Window;
		CMainWindowItem *	p_Up;
		CMainWindowItem *	p_Down;
		CMainWindowItem *	p_Left;
		CMainWindowItem *	p_Right;
};

//******************************************************************************
//   Externs
//******************************************************************************

//******************************************************************************
//   Prototypes
//******************************************************************************

#endif /* CMAINWINDOW_H_ */
