/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CMessagesWindow.h

  Description :	

  Last Modified $Date: 2006/11/25 01:27:01 $

  $Revision: 1.3 $

*******************************************************************************/

#ifndef CMessagesWindow_H_
#define CMessagesWindow_H_

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CTypes.h"
#include "CWindow.h"
#include "CFileSystem.h"
#include "CString.h"
#include "CGfx.h"
#include "CContextMenu.h"
#include "CContextMenuItem.h"
#include "CMailbox.h"
#include "CMessageViewWindow.h"
#include "CControlManager.h"
#include "CControlLabel.h"
#include "CControlHeader.h"
#include "CControlProgressBar.h"
#include "CControlScrollBar.h"
#include "CDialogBoxInput.h"
#include "CComposeWindow.h"

//******************************************************************************
//   Macros
//******************************************************************************

//******************************************************************************
//   Types
//******************************************************************************

//******************************************************************************
//   Constants
//******************************************************************************
#define DISPLAYED_MSG_COUNT 10
#define DISPLAYED_COL_COUNT 3

//******************************************************************************
//   Class definitions
//******************************************************************************
class CMessagesWindow : public CWindow, public MMailboxEventListener
{
	
	public: // Inherited from CWindow
	
		virtual void	Render();
		virtual void	Process();
		virtual bool	ProcessInput();
		
	public: // Inherited from MMailboxEventListener
	
		void storingDone(CMailbox* aMailbox, u32 aTotal);
		
		// For the progress bar
		void retrievalStarted(CMailbox* aMailbox, u32 aTotal);
		void retrievedOne(CMailbox* aMailbox, u32 aIndex, u32 aTotal);
		void retrievedNone(CMailbox* aMailbox);
		void retrieveFailed(CMailbox* aMailbox, CString aErrorText);
		void storingStarted(CMailbox* aMailbox, u32 aTotal);
		void storedOne(CMailbox* aMailbox, u32 aIndex, u32 aTotal);
		void storeFailed(CMailbox* aMailbox, CString aErrorText);
	
	public: // New methods

		CMessagesWindow(CMailbox * mailbox);
		~CMessagesWindow();

	private: // New methods

		void OpenMessage();
		void MarkMessage();
		void DeleteMessage();
		void DeleteMarkedMessages();
		void composeReplyFromSelected();
		void composeReplyAllFromSelected();
		void composeForwardFromSelected();
		void RebuildDisplayedMessages();
		void BuildDisplayListFromScratch();
		void UpdateDisplayListText();
		void MessagesArrived();
		void SortChanged();
		void SortBySettings();
		void InitProgressBar();
		void ClearProgressBar();
		void UpdateScrollBar();
		
		void MoveUp();
		void MoveDown();
		void MovePageUp();
		void MovePageDown();

	private: // Sub windows
		CContextMenu* m_pCContextMenu;
		CComposeWindow* m_pComposeWindow;
		CMessageViewWindow* m_pMessageViewWindow;
		CDialogBoxInput* m_pErrorDlg;

	private: // Network
		CMailbox* m_pMailbox;

	private: // Controls
		CControlManager* m_pControlManager;
		CControlLabel* m_pMessageLabels[DISPLAYED_MSG_COUNT][DISPLAYED_COL_COUNT];
		CControlLabel* m_pEmptyListLabel;
		CControlHeader* m_pControlHeader;
		CControlProgressBar* m_pProgressBar;
		CControlScrollBar* m_pScrollBar;
		
	private:
		CMailbox::TMessageList* m_pDisplayedMessages;
		CMessageHeader* m_pSelectedMessage;
		int m_selectedMsgNum;
		CMessageHeader* m_pFirstMessage;
		int m_firstMsgNum;
		int m_totalMarkedCount;
	
		enum eMenuItems {
			HIDE = 0,
			OPEN,
			MARK,
			REPLY,
			REPLYALL,
			FORWARD,
			CLOSE,
			DELETE,
			DELETEMARKED,
			CHECKMAIL,
			SORT_DATE_ASC,
			SORT_DATE_DESC,
			SORT_SUBJECT_ASC,
			SORT_SUBJECT_DESC,
			SORT_FROM_ASC,
			SORT_FROM_DESC
		};

};

//******************************************************************************
//   Externs
//******************************************************************************

//******************************************************************************
//   Prototypes
//******************************************************************************

#endif /* CMessagesWindow_H_ */
