/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CPOP3Retriever.h

  Description :	

*******************************************************************************/

#ifndef CPOP3RETRIEVER_H_
#define CPOP3RETRIEVER_H_

#include "CMailRetriever.h"
#include "CString.h"
#include "CConfigFile.h"
#include "VMimeTypes.h"
#include <vmime/utility/progressListener.hpp>

class CMaildirStore;

class CPOP3Retriever : public CMailRetriever, public vmime::utility::progressListener
{
	public: // Constructors & Destructors
	
		CPOP3Retriever(CMaildirStore* aLocalStore, CAccount* aAccount,
		               SceUID aMailboxMbx, SceUID aRetrieverMbx, vmime::ref<CCertificateManager> aCertManager);
		virtual ~CPOP3Retriever();
		
	public: // Inherited from CMailRetriever
	
		virtual void retrieve();
		virtual void certDeclined();
		virtual void certAccepted();
		
	public: // Inherited from vmime::utility::progressListener
	
		const bool cancel() const;
		void start(const int predictedTotal);
		void progress(const int current, const int currentTotal);
		void stop(const int total);
		
	private: // New methods
		
		void connect();
		void disconnect();
		
		void open();
		void close();
		void failed(CString aErrorText);
		void disconnected();
		
	private: // Data members
	
		CAccount m_account;
		SceUID m_mailbox_mbx;
		SceUID m_retriever_mbx;
		
		VSessionRef m_sess_ref;
		VStoreRef m_store_ref;
		VFolderRef m_folder_ref;
		
		CMaildirStore* m_local_store;
		
		int m_last_retrieved;
};

#endif /*CPOP3RETRIEVER_H_*/
