/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) 71M, Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CPSPoste.cpp

  Description :	

  Last Modified $Date: 2006/12/19 02:29:57 $

  $Revision: 1.7 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CPSPoste.h"
#include "CFrameWork.h"
#include "CGfx.h"
#include "CWindow.h"
#include "CComposeWindow.h"
#include "CMessagesWindow.h"
#include "CSettingsWindow.h"
#include "CContactsWindow.h"
#include "CHUHelp.h"
#include "CTextureManager.h"
#include "CPSprintGame.h"
#include "CKeyboardNavWrapper.h"

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************
#define PSPOSTE_FADE_IN_RATE 12.f

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************
CPSPoste *	CPSPoste::s_pInstance( NULL );
CWindow::eWindow CPSPoste::s_CurrentWindow;
CWindow::eWindow CPSPoste::s_NewWindow;

//******************************************************************************
//   Class Definition
//******************************************************************************

//******************************************************************************
//	
//******************************************************************************
void	CPSPoste::Create()
{
	ASSERT( s_pInstance == NULL, "Instance already created" );

	s_pInstance = new CPSPoste();
}

//******************************************************************************
//	
//******************************************************************************
void	CPSPoste::Destroy()
{
	SAFE_DELETE( s_pInstance );
}

//******************************************************************************
//	
//******************************************************************************
CPSPoste *	CPSPoste::Get()
{
	ASSERT( s_pInstance != NULL, "Trying to access a NULL psp mail" );

	return s_pInstance;
}

//******************************************************************************
//	
//******************************************************************************
CPSPoste::CPSPoste() : m_pMainWindow(NULL),
	m_pComposeWindow(NULL),
	m_pContactsWindow(NULL),
	m_pSettingsWindow(NULL),
	m_pMessagesWindow(NULL),
	m_pPSprintGame(NULL),
#ifndef RELEASE
	m_pBenTestWindow(NULL),
#endif
	m_pMailbox(NULL),
	m_FadeInLevel(0),
	m_screenshotNum(0)
{
	//
	//	Create the main window
	//
	ResetMailbox();
	m_pMainWindow = new CMainWindow(m_pMailbox);

	s_CurrentWindow = CWindow::WINDOW_MAIN;
	s_NewWindow = CWindow::WINDOW_MAIN;

	CHUHelp::Create();
}

//******************************************************************************
//	
//******************************************************************************
CPSPoste::~CPSPoste()
{
	SAFE_DELETE( m_pMainWindow );
	SAFE_DELETE( m_pComposeWindow );
	SAFE_DELETE( m_pSettingsWindow );
	SAFE_DELETE( m_pMessagesWindow );
	SAFE_DELETE( m_pContactsWindow );
	SAFE_DELETE( m_pMailbox );
	SAFE_DELETE(m_pPSprintGame);

#ifndef RELEASE
	SAFE_DELETE( m_pBenTestWindow );
#endif 

	CHUHelp::Destroy();
}

//******************************************************************************
//	
//******************************************************************************
void	CPSPoste::Process()
{	
	//
	//	Fade the screen in
	//
	m_FadeInLevel += PSPOSTE_FADE_IN_RATE;

	if ( m_FadeInLevel > 255.f )
	{
		CGfx::SetFadeLevel( 255 );
	}
	else
	{
		CGfx::SetFadeLevel( static_cast< u8 >( m_FadeInLevel ) );
	}

	if (CKeyboardNavWrapper::HideHUH())
		CHUHelp::Show(false);
	else if (CKeyboardNavWrapper::ShowHUH())
		CHUHelp::Show(true);

	// Handle Windowing
	if (CPSPoste::s_CurrentWindow != CPSPoste::s_NewWindow) {

		// Remove existing window
		switch(CPSPoste::s_CurrentWindow) {
			case CWindow::WINDOW_MAIN:
				SAFE_DELETE(m_pMainWindow);
				break;
			case CWindow::WINDOW_COMPOSE:
				SAFE_DELETE(m_pComposeWindow);
				break;
			case CWindow::WINDOW_CONTACTS:
				SAFE_DELETE(m_pContactsWindow);
				break;
			case CWindow::WINDOW_MESSAGES:
				SAFE_DELETE(m_pMessagesWindow);
				break;
			case CWindow::WINDOW_SETTINGS:
				SAFE_DELETE(m_pSettingsWindow);
				break;
			case CWindow::WINDOW_GAME:
				SAFE_DELETE(m_pPSprintGame);
				break;
#ifndef RELEASE
			case CWindow::WINDOW_BEN_TEST:
				SAFE_DELETE(m_pBenTestWindow);
				break;
#endif 
			default:
				break;
		}

		// Draw new window
		switch(CPSPoste::s_NewWindow) {
			case CWindow::WINDOW_MAIN:
				m_pMainWindow = new CMainWindow(m_pMailbox, CPSPoste::s_CurrentWindow);
				break;
			case CWindow::WINDOW_COMPOSE:
				m_pComposeWindow = new CComposeWindow(m_pMailbox);
				break;
			case CWindow::WINDOW_CONTACTS:
				m_pContactsWindow = new CContactsWindow(m_pMailbox);
				break;
			case CWindow::WINDOW_MESSAGES:
				m_pMessagesWindow = new CMessagesWindow(m_pMailbox);
				break;
			case CWindow::WINDOW_SETTINGS:
				m_pSettingsWindow = new CSettingsWindow();
				break;
			case CWindow::WINDOW_GAME:
				m_pPSprintGame = new CPSprintGame();
				break;
			case CWindow::WINDOW_EXIT:
				CFrameWork::setExit();
				break;
#ifndef RELEASE
			case CWindow::WINDOW_BEN_TEST:
				m_pBenTestWindow = new CBenTestWindow();
				break;
#endif 
			default:
				break;
		}

		CPSPoste::s_CurrentWindow = CPSPoste::s_NewWindow;
	}
}

//******************************************************************************
//	
//******************************************************************************
bool	CPSPoste::ProcessInput()
{
	/* Screenshot code deprecated, use the psplink scrshot command instead.
	if (CInput::IsButtonClicked(CInput::START)) {
		if (CInput::GetScreenshot()) {
			CString tempText("");
			tempText.Printf("ms0:/PSP/GAME/PSPoste/screenshot_%d.png", CPSPoste::m_screenshotNum++);
			CTextureManager::screenshot((const char *)tempText);	
		}	
	}
	*/

	return false;
}

//******************************************************************************
//	
//******************************************************************************
void	CPSPoste::ResetMailbox()
{
	SAFE_DELETE(m_pMailbox);
	m_pMailbox = new CMailbox(CConfigFile::s_pSettings->selectedAccount);
	if (s_CurrentWindow == CWindow::WINDOW_MAIN && m_pMainWindow)
		m_pMainWindow->setMailbox(m_pMailbox);
}

//******************************************************************************
//	
//******************************************************************************
void CPSPoste::ActivateAccount(u32 aIdx)
{
	if (IsAccountSwitchSafe())
	{
		CConfigFile::s_pSettings->selectedAccount = CConfigFile::s_pSettings->accountList->at(aIdx);
		ResetMailbox();
	}
}

//******************************************************************************
//	
//******************************************************************************
bool CPSPoste::IsAccountSwitchSafe()
{
	return (!m_pMailbox->isRetrieving() && !m_pMailbox->isSending());
}

//******************************************************************************
//	
//******************************************************************************
void CPSPoste::RenameMailbox(CString oldName, CString newName)
{
	/* BEN TODO: Improve renaming, currently, a rename of the active account on
	 * settings save could reset the mailbox twice (once for save). */
	bool resetRequired = false;
	if (IsAccountSwitchSafe() || ((oldName != CConfigFile::s_pSettings->selectedAccount->display_name) && (resetRequired = true)))
	{
		CAccount* account = FindAccountByName(oldName);
		if (account)
		{
			account->display_name = newName;
			CString oldPath = "store/";
			oldPath = CFileSystem::MakeFullPath(oldPath + oldName);
			if (CFileSystem::DirectoryExists(oldPath))
			{
				CString newPath = "store/";
				newPath = CFileSystem::MakeFullPath(newPath + newName);
				CFileSystem::Rename(oldPath, newPath);
			}
			
			if (resetRequired)
			{
				ResetMailbox();
			}
		}
	}
}

//******************************************************************************
//	
//******************************************************************************
CAccount* CPSPoste::FindAccountByName(CString accountName)
{
	CSettings::CAccountList::iterator it = CConfigFile::s_pSettings->accountList->begin();
	while (it != CConfigFile::s_pSettings->accountList->end())
	{
		if ((*it)->display_name == accountName)
			return *it;
			
		++it;
	}
	
	return NULL;
}

//*******************************  END OF FILE  ********************************
