/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CPop3.cpp

  Description :	

*******************************************************************************/

#include "CPop3.h"

#include <vmime/vmime.hpp>

#include "CLog.h"

int CPop3::Test(std::vector<std::string>* displayMsgs)
{	
	try
	{
		// Create a new session
		vmime::utility::url url("pop3://solutionsatsource.com");
		vmime::ref <vmime::net::session> sess = vmime::create <vmime::net::session>();
		
		// Create an instance of the store
		vmime::ref <vmime::net::store> st = sess->getStore(url);
		
		sess->getProperties()[st->getInfos().getPropertyPrefix() + "auth.username"] = "ben";
		sess->getProperties()[st->getInfos().getPropertyPrefix() + "auth.password"] = "ben";
		
		// Connect it
		st->connect();
		
		// Get the Inbox and open it for retrieval
		CLog::Debug("fld = st->getDefaultFolder()\n");
		vmime::ref <vmime::net::folder> fld = st->getDefaultFolder();
		CLog::Debug("fld->open(vmime::net::folder::MODE_READ_ONLY)\n");
		fld->open(vmime::net::folder::MODE_READ_ONLY);
		
		// Retrieve the message "enveolopes"
		CLog::Debug("fld->getMessages()\n");
		std::vector < vmime::ref <vmime::net::message> > allMessages = fld->getMessages();
		CLog::Debug("fld->fetchMessages(allMessages, vmime::net::folder::FETCH_ENVELOPE)\n");
		fld->fetchMessages(allMessages, vmime::net::folder::FETCH_ENVELOPE);
		
		for (u32 i = 0; i < allMessages.size(); i++)
		{
			vmime::ref <vmime::net::message> msg = allMessages[i];
			
			CLog::Debug("hdr = msg->getHeader()\n");
			vmime::ref <const vmime::header> hdr = msg->getHeader();
			
			CLog::Debug("hdr->Date()->generate()\n");
			std::string aMsg = hdr->Date()->generate();
			aMsg += "\n";
			CLog::Debug("hdr->Subject()->generate()\n");
			aMsg += hdr->Subject()->generate();
			aMsg += "\n";
			CLog::Debug("hdr->From()->generate()\n");
			aMsg += hdr->From()->generate();
			aMsg += "\n\n";
			
			displayMsgs->push_back(aMsg);
		}
		
		// We have finished using the service
		CLog::Debug("st->disconnect()\n");
		st->disconnect();
		
		CLog::Debug("Done!\n");
		return 0;
	}
	catch (vmime::exception& e)
	{
		CLog::Error(e.what());
		return 1;
	}
	catch (std::exception& e)
	{
		CLog::Error(e.what());
		return 1;
	}
	
	return 0;
}
