/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CScrollableText.cpp

  Description :	

*******************************************************************************/

#include "CScrollableText.h"
#include "CGfx.h"
#include "CSkinManager.h"
#include "CFont.h"
#include <CTextAsciiRenderTable.h>

static const float  CURSOR_WIDTH( 1.f );

CScrollableText::CScrollableText(CText* aText, const V2 & size, const V2 & pos, ARGB aColor, bool aEditable) :
	m_text(aText),
	m_size(size),
	m_pos(pos),
	m_color(aColor),
	m_editable(aEditable)
{
	CFont* font = CFont::GetActiveTextFont();
	m_lineBuf = new CTextLineBuffer(font->GetRenderTable(), aText,
	                                (size_t)size.x,
	                                (size_t)((size.y - font->GetHeight()) / (font->GetHeight() + 1)) + 1,
	                                aEditable);
}

CScrollableText::~CScrollableText()
{
	SAFE_DELETE(m_lineBuf);
}

void CScrollableText::render(bool aShowCursor)
{
	const CTextLine* const lines = m_lineBuf->getLines();
	size_t lineCount = m_lineBuf->getLineCount();
	V2 pos(static_cast<float>(static_cast<s32>(m_pos.x)), static_cast<float>(static_cast<s32>(m_pos.y)));
	CFont* font = CFont::GetActiveTextFont();
	CTextPtr aPtr;
	u32 charCount;
	
	for (unsigned i = 1; i <= lineCount; i++)
	{
		aPtr = lines[i].m_head;
		charCount = lines[i].m_char_count;
		if (aPtr.valid())
		{
			if (*aPtr == ' ')
			{
				++aPtr;
				charCount -= 1;
			}
				
			font->Print(aPtr, charCount, pos, m_color);
		}
		pos.y += (font->GetHeight() + 1);
	}
	
	if (m_editable && aShowCursor)
		renderCursor();
	
	//CFont::GetDefaultFont()->Print(m_text, m_startLine, m_size, m_pos, m_color);
}

void CScrollableText::renderCursor()
{
	CFont* font = CFont::GetActiveTextFont();
	CGfx::DrawQuad( V2( m_lineBuf->getCursorOffset() + m_pos.x,
	                ((font->GetHeight() + 1) * (m_lineBuf->getCursorLineIdx() - 1)) - 1.f + m_pos.y),
			        V2( CURSOR_WIDTH, font->GetHeight()),
			        CSkinManager::GetColor( "text_box", "cursor_color",
			        0xFFFFFFFF));
}

void CScrollableText::down()
{
	if (m_editable)
		m_lineBuf->downCursor();
	else
		m_lineBuf->scrollDown();
}

void CScrollableText::up()
{
	if (m_editable)
		m_lineBuf->upCursor();
	else
		m_lineBuf->scrollUp();
}

void CScrollableText::dumpDebug()
{
	m_lineBuf->dumpLineInfo();
}
