/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CScrollingText.cpp

  Description :	

  Last Modified $Date: 2007/01/01 21:55:47 $

  $Revision: 1.2 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CScrollingText.h"
#include "CTextureManager.h"
#include "CFont.h"
#include "CGfx.h"
#include "CFrameWork.h"

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************
static const u64 SCROLL_SPEED = 120; //scroll every SCROLL_SPEED cpu ticks
static const CString SPACER("   "); // between the items

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************

//******************************************************************************
//   Class Definition
//******************************************************************************

//******************************************************************************
//	
//******************************************************************************
CScrollingText::CScrollingText(CString const text):
	m_text(text),
	m_currentOffset(0)
{
	m_tickResolution = sceRtcGetTickResolution();
	m_lastTime = CFrameWork::GetTicks();
}

//******************************************************************************
//	
//******************************************************************************
CScrollingText::~CScrollingText()
{

}	

//******************************************************************************
//	
//******************************************************************************
void	CScrollingText::Render(float width, V2 position, ARGB color, bool scroll)
{
	V2 text_size(CFont::GetActiveUiFont()->GetStringSize(m_text));
	V2 spacer_size(CFont::GetActiveUiFont()->GetStringSize(SPACER));


	if (text_size.x > width && scroll) {

		u64 thisTicks = CFrameWork::GetTicks();
		u64 l_cur = thisTicks - m_lastTime;

		if (l_cur > SCROLL_SPEED) {
			m_currentOffset += l_cur / SCROLL_SPEED;
			m_currentOffset %= (int)(text_size.x + spacer_size.x)*SCROLL_SPEED;
			m_lastTime = thisTicks;
		}

		CGfx::ClearClipRegions();
		CGfx::SetClipRegion( position, V2(width, text_size.y));
		CFont::GetActiveUiFont()->Print( m_text, position - V2(m_currentOffset/SCROLL_SPEED, 0), color );
		CFont::GetActiveUiFont()->Print( m_text + SPACER, position - V2(m_currentOffset/SCROLL_SPEED - text_size.x - spacer_size.x, 0), color );
		CGfx::DisableClipRegions();
		
	} else {
		CGfx::ClearClipRegions();
		CGfx::SetClipRegion( position, V2(width, text_size.y));
		CFont::GetActiveUiFont()->Print( m_text, position, color );
		CGfx::DisableClipRegions();
	}
}

//******************************************************************************
//	Only reset the counter if the text has been changed
//******************************************************************************
void	CScrollingText::updateText(CString text) {
	if (!text.Equals(m_text)) {
		m_text = text;
		m_lastTime = CFrameWork::GetTicks();
		m_currentOffset = 0;
	}
}
//*******************************  END OF FILE  ********************************
