/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Carlos Carrasco Martinez (carloscm at gmail dot com) - Bookr

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	PNGReader.cpp

  Description :	

  Last Modified $Date: 2007/01/03 05:31:14 $

  $Revision: 1.2 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CFrameWork.h"
#include "CTextureManager.h"
#include "CFileSystem.h"
#include "CFont.h"
#include <stdio.h>
#include <stdlib.h>
#include <pspkernel.h>
#include <pspdebug.h>
#include <pspdisplay.h>
#include <png.h>
#include <malloc.h> // For memalign


//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************

//******************************************************************************
//   Class Definition
//******************************************************************************

//******************************************************************************
//	
//******************************************************************************
CFont* CTextureManager::ReadTTF(const CString & filename, u32 size, CFont* destFont)
{
	// This initializes FreeType
	FT_Library library;
	if (FT_Init_FreeType(&library) != 0) {
		CLog::Error("Failed to initialize FreeType\n");
		return NULL;
	}

	// Load the font
	FT_Face face;
	if (FT_New_Face(library, filename, 0, &face) != 0) {
		CLog::Error("Failed to load font\n");
		return NULL;
	}

	return RenderTTF(library, face, size, destFont);
}

CFont* CTextureManager::RenderTTF(FT_Library& library, FT_Face& face, u32 fontSize, CFont* destFont)
{
	// Margins around characters to prevent them from 'bleeding' into
	// each other.
	int margin = 3;
	int image_height = 0, image_width = 256;

	// Abort if this is not a 'true type', scalable font.
	if (!(face->face_flags & FT_FACE_FLAG_SCALABLE) or !(face->face_flags & FT_FACE_FLAG_HORIZONTAL)) {
		CLog::Error("Unsupported font\n");
		return 0;
	}

	// Set the font size
	FT_Set_Pixel_Sizes(face, fontSize, 0);

	// First we go over all the characters to find the max descent
	// and ascent (space required above and below the base of a
	// line of text) and needed image size. There are simpler methods
	// to obtain these with FreeType but they are unreliable.
	int max_descent = 0, max_ascent = 0;
	int space_on_line = image_width - margin, lines = 1;

	int loadMode = FT_LOAD_DEFAULT;
	//if (autohint)
	//	loadMode |= FT_LOAD_FORCE_AUTOHINT;
	for (int i = 0; i < 256; ++i) {
		// Look up the character in the font file.
		int char_index = FT_Get_Char_Index(face, i);
		if (i < 32)
			char_index = 0;
	
		// Render the current glyph.
		FT_Load_Glyph(face, char_index, loadMode);
		FT_Render_Glyph(face->glyph, FT_RENDER_MODE_NORMAL);
	
		int advance = (face->glyph->metrics.horiAdvance >> 6) + margin;
		if (advance > space_on_line) {
			space_on_line = image_width - margin;
			++lines;
		}
		space_on_line -= advance;

		int t = face->glyph->bitmap_top;
		if (t > max_ascent)
			max_ascent = t;
		t = face->glyph->bitmap.rows - face->glyph->bitmap_top;
		if (t > max_descent)
			max_descent = t;
	}

	// Compute how high the texture has to be.
	int needed_image_height = (max_ascent + max_descent + margin) * lines + margin;
	// Get the first power of two in which it fits.
	image_height = 16;
	while (image_height < needed_image_height)
		image_height *= 2;

	// Allocate memory for the texture
	CFont* font(NULL);
	if (destFont != NULL)
		font = destFont;
	else
		font = new CFont(fontSize);
	
	font->Init(image_width, image_height);
	//u8* fontTexture = font->m_pBuffer;
	u32* fontTexture = (u32*)font->m_pBuffer;
	int fontLineHeight = max_ascent + max_descent + margin;

	// Allocate space for the GlyphEntries
	CFont::Metrics* fontMetrics = (CFont::Metrics*)memalign(16, 256 * sizeof(CFont::Metrics));
	memset(fontMetrics, 0, 256 * sizeof(CFont::Metrics));

	// These are the position at which to draw the next glyph
	int x = margin, y = margin + max_ascent;

	// Drawing loop
	for (int i = 0; i < 256; ++i) {
		int char_index = FT_Get_Char_Index(face, i);
		if (i < 32)
			char_index = 0;
		
		// Render the glyph
		FT_Load_Glyph(face, char_index, FT_LOAD_DEFAULT);
		FT_Render_Glyph(face->glyph, FT_RENDER_MODE_NORMAL);
		
		// See whether the character fits on the current line
		int advance = (face->glyph->metrics.horiAdvance >> 6) + margin;
		if (advance > image_width - x){
			x = margin;
			y += (max_ascent + max_descent + margin);
		}
		
		// Fill in the GlyphEntry
		fontMetrics[i].width = advance - 3;
		fontMetrics[i].xadvance = advance - 3;
		fontMetrics[i].xoffset = 0;
		fontMetrics[i].yoffset = 0;
		fontMetrics[i].height = max_ascent + max_descent;
		fontMetrics[i].x = x;
		fontMetrics[i].y = y - max_ascent;
		
		// Copy the image gotten from FreeType onto the texture
		// at the correct position
		for (int row = 0; row < face->glyph->bitmap.rows; ++row){
			for (int pixel = 0; pixel < face->glyph->bitmap.width; ++pixel){
				fontTexture[(x + face->glyph->bitmap_left + pixel) +
					(y - face->glyph->bitmap_top + row) * image_width] =
					((face->glyph->bitmap.buffer[pixel + row * face->glyph->bitmap.pitch] << 24) | 0xffffff);
			}
		}
		x += advance;    
	}

	FT_Done_FreeType(library);
	font->SetMetrics(fontMetrics, 256);
	font->SetHeight(fontLineHeight);
	font->Optimize();

	return font;
}

//*******************************  END OF FILE  ********************************
