/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CKeyboardGUI.cpp

  Description :	

  Last Modified $Date: 2007/01/01 21:55:47 $

  $Revision: 1.6 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CKeyboardGUI.h"
#include "CKeyboardNavWrapper.h"
#include "CInput.h"
#include "CGfx.h"
#include "CSkinManager.h"
#include "CFont.h"

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************
u16 CKeyboardGUI::s_lowerCharTable[GUI_KB_CHARS_HIGH][GUI_KB_CHARS_WIDE] =
{
	{ '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', },
	{ 'q', 'w', 'e', 'r', 't', 'y', 'u', 'i', 'o', 'p', },
	{ 'a', 's', 'd', 'f', 'g', 'h', 'j', 'k', 'l', 'E', },
	{ 'z', 'x', 'c', 'v', 'b', 'n', 'm', ',', '.', '/', },
	{ '@', '~', '-', '=', '[', ']', '\\', ';', '\'', ' ', },
};

u16 CKeyboardGUI::s_upperCharTable[GUI_KB_CHARS_HIGH][GUI_KB_CHARS_WIDE] =
{
	{ '!', '@', '#', '$', '%', '^', '&', '*', '(', ')', },
	{ 'Q', 'W', 'E', 'R', 'T', 'Y', 'U', 'I', 'O', 'P', },
	{ 'A', 'S', 'D', 'F', 'G', 'H', 'J', 'K', 'L', 'e', },
	{ 'Z', 'X', 'C', 'V', 'B', 'N', 'M', '<', '>', '?', },
	{ '@', '~', '_', '+', '{', '}', '|', ':', '"', ' ', },
};

//******************************************************************************
//   Class Definition
//******************************************************************************
bool CKeyboardGUI::doProcess(SceCtrlData *aPadData)
{
	// Clear the keys produced by the last call to Process
	m_charKey = 0;
	m_controlKey = 0;
	
	m_pspPadRepeat.process(aPadData);
	u32 repeatButtons = m_pspPadRepeat.getRepeatButtons();
	
	guiProcess();
	
	if (CInput::IsButtonDown(CInput::RTRIGGER))
		m_shift = true;
	else
		m_shift = false;
	
	if (CInput::IsButtonClicked(CInput::SELECT))
	{
		if (m_position.y != 2)
		{
			m_position.y = 2;
			m_selectPosition.y -= 198;
		}
		else
		{
			m_position.y = 200;
			m_selectPosition.y += 198;
		}
	}
	
	if (repeatButtons & PSP_CTRL_SQUARE)
		m_charKey = 32; // Space
	
	if (repeatButtons & PSP_CTRL_CIRCLE)
		m_controlKey = CKeyboard::BACKSPACE; // Backspace
		
	if (CKeyboardNavWrapper::DeactivateControl())
	{
		m_shouldDisable = true;
		return false;
	}
	
	return (m_charKey || m_controlKey);
}

//******************************************************************************
//
//******************************************************************************
void CKeyboardGUI::guiProcess()
{
	if (CInput::IsButtonClicked(CInput::TRIANGLE))	
		m_editMode = !m_editMode;
	
	if (m_editMode)
		editModeProcess();
	else
		cursorModeProcess();
}

//******************************************************************************
//
//******************************************************************************
void CKeyboardGUI::editModeProcess()
{
	u32 repeatButtons = m_pspPadRepeat.getRepeatButtons();
	
	if (repeatButtons & PSP_CTRL_UP)
	{
		selectUp();
	}
	
	if (repeatButtons & PSP_CTRL_DOWN)
	{
		selectDown();
	}
	
	if (repeatButtons & PSP_CTRL_LEFT)
	{
		selectLeft();
	}
	
	if (repeatButtons & PSP_CTRL_RIGHT)
	{
		selectRight();
	}
	
	if (repeatButtons & PSP_CTRL_CROSS)
	{
		select();
	}
}

//******************************************************************************
//
//******************************************************************************
void CKeyboardGUI::cursorModeProcess()
{
	u32 repeatButtons = m_pspPadRepeat.getRepeatButtons();
	
	if (repeatButtons & PSP_CTRL_UP)
	{
		m_controlKey = CKeyboard::UP;
	}
	else if (repeatButtons & PSP_CTRL_DOWN)
	{
		m_controlKey = CKeyboard::DOWN;
	}
	else if (repeatButtons & PSP_CTRL_LEFT)
	{
		m_controlKey = CKeyboard::LEFT;
	}
	else if (repeatButtons & PSP_CTRL_RIGHT)
	{
		m_controlKey = CKeyboard::RIGHT;
	}
}

//******************************************************************************
//
//******************************************************************************
bool CKeyboardGUI::doRender()
{
	const V2 boxBorder(2,2);
	// Draw Keyboard Box Border
	CGfx::DrawQuad(m_position - boxBorder, V2(100, 70) + boxBorder*2, CSkinManager::GetColor( "main_window", "selected_icon_color", 0xFF96CBFE));
	// Draw Keyboard Box
	CGfx::DrawQuad(m_position, V2(100, 70), CSkinManager::GetColor("main_window", "selected_icon_outline_color", 0xFF96CBFE));
	// Draw Selected Key Box
	CGfx::DrawQuad(m_selectPosition, V2(10, 14), CSkinManager::GetColor("main_window", "selected_icon_color", 0xFF96CBFE));
	for (int i = 0; i < GUI_KB_CHARS_HIGH; i++)
	{
	 	for (int j = 0; j < GUI_KB_CHARS_WIDE; j++)
	 	{
		 	CString key("");
		 	key.Printf("%c", (char)(getKeyAt(i,j) & 0xff));
	 	 	CFont::GetActiveUiFont()->Print( key, m_position + V2(j*10+1, i*14), CSkinManager::GetColor( "text_box", "text_color", 0xFFFFFFFF));
	 	}
	}
	
	return true;
}

//******************************************************************************
//
//******************************************************************************
void CKeyboardGUI::reset()
{
	m_position = V2(378, 200);
	m_selectPosition = V2(378, 200);
	m_selected = 0;
	m_shift = 0;
	m_editMode = true;
	m_pspPadRepeat.reset();
}

//******************************************************************************
//
//******************************************************************************
void CKeyboardGUI::selectUp()
{
	if (m_selected > 9)
	{
		m_selectPosition.y -= 14;
		m_selected -= 10;
	}
	else
	{
		m_selectPosition.y += 56;
		m_selected += 40;
	}
}

//******************************************************************************
//
//******************************************************************************
void CKeyboardGUI::selectUpNoWrap()
{
	if (m_selected > 9)
	{
		m_selectPosition.y -= 14;
		m_selected -= 10;
	}
}

//******************************************************************************
//
//******************************************************************************
void CKeyboardGUI::selectDown()
{
	if (m_selected < 40)
	{
		m_selectPosition.y += 14;
		m_selected += 10;
	}
	else
	{
		m_selectPosition.y -= 56;
		m_selected -= 40;
	}
}

//******************************************************************************
//
//******************************************************************************
void CKeyboardGUI::selectDownNoWrap()
{
	if (m_selected < 40)
	{
		m_selectPosition.y += 14;
		m_selected += 10;
	}
}

//******************************************************************************
//
//******************************************************************************
void CKeyboardGUI::selectLeft()
{
	if ((m_selected % 10) != 0)
	{  
		m_selectPosition.x -= 10;
		m_selected--;
	}
	else
	{
		m_selectPosition.x += 90;
		m_selected += 9;
	}
}

//******************************************************************************
//
//******************************************************************************
void CKeyboardGUI::selectLeftNoWrap()
{
	if ((m_selected % 10) != 0)
	{  
		m_selectPosition.x -= 10;
		m_selected--;
	}
}

//******************************************************************************
//
//******************************************************************************
void CKeyboardGUI::selectRight()
{
	if ((m_selected % 10) != 9)
	{
		m_selectPosition.x += 10;
		m_selected++;
	}
	else
	{
		m_selectPosition.x -= 90;
		m_selected -= 9; 	
	}
}

//******************************************************************************
//
//******************************************************************************
void CKeyboardGUI::selectRightNoWrap()
{
	if ((m_selected % 10) != 9)
	{
		m_selectPosition.x += 10;
		m_selected++;
	}
}

//******************************************************************************
//
//******************************************************************************
void CKeyboardGUI::select()
{
 	m_charKey = getSelectedKey();
	if (m_selected == 29) // Enter Key
 	{
 	 	m_controlKey = CKeyboard::ENTER;
 	 	m_charKey = 0;
 	}
}
