/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CKeyboardIrda.h

  Description :	

  Last Modified $Date: 2006/11/22 02:37:40 $

  $Revision: 1.3 $

*******************************************************************************/

#ifndef CKEYBOARDIRDA_H_
#define CKEYBOARDIRDA_H_

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CKeyboard.h"
#include "irda_Keyboard.h"

//******************************************************************************
//   Macros
//******************************************************************************

//******************************************************************************
//   Types
//******************************************************************************

//******************************************************************************
//   Constants
//******************************************************************************
// One IrDa keyboard packet is comprised of 6 bytes, and a typical up down
// keypress consists of 3 packets, so each time the 
#define IRDA_PACKET_SIZE 6

//******************************************************************************
//   Class definitions
//******************************************************************************
/* The CKeyboard class defines the interface PSPoste is to use when wanting to
 * poll for key presses when in text entry mode. See the CKeyboardNav class for
 * the class responsible for navigating the interface.
 */
class CKeyboardIrda : public CKeyboard
{
	public: /* Constructors & Destructors */
	
		CKeyboardIrda() : CKeyboard(),
			m_bufLeft(IRDA_PACKET_SIZE),
			m_syncRequired(false),
			m_allowHUH(true)
		{
			m_inputEnabled = true; // We always want to process the irda keyboard
			m_irdafd = sceIoOpen("irda0:", PSP_O_RDWR, 0);
			InitIrdaKeyboard();
		};
		
		~CKeyboardIrda()
		{
			sceIoClose(m_irdafd);
		}
	
	protected: /* Overriding methods */
	
		virtual void enableInput() { m_allowHUH = false; };
		virtual void disableInput() { m_allowHUH = true; m_shouldDisable = false; };
		virtual void enableRender() {};
		virtual void disableRender() {};
		virtual bool doProcess(SceCtrlData *aPadData);
		virtual bool allowHUH() { return m_allowHUH; };
		
	private: /* Helper methods */
	
		void processCode(int code);
		
	private: /* Control data */
		
		unsigned char m_irdaPacket[IRDA_PACKET_SIZE];
		int m_bufLeft;
		int m_irdafd;
		bool m_syncRequired;
		bool m_allowHUH;
};

//******************************************************************************
//   Externs
//******************************************************************************

//******************************************************************************
//   Prototypes
//******************************************************************************

#endif /* CKEYBOARDIRDA_H_ */
