/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	main_psplink.c

  Description :	Main file for psplink mode targetting the 1.5 firmware.

  Last Modified $Date: 2007/02/04 21:55:40 $

  $Revision: 1.3 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include <pspthreadman.h>
#include "main_globals.h"
#include "CFrameWork.h"
#include "CConnectionManager.h"
#include "CLog.h"

PSP_MODULE_INFO(PSPoste, 0, 1, 1 );
PSP_MAIN_THREAD_ATTR( THREAD_ATTR_USER );

PSP_HEAP_SIZE_KB(PSPOSTE_HEAP_SIZE_MB * 1024);

//******************************************************************************
// 
//******************************************************************************
int main(int argc, char* argv[])
{
	int	(*user_func)(SceSize argc, void* argp) = user_main; // Function pointer
	SceKernelThreadInfo thinfo; // Used to get the user thread exit status
	thinfo.size = sizeof(SceKernelThreadInfo);
	SceUID thid; // Thread id of user thread
	SceUID connManThid; // Thread id of connection manager
	
	// Initialise the debug output so we can catch any early DebugScreenPrintfs
	pspDebugScreenInit();

	// Open the log file now to catch any early log messages
	CLog::Open();
	
	// Don't setup stdout and stderr handlers! Will fail to start the app in
	// PSPLink
	
	// We drop the MSB of the function address to ensure user mode addressing
	user_func = (int (*)(SceSize, void*))((u32)user_func & 0x7fffffff);
	
	// Start the connection manager thread
	connManThid = sceKernelCreateThread("Conn Man Thread",
	                                    CConnectionManager::ConnManThreadCB,
	                                    0x11, CONN_MAN_THREAD_STACK_SIZE,
	                                    PSP_THREAD_ATTR_USER, NULL);

	if (connManThid & 0x80000000)
	{
		CLog::Error("Failed to create the connection manager thread: 0x%x\n",
		            connManThid);
	}
	else
	{
		sceKernelStartThread(connManThid, 0, NULL);
	}
	
	// This path is used to set the working directory in the user thread, it's
	// also used in this function to reboot the app
	strncpy((char*)&s_ebootPath, argv[0], 255);
	
	// 0x11 is the priority, 0x40000 is the stack size and corresponds to the
	// main thread stack size as determined from the crt0.c _start function
	// Note that 0x2D000 is the max free memory with PSPLink running
	thid = sceKernelCreateThread("User Mode Thread", user_func, 0x11,
	                             USER_THREAD_STACK_SIZE, PSP_THREAD_ATTR_USER,
	                             NULL);

	if (thid & 0x80000000)
	{
		CLog::Error("Failed to create the user thread: 0x%x\n", thid);
	}
	else
	{
		// Start thread with no parameters
		sceKernelStartThread(thid, 0, NULL);
	}
	
	// Wait for the user thread to terminate
	sceKernelWaitThreadEnd(thid, NULL);
	
	// Kill the conn man then clean up
	CConnectionManager::RequestKill();
	sceKernelWaitThreadEnd(connManThid, NULL);
	
	// Now that the user thread has resumed, this thread may resume, let's check
	// the user thread return status
	sceKernelReferThreadStatus(thid, &thinfo);
	
	// If the user thread exited with the REBOOT_RETURN_CODE, let's reboot the
	// app. Else, exit the game. Note that to prevent exiting the game on
	// failure (to look at the bluescreen for instance), should check for exit
	// code != 0, in that case just call sceKernelSleepThread
	if (thinfo.exitStatus == REBOOT_RETURN_CODE)
		CFrameWork::RunExecutable(s_ebootPath);
	else
		sceKernelExitGame();
	
	return 0;
}
