/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	user_main.c

  Description :	

  Last Modified $Date: 2007/02/10 03:11:30 $

  $Revision: 1.2 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "main_globals.h"
#include "CFrameWork.h"
#include "CPSPoste.h"
#include "CFileSystem.h"
#include "psp_irkeyb.h"

//******************************************************************************
//   Global Variables
//******************************************************************************
// The default main thread stack size is 256 KB, which is excessive since our
// main thread basically dispatches other threads to do the work
unsigned int sce_newlib_stack_kb_size = 4;
char s_ebootPath[256];

//******************************************************************************
// 
//******************************************************************************
int	user_main(SceSize argc, void* argp)
{
	/* This code must run in user mode. */
	int ret;
	
# ifdef USE_PSP_IRKEYB
  if (psp_irkeyb_init()) {
    fprintf(stdout, "failed to initialize IR keyboard !\n");
    return 0;
  }
# endif

	// Set the working folder
	CString	drive, dir;
	CFileSystem::SplitPath( CString( s_ebootPath ), &drive, &dir, NULL, NULL );
	drive += dir;
	CFileSystem::SetRoot( drive );
	
	// Open the framework
	if ( CFrameWork::Open() == true )
	{
		// Create the application
		CPSPoste::Create();

		// The main loop
		while ((ret = CFrameWork::Process()) == 0)
		{
		}
	}
	else
	{
		CLog::Close();
		ASSERT(0, "CFrameWork::Open()");
	}
# ifdef USE_PSP_IRKEYB
  psp_irkeyb_exit();
# endif
		

	return ret;
}
